%csbinit;

options mlogic mprint;

%global model_rk  abt_dates 

		param_run_status_cd

		param_run_status_msg_key;



%let job_rc = 0;

%let trans_rc = 0;

%let sqlrc = 0;


data _NULL_;

	name=put(time(),mmss8.1);

	name1=kscan(name,1,":.");

	name2=kscan(name,2,":.");

	name3=kscan(name,3,":.");

	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));

	call symputx('num_timestamp',name4);

	call symputx('log_name',"&LOG_PATH_CS"||"csbmva_backtesting_process"|| ktrim(kleft(name4))||".log");

run;

proc printto log="&log_name";
run;




%macro csbmva_backtest;



%let abt_dates = %quote(&abt_dates);



*==============================================================;

* declaring local macro variable ;

*==============================================================;



%local dt_var_len 		abt_dts        abt_dt_cnt  

	   dev_data       model_type     m_tgt_prd   tgt_time_sk  

		m_tgt_freq_cd  fail_cnt     m_abt_dt 	fm_data

		m_abt_dttm       m_tgt_dttm     scr_time_sk        

		m_model_sk       i			       max_tgt_dttm 	
		debug_lib    tot_no_of_rows_in_abt;

%let isDevelopmentCycle  = true;

/** Assigning the debug library to be used for processing**/

%let debug_lib=usrdebug;

*==============================================================;

* Converting abt_dates in desired format  ;

*==============================================================;



%let dt_var_len = %length(&abt_dates);



data _null_;

  length abt_dts $&dt_var_len;



  x = "&abt_dates";

  cnt = count(x,',') + 1;



  do i = 1 to cnt;

    a = scan(x,i,',');

    b = input(a,mmddyy10.);

    if i = 1 then

      abt_dts = compress(put(b,8.));

    else

      abt_dts = compress(abt_dts) || ',' || compress(put(b,8.));

  end;



  call symput('abt_dts',abt_dts);

  call symput('abt_dt_cnt',cnt);

run;

%bankfdn_rcset(&syserr);



%let abt_dts = &abt_dts;

%let abt_dts = %quote(&abt_dts);



*==============================================================;

* Pre-requisite validations ;

*==============================================================;


/* Checking whether foundation mart data is loaded for the model and requested time point */

%do p=1 %to &abt_dt_cnt;
   %let abt_dt_value = %scan(&abt_dts,&p,%str(','));
	proc sql noprint;
		select 1 into: fm_data
		from control.Foundation_mart_load_dates
		where LOAD_DT eq (&abt_dt_value)
		and LOAD_STATUS=&CHECK_FLAG_TRUE;
	quit;

	%if &fm_data  ne 1 %then %do;

  		%let param_run_status_cd = -1;

  		%let param_run_status_msg_key = backtest.param_run_status_cd.1;

  		%goto status_update;

	%end;

%end;


/* Checking whether Development data is loaded for the model */


%if %sysfunc(exist(csbfact._&model_rk._measure_fact)) eq 1 %then %do;

  proc sql noprint;

    select 1 into :dev_data

	from csbfact._&model_rk._measure_fact

	where dev_cnt = 1;

  quit;

  %bankfdn_rcset(&sqlrc);

%end;



%if &dev_data ne 1 %then %do;

  %let param_run_status_cd = -2;

  %let param_run_status_msg_key = backtest.param_run_status_cd.2;

  %goto status_update;

%end;



*==============================================================;

* Finding model type & target time ;

*==============================================================;



proc sql noprint;

  select model_type_cd, target_period_cnt, target_period_time_freq_cd into

     :model_type, :m_tgt_prd, :m_tgt_freq_cd

  from &dabt_lib..model_master_extension

  where model_key  = &model_rk;

quit;

%bankfdn_rcset(&sqlrc);

%let model_type = &model_type;
%let model_type_wq = &model_type;

%let model_type = %str(%')&model_type%str(%');

%let m_tgt_freq_cd = &m_tgt_freq_cd;



*==============================================================;

* Validating whether abt dates map to valid time sk or not ;

*==============================================================;

%let target_prd_adj_cnt = 0;
%if &model_type eq &LGD_MODEL %then %do;
	%let target_prd_adj_cnt = &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR;
%end;



data tgt_dates(keep = tgt_dttm);

  abt_dt_cnt = &abt_dt_cnt;

  abt_dts = "&abt_dts";

  tgt_freq = "&m_tgt_freq_cd";

  tgt_prd = &m_tgt_prd;



  do i = 1 to abt_dt_cnt;

    abt_dt = scan(abt_dts,i,',');

	/* tgt_dt = intnx(tgt_freq,abt_dt,tgt_prd); Original code */ 
	/* HF CSB 53 */ tgt_dt = intnx(tgt_freq,abt_dt, tgt_prd + &target_prd_adj_cnt , 'end');
    
    tgt_dttm = dhms(tgt_dt,23,59,59);

    output;

  end;



run;

%bankfdn_rcset(&syserr);



proc sort data = tgt_dates;

  by descending tgt_dttm ;

run;



data _null_;

  set tgt_dates(obs = 1);

  call symput('max_tgt_dttm',tgt_dttm);

run;

%bankfdn_rcset(&syserr);



%let max_tgt_sk = .;

proc sql noprint;

  select time_sk into :max_tgt_sk

  from dim.time_dim

  where PERIOD_FIRST_DTTM <= &max_tgt_dttm <= PERIOD_LAST_DTTM;

quit;

%bankfdn_rcset(&sqlrc);



%if &max_tgt_sk eq . or &max_tgt_sk eq  %then %do;

  %let param_run_status_cd = -6;

  %let param_run_status_msg_key = backtest.param_run_status_cd.6;

  %goto status_update;

%end;



*==============================================================;

*  Creating ABT for backtesting and checking status ;

*==============================================================;
%let mdl_abt_name=_&model_rk._mdl_bck;

%let scr_abt_name=_&model_rk._scr_bck;


%csbmva_backtest_abt(model_rk   = &model_rk,

                     abt_dt_lst = &abt_dts,

                     abt_dt_cnt = &abt_dt_cnt,

                     mdl_abt_name=&mdl_abt_name,

                     scr_abt_name=&scr_abt_name);



proc sql noprint;

  select count(*) into :fail_cnt

  from &csb_apdm_libref..build_backtest_abt_status

  where model_rk = &model_rk

  and   compress(status_cd) ne ('CMPT');

quit;


%bankfdn_rcset(&sqlrc);

%let fail_cnt=&fail_cnt;



%if %eval(&fail_cnt) > 0 or &syscc > 4 %then %do;

  %let param_run_status_cd = -3;

  %let param_run_status_msg_key = backtest.param_run_status_cd.3;

  %goto status_update;

%end;

/*Abort if abt does not contain any data*/

%let tot_no_of_rows_in_abt=0;
%let dsid=%sysfunc(open(usrdebug.&scr_abt_name));
%let tot_no_of_rows_in_abt=(%eval(%sysfunc(attrn(&dsid,nobs))));
%let rc=%sysfunc(close(&dsid));

%if %eval(&tot_no_of_rows_in_abt) le 0 %then %do;

  %let param_run_status_cd = -9;

  %let param_run_status_msg_key = backtest.param_run_status_cd.9;

  %goto status_update;

%end;

/*Generate audit report if configured*/
%csbmva_audit_report_wrapper(model_rk=&model_rk,called_by_process=BACKTESTING,pool_scheme_sk=,abt_dates=%str(&abt_dts));

/*Abort if audit report creation failed*/
%if &syscc > 4 or &syserr > 4 %then %do;

  %let param_run_status_cd = -8;

  %let param_run_status_msg_key = backtest.param_run_status_cd.8;

  %goto status_update;

%end;

ods listing;

*==============================================================;

* Check if oot_measure_fact already exists for this model;

*==============================================================;
%do i=1 %to &abt_dt_cnt;
 %let pdm_abt_dt = %scan(&abt_dts,&i,%str(,));
data _null_;
    dt = &pdm_abt_dt;
	ch_dt = put(dt,date9.);
	dttm = dhms(dt,23,59,59);    
	call symput('m_abt_dttm',dttm);
	
run;
%bankfdn_rcset(&syserr);

proc sql noprint;
  select time_sk into :scr_time_sk
  from dim.time_dim
  where PERIOD_FIRST_DTTM <= &m_abt_dttm <= PERIOD_LAST_DTTM;
quit;

%let scr_time_sk = &scr_time_sk;

	%if &model_type eq &PD_APP_MODEL or &model_type eq &PD_ONG_MODEL %then %let suf=PD;
														
   	%else %let suf=&model_type_wq ;

/* Delete older feed and fact datasets from temporary location */

proc datasets lib = usrdebug nodetails;

  delete _&model_rk._&scr_time_sk._oot_&suf._fact 
  	 _&model_rk._&scr_time_sk._oot_&suf._scr_fd;

quit;

%end;

/* Delete older oot_measure_fact from temporary location */

proc datasets lib = usrdebug nodetails;

  delete _&model_rk._oot_measure_fact;

quit;


/* Copying oot_measure_fact to temporary location */

%if %sysfunc(exist(backtest._&model_rk._oot_measure_fact)) eq 1 %then %do;

  proc copy in = backtest out = usrdebug  constraint=yes index=yes;

    select _&model_rk._oot_measure_fact;

  run;

%end;



%do i = 1 %to &abt_dt_cnt;

  

  %let m_abt_dt = %scan(&abt_dts,&i,%str(,));



  /* Creating PD/LGD/CCF fact */

  %csbmva_pdlgdccf_backtest_fact(model_rk   = &model_rk,

								 abt_dt     = &m_abt_dt,

                                 model_type = &model_type,

                                 scr_ds_name=&scr_abt_name,

								 scr_ds_lib=usrdebug,

                                 out_ds_lib=usrdebug);



  /* Creating FEED dataset */

  %csbmva_backtest_feed(model_rk   = &model_rk,

						abt_dt     = &m_abt_dt,

                        model_type = &model_type);



  /* Creating Model Measure Fact */

  %csbmva_backtest_measure_fact(model_rk     = &model_rk,

								abt_dt       = &m_abt_dt,

								model_type   = &model_type,

                                temp_out_lib = usrdebug);



  %if &job_rc > 4 or &syscc > 4 %then %do;

    %let param_run_status_cd = -5;

    %let param_run_status_msg_key = backtest.param_run_status_cd.5;

    %goto status_update;

  %end;



%end;




%do i=1 %to &abt_dt_cnt;
 %let m_abt_dt = %scan(&abt_dts,&i,%str(,));
data _null_;
    dt = &m_abt_dt;
	ch_dt = put(dt,date9.);
	dttm = dhms(dt,23,59,59);    
	call symput('m_abt_dttm',dttm);
	
run;
%bankfdn_rcset(&syserr);

proc sql noprint;
  select time_sk into :scr_time_sk
  from dim.time_dim
  where PERIOD_FIRST_DTTM <= &m_abt_dttm <= PERIOD_LAST_DTTM;
quit;

%let scr_time_sk = &scr_time_sk;

	%if &model_type eq &PD_APP_MODEL or &model_type eq &PD_ONG_MODEL %then %let suf=PD;
														
   	%else %let suf=&model_type_wq ;

/* Copying feed and fact to permanent location - loading was successful */

proc copy in = usrdebug out = backtest  constraint=yes index=yes;

  select _&model_rk._&scr_time_sk._oot_&suf._fact 
  	 _&model_rk._&scr_time_sk._oot_&suf._scr_fd;

quit;
%end;

/* Copying oot_measure_fact to permanent location - loading was successful */

proc copy in = usrdebug out = backtest  constraint=yes index=yes;

  select _&model_rk._oot_measure_fact;

run;


*==============================================================;

* Inserting rows in Model_backtest_summary table ;

*==============================================================;


%do i = 1 %to &abt_dt_cnt;

  

  %let m_abt_dt = %scan(&abt_dts,&i,%str(,));



  /* START - Calculate score_time_sk & target_time_sk */ 



  data _null_;

    dt = &m_abt_dt;

	dttm = dhms(dt,23,59,59);

    call symput('m_abt_dttm',dttm);


    tgt_dt = intnx("&m_tgt_freq_cd",&m_abt_dt,&m_tgt_prd + &target_prd_adj_cnt , 'end');

    tgt_dttm = dhms(tgt_dt,23,59,59);

    call symput('m_tgt_dttm',tgt_dttm);

  run;

  %bankfdn_rcset(&syserr);



  %let scr_time_sk = .;

  proc sql noprint;

    select time_sk into :scr_time_sk

    from dim.time_dim

    where PERIOD_FIRST_DTTM <= &m_abt_dttm <= PERIOD_LAST_DTTM;

  quit;

  %bankfdn_rcset(&sqlrc);



  %let tgt_time_sk = .;

  proc sql noprint;

    select time_sk into :tgt_time_sk

    from dim.time_dim

    where PERIOD_FIRST_DTTM <= &m_tgt_dttm <= PERIOD_LAST_DTTM;

  quit;

  %bankfdn_rcset(&sqlrc);



  %let scr_time_sk = &scr_time_sk;

  %let tgt_time_sk = &tgt_time_sk;



  /* END - Calculate score_time_sk & target_time_sk */ 

/* Finding version_no of development statistics for whcih backtesting is performed */ 

 proc sql;
   select max(version_no) into : dev_version_no
   from backtest._&model_rk._oot_measure_fact
   where dev_cnt=1;
 quit;
  %bankfdn_rcset(&sqlrc);

 %let dev_version_no=&dev_version_no;


/*Deleting rows for the same time point and model*/

proc sql;
  delete  from backtest.MODEL_BACKTEST_SUMMARY
 	 where model_rk=&model_rk
	 and SCORE_TIME_SK=&scr_time_sk;
quit;
 %bankfdn_rcset(&sqlrc);


  /* Inserting the rows */

  proc sql undo_policy = none;

    insert into backtest.MODEL_BACKTEST_SUMMARY

	  (MODEL_RK,SCORE_TIME_SK,TARGET_TIME_SK,DEV_MEASURE_VERSION_NO)

	  values

	  (&model_rk,&scr_time_sk,&tgt_time_sk,&dev_version_no);

  quit;

  %bankfdn_rcset(&sqlrc);



%end;



%if &job_rc > 4 or &syscc > 4 %then %do;

    %let param_run_status_cd = -7;

    %let param_run_status_msg_key = backtest.param_run_status_cd.7;

    %goto status_update;

%end;



*==============================================================;

* Updating status ;

*==============================================================;



%let param_run_status_cd = 0;

%let param_run_status_msg_key = backtest.param_run_status_cd.0;

    

%status_update:



%if &param_run_status_cd = 0 %then %do;

  %let job_status = SUC;

%end;

%else %do;

  %let job_status = FLD;

%end;



proc sql;

  select job_id into :job_id

  from &dabt_lib..job_status_summary

  where component_key = &model_rk

  and   upcase(compress(component_nm)) = "CSBMVA_BACKTESTING_PROCESS"

  and   JOB_START_DTTM = (select max(JOB_START_DTTM)

                          from &dabt_lib..job_status_summary

                          where component_key = &model_rk

                          and   upcase(compress(component_nm)) = "CSBMVA_BACKTESTING_PROCESS")

  ;

quit;  

 

proc sql;

  update &dabt_lib..job_status_summary

  set job_end_dttm = datetime(),

      job_status_cd = "&job_status",

	  job_status_msg_key = "&param_run_status_msg_key"

  where compress(job_id) = "%trim(%left(&job_id))";

quit;  



%mend;



%csbmva_backtest;



proc printto;

run;  

