%macro csbmva_build_abt_model(pool_scheme_no = ,model_rk = ,
                              model_type     = ,
                              sas_repository = );

*==================================================;
* Defining local macro variables ;
*==================================================;

%local m_model_id      m_level_sk
       m_mdl_abt_ds    m_scr_abt_ds   m_abt_dts
       m_longrun_flag  m_abt_stk_flg  keep_keys_lst    
       abt_run_status  stg_path;
%let trans_rc = 0;

*==================================================;
* Getting model_id from MODEL_MASTER table;
*==================================================;

proc sql noprint;
  select model_id, level_sk into :m_model_id, :m_level_sk
  from  &dabt_lib..model_master 
  where model_sk = &model_rk
  ;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================;
* Getting required information from;
* BUILD_POOL_ABT_STATUS table;
*==================================================;

proc sql noprint;
  select abt_no, mdl_abt_ds, scr_abt_ds, abt_dts, longrun_flag into
    :m_abt_no, :m_mdl_abt_ds, :m_scr_abt_ds, :m_abt_dts, :m_longrun_flag
  from  &csb_apdm_libref..build_pool_abt_status
  where pool_scheme_no = &pool_scheme_no
  and   model_rk = &model_rk;
quit;
%bankfdn_rcset(&sqlrc);

%let m_abt_dts = &m_abt_dts;
%let m_abt_dts = %quote(&m_abt_dts);

*==================================================;
* Creating modeling abt and score it;
*==================================================;

%if &m_longrun_flag = 1 %then %do;
  %let m_abt_stk_flg = Y;
%end;
%else %do;
  %let m_abt_stk_flg = N;
%end;

/* Staging path location for Pooling ABT */
/*%let csb_app_user = %scan(&_metauser,1,@);*/
%let stg_path = &CSB_CONFIG_DIR./AppData/&CSB_CONFIG_FOLDER_NM./&CSB_VERSION./logs/userlog/&CSB_APP_USER./pooling/&pool_scheme_no./model/&model_rk.;

/* Macro to create modeling ABT */
%dabt_build_bck_plg_abt_wrapper(m_model_id     = &m_model_id, m_pool_scheme_no= &pool_scheme_no, m_mdl_type        = PLG,
								m_abt_bld_dt   = &m_abt_dts , m_abt_stk_flg     = &m_abt_stk_flg,
								m_abt_stk_size = 0          , m_replace_abt_flg = Y,
								m_abt_libref          = &DABT_POOLING_ABT_LIBREF,
								m_abt_tbl_nm          = &m_mdl_abt_ds,
								m_perform_scoring_flg = Y,
								m_scored_libref       = &DABT_POOLING_ABT_LIBREF,
								m_scored_ds_nm        = &m_scr_abt_ds,
								m_abt_lst_prc_by_usr  = &SYSUSERID,
								m_abt_build_status    = abt_run_status,
								m_unique_id_for_scr_tbl= &m_abt_no, 
								m_ref_path_bld        = %quote(&stg_path.),
								m_log_file_nm		  = %quote(pooling_abt_wrapper.log) );

%let m_cs_log_loc_tmp = %sysfunc(index(&log_name,.log));      
%let m_cs_log_nm_tmp = %sysfunc(substr(&log_name,1,%eval(&m_cs_log_loc_tmp-1)))_&pool_scheme_no._&model_rk.;                                                            

proc printto log="&m_cs_log_nm_tmp..log";
run;

%let build_status_flag = &abt_run_status;

%if &abt_run_status ne 0 %then %do;
  %bankfdn_rcset(999);
  %return;
%end;

/* Copying it in CSB area */
data poolscr.&m_scr_abt_ds;
  set &DABT_POOLING_ABT_LIBREF..&m_scr_abt_ds;
run;

*==================================================;
*	Renaming the variables in scored dataset ;
*==================================================;

/* Finding the outcome variable of the model */
proc sql noprint;
  select VARIABLE_COLUMN_NM into :m_outcome_var
  from &dabt_lib..model_x_act_outcome_var as a,
       &dabt_lib..variable_master as b
  where a.variable_sk  = b.variable_sk
  and   a.target_variable_flg = 'Y'
  and   a.model_sk = &model_rk;
quit;
%bankfdn_rcset(&sqlrc);
%let m_outcome_var = &m_outcome_var;

/* START - renaming variable for actual value */

proc datasets library = poolscr nodetails;
  modify &m_scr_abt_ds;
    %if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
	  rename &m_outcome_var = ACTUAL_GOOD_BAD;
	%end;
	%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
      rename LGD = ACTUAL_LGD;
	%end;
	%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
	  rename &m_outcome_var = ACTUAL_CCF;
	%end;
  run;
quit;
%bankfdn_rcset(&syserr);

/* END - renaming variable for actual value */

/* START - renaming variable for ESTIMATED value */

proc datasets library = poolscr nodetails;
  modify &m_scr_abt_ds;
    %if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
	  rename SCORECARD_POINTS = SCORE_POINTS 
        	 P_&m_outcome_var.1 = ESTIMATED_PD;
	%end;
	%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
      rename P_LGD = ESTIMATED_LGD;
	%end;
	%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
	  rename P_&m_outcome_var = ESTIMATED_CCF;
	%end;
  run;
quit;
%bankfdn_rcset(&syserr);

/* END - renaming variable for ESTIMATED value */

*==================================================;
* code to keep required variables ;
*==================================================;

/* Find list of all higher level keys */
proc sql noprint;
  select LEVEL_KEY_COLUMN_NM into :keep_keys_lst separated by ' '
  from &dabt_lib..level_master
  where level_sk in (select assoc_level_sk
  					 from &dabt_lib..level_assoc
					 where level_sk = &m_level_sk)
  or level_sk = &m_level_sk;
quit;
%bankfdn_rcset(&sqlrc);

%if %kupcase(&model_type) = &PD_ONG_MODEL. or %kupcase(&model_type) = &PD_APP_MODEL. %then %do;
  data POOLSCR.&m_scr_abt_ds;
    set POOLSCR.&m_scr_abt_ds;
	format year_dt date9.;

    %if &m_longrun_flag eq 1 %then %do;
	  year_dt = datepart(build_dttm);
	%end;
	%else %do;
	  year_dt = input("&m_abt_dts",date9.);
	%end;

	if actual_good_bad not in ( 1 . ) then actual_good_bad = 0;

	keep &keep_keys_lst score_points estimated_pd actual_good_bad year_dt;
  run;
  %bankfdn_rcset(&syserr);
%end;
%else %if %kupcase(&model_type) = &LGD_MODEL. %then %do;
  data POOLSCR.&m_scr_abt_ds;
    set POOLSCR.&m_scr_abt_ds;
	format year_dt date9.;

    %if &m_longrun_flag eq 1 %then %do;
	  year_dt = datepart(build_dttm);
	%end;
	%else %do;
	  year_dt = input("&m_abt_dts",date9.);
	%end;

	keep &keep_keys_lst estimated_lgd actual_lgd year_dt;
  run;
  %bankfdn_rcset(&syserr);
%end;
%else %if %kupcase(&model_type) = &CCF_MODEL. %then %do;
  data POOLSCR.&m_scr_abt_ds;
    set POOLSCR.&m_scr_abt_ds;
	format year_dt date9.;

    %if &m_longrun_flag eq 1 %then %do;
	  year_dt = datepart(build_dttm);
	%end;
	%else %do;
	  year_dt = input("&m_abt_dts",date9.);
	%end;

	keep &keep_keys_lst estimated_ccf actual_ccf year_dt;
  run;
  %bankfdn_rcset(&syserr);
%end;

%if &trans_rc > 4 %then %do;
  %let build_status_flag = 1;
%end;

%if &build_status_flag = 0 %then %do;
%let status_cd = CMPT;
%end;
%else %do;
%let status_cd = FAIL;
%end;

proc sql;
update  &csb_apdm_libref..build_pool_abt_status
  set status_cd = "&status_cd"
  where pool_scheme_no = &pool_scheme_no
  and   model_rk       = &m_model_rk;
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_build_abt_model;
