%macro csbmva_build_abt_pooling(pool_scheme_no = ,pool_scheme_type_cd = ,
                                abt_dt =         ,action              = );

*==================================================;
* Getting details of pool_scheme ;
*==================================================;

proc sql noprint;
  select pd_method_type_cd, pd_model_rk, lgd_method_type_cd, lgd_model_rk,
    ccf_method_type_cd, ccf_model_rk into
    :pd_method, :pd_model_key, :lgd_method, :lgd_model_key,
    :ccf_method, :ccf_model_key
  from  &csb_apdm_libref..pool_scheme_defn
  where pool_scheme_no = &pool_scheme_no.;
quit;
%bankfdn_rcset(&sqlrc);

%let pd_method = &pd_method;
%let pd_model_key = &pd_model_key;
%let lgd_method = &lgd_method;
%let lgd_model_key = &lgd_model_key;
%let ccf_method = &ccf_method;
%let ccf_model_key = &ccf_model_key;

*==================================================;
* Getting level of each model ;
*==================================================;

%local level_sk_pd level_sk_lgd level_sk_ccf;
proc sql noprint;
  select b.LEVEL_SK, LEVEL_KEY_COLUMN_NM into :level_sk_pd, :level_var_pd
  from &dabt_lib..model_master_extension as a, &dabt_lib..level_master as b
  where a.level_sk = b.level_sk
  and   a.model_key = &pd_model_key
  ;

  select b.LEVEL_SK, LEVEL_KEY_COLUMN_NM into :level_sk_lgd, :level_var_lgd
  from &dabt_lib..model_master_extension as a, &dabt_lib..level_master as b
  where a.level_sk = b.level_sk
  and   model_key = &lgd_model_key
  ;

  select b.LEVEL_SK, LEVEL_KEY_COLUMN_NM into :level_sk_ccf, :level_var_ccf
  from &dabt_lib..model_master_extension as a, &dabt_lib..level_master as b
  where a.level_sk = b.level_sk
  and   model_key = &ccf_model_key
  ;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================;
* Aligning ABT date and creating char date;
*==================================================;

proc sql noprint;
  select upcase(model_freq_cd) into :m_model_freq_cd
  from &dabt_lib..model_master_extension
  where model_key = &pd_model_key;
quit;
%bankfdn_rcset(&sqlrc);

%let abt_dt = %sysfunc(intnx(&m_model_freq_cd,&abt_dt,0,end));

data _null_;
  dt = &abt_dt;
  dd = put(day(dt),z2.);
  mm = put(month(dt),z2.);
  yy = put(year(dt),4.);
  call symput('abt_char_dt',yy||mm||dd);
run;
%bankfdn_rcset(&syserr);

*==================================================;
* In case of Restart delete records from ;
* POOL.build_pool_abt_status table;
*==================================================;

%if &action = R %then %do;
  proc sql;
    delete * from &csb_apdm_libref..build_pool_abt_status
    where POOL_SCHEME_NO = &pool_scheme_no;
    
    delete * from &csb_apdm_libref..pool_defn
    where POOL_SCHEME_NO = &pool_scheme_no;
  quit;
  %bankfdn_rcset(&sqlrc);
%end;

*==================================================;
* Setting longrun flag ;
*==================================================;

%let longrun_flag = 0;
%let pd_mtd=%quote(%'&pd_method%');
%let lgd_mtd=%quote(%'&lgd_method%');
%let ccf_mtd=%quote(%'&ccf_method%');

%if (%kupcase(&pd_mtd) eq %kupcase(&LONG_RUN_MAX_TYPE.)) or
    (%kupcase(&pd_mtd) eq %kupcase(&LONG_RUN_AVG_TYPE.)) or
    (%kupcase(&lgd_mtd) eq %kupcase(&LONG_RUN_MAX_TYPE.)) or
    (%kupcase(&lgd_mtd) eq %kupcase(&LONG_RUN_AVG_TYPE.)) or
    (%kupcase(&ccf_mtd) eq %kupcase(&LONG_RUN_MAX_TYPE.)) or
    (%kupcase(&ccf_mtd) eq %kupcase(&LONG_RUN_AVG_TYPE.))
%then %do;
  %let longrun_flag = 1;
%end;
%else %do;
  %let longrun_flag = 0;
%end;

*==================================================;
* processing for pd lgd ccf based scheme ;
*==================================================;

%if &pool_scheme_type_cd = &PD_LGD_CCF_RANGE_TYPE %then %do;

  %if &longrun_flag = 1 %then %do; /* START - longrun loop */

*--------------------------------------------------;
* finding the recovery period of LGD model ;
*--------------------------------------------------;

    proc sql noprint;
      select TARGET_PERIOD_CNT, TARGET_PERIOD_TIME_FREQ_CD
        into :tgt_period_cnt , : tgt_freq_cd
      from &dabt_lib..model_master_extension
      where model_key = &lgd_model_key ;
    quit;
	%bankfdn_rcset(&sqlrc);

    data _null_;
      recv_dt = intnx("&tgt_freq_cd", &abt_dt, -(&tgt_period_cnt + &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR), 'end');
      dd = put(day(recv_dt),z2.);
      mm = put(month(recv_dt),z2.);
      yy = put(year(recv_dt),4.);
      call symput('recv_dt_suff',yy||mm||dd);
      call symput('dt_recovery_start', recv_dt);
    run;
	%bankfdn_rcset(&syserr);

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

	%let lgd_scr_abt = _lng_&pool_scheme_no._&lgd_model_key._scr_ds ;
    %let lgd_mdl_abt = _lng_&pool_scheme_no._&lgd_model_key._mdl_ds ;

    %let pd_scr_abt = _lng_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _lng_&pool_scheme_no._&pd_model_key._mdl_ds ;

    %let ccf_scr_abt = _lng_&pool_scheme_no._&ccf_model_key._scr_ds ;
    %let ccf_mdl_abt = _lng_&pool_scheme_no._&ccf_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;

    data longrun_dts;
	  dt = put(&abt_dt,date9.);
	  output;
	  do i = 0 to %eval(&LONG_RUN_YRS - 1);
		dt_temp = intnx('year', &dt_recovery_start, -(i), 'same');
		dt = put(intnx("&tgt_freq_cd", dt_temp, 0, 'end'),date9.);
        output;
      end;
    run;
	%bankfdn_rcset(&syserr);

    proc sql noprint;
      select dt into :all_abt_dts separated by ','
      from longrun_dts;
    quit;
	%bankfdn_rcset(&sqlrc);

  %end; /* END - longrun loop */

  %else %do; /* START - Estimated Method */

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

	%let lgd_scr_abt = _est_&pool_scheme_no._&lgd_model_key._scr_ds ;
    %let lgd_mdl_abt = _est_&pool_scheme_no._&lgd_model_key._mdl_ds ;

    %let pd_scr_abt = _est_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _est_&pool_scheme_no._&pd_model_key._mdl_ds ;

    %let ccf_scr_abt = _est_&pool_scheme_no._&ccf_model_key._scr_ds ;
    %let ccf_mdl_abt = _est_&pool_scheme_no._&ccf_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;
    data _null_;
      dt = put(&abt_dt,date9.);
      call symput('all_abt_dts',dt);
    run;
	%bankfdn_rcset(&syserr);

  %end; /* END - Estimated Method */


*--------------------------------------------------;
* inserting row in build_pool_abt_status table ;
*--------------------------------------------------;

  /* ABT_NO is an auto increment column, no need to pass its value during insertion */
  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
            VALUES
      (&pool_scheme_no,"&pd_method",&longrun_flag, "&pd_mdl_abt",  "&pd_scr_abt",
       &pd_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
          VALUES
      (&pool_scheme_no,"&lgd_method",&longrun_flag, "&lgd_mdl_abt",  "&lgd_scr_abt",
       &lgd_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
            VALUES
      (&pool_scheme_no,"&ccf_method",&longrun_flag, "&ccf_mdl_abt",  "&ccf_scr_abt",
       &ccf_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

%end;

*==================================================;
* processing for pd lgd based scheme ;
*==================================================;

%if &pool_scheme_type_cd = &PD_LGD_RANGE_TYPE %then %do;

  %if &longrun_flag = 1 %then %do; /* START - longrun loop */

*--------------------------------------------------;
* finding the recovery period of LGD model ;
*--------------------------------------------------;

    proc sql noprint;
      select TARGET_PERIOD_CNT, TARGET_PERIOD_TIME_FREQ_CD
        into :tgt_period_cnt , : tgt_freq_cd
      from &dabt_lib..model_master_extension
      where model_key = &lgd_model_key 
            ;
    quit;
	%bankfdn_rcset(&sqlrc);

    data _null_;
      recv_dt = intnx("&tgt_freq_cd", &abt_dt, -(&tgt_period_cnt + &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR), 'end');
      dd = put(day(recv_dt),z2.);
      mm = put(month(recv_dt),z2.);
      yy = put(year(recv_dt),4.);
      call symput('recv_dt_suff',yy||mm||dd);
      call symput('dt_recovery_start', recv_dt);
    run;
	%bankfdn_rcset(&syserr);

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

	%let lgd_scr_abt = _lng_&pool_scheme_no._&lgd_model_key._scr_ds ;
    %let lgd_mdl_abt = _lng_&pool_scheme_no._&lgd_model_key._mdl_ds ;

    %let pd_scr_abt = _lng_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _lng_&pool_scheme_no._&pd_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;

    data longrun_dts;
      dt = put(&abt_dt,date9.);
	  output;
	  do i = 0 to %eval(&LONG_RUN_YRS - 1);
		dt_temp = intnx('year', &dt_recovery_start, -(i), 'same');
		dt = put(intnx("&tgt_freq_cd", dt_temp, 0, 'end'),date9.);
        output;
      end;
    run;
	%bankfdn_rcset(&syserr);

    proc sql noprint;
      select dt into :all_abt_dts separated by ','
      from longrun_dts;
    quit;
	%bankfdn_rcset(&sqlrc);

  %end; /* END - longrun loop */

  %else %do; /* START - Estimated Method */

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

	%let lgd_scr_abt = _est_&pool_scheme_no._&lgd_model_key._scr_ds ;
    %let lgd_mdl_abt = _est_&pool_scheme_no._&lgd_model_key._mdl_ds ;

    %let pd_scr_abt = _est_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _est_&pool_scheme_no._&pd_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;
    data _null_;
      dt = put(&abt_dt,date9.);
      call symput('all_abt_dts',dt);
     run;
	%bankfdn_rcset(&syserr);

  %end; /* END - Estimated Method */


*--------------------------------------------------;
* inserting row in build_pool_abt_status table ;
*--------------------------------------------------;

  /* ABT_NO is an auto increment column, no need to pass its value during insertion */
  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
            VALUES
      (&pool_scheme_no,"&pd_method",&longrun_flag, "&pd_mdl_abt",  "&pd_scr_abt",
       &pd_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
          VALUES
      (&pool_scheme_no,"&lgd_method",&longrun_flag, "&lgd_mdl_abt",  "&lgd_scr_abt",
       &lgd_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

%end;

*==================================================;
* processing for pd or score based scheme or ;
* PD to master scale;
*==================================================;

%if &pool_scheme_type_cd = &PD_RANGE_TYPE or &pool_scheme_type_cd = &MAP_PD_MASTER_TYPE or
    &pool_scheme_type_cd = &SCORE_RANGE_TYPE
%then %do;

  %if &longrun_flag = 1 %then %do; /* START - longrun loop */

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

    %let pd_scr_abt = _lng_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _lng_&pool_scheme_no._&pd_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;

	proc sql noprint;
      select TARGET_PERIOD_CNT, TARGET_PERIOD_TIME_FREQ_CD
        into :tgt_period_cnt , :tgt_freq_cd
      from &dabt_lib..model_master_extension
      where model_key = &pd_model_key ;
    quit;
	%bankfdn_rcset(&sqlrc);
	%let tgt_freq_cd = &tgt_freq_cd;

    data longrun_dts;
      dt = put(&abt_dt,date9.);
	  dt1 = intnx("&tgt_freq_cd", &abt_dt, -(&tgt_period_cnt ), 'end');
	  output;
	  do i = 0 to %eval(&LONG_RUN_YRS - 1);
		dt_temp = intnx('year', dt1, -(i), 'same');
		dt = put(intnx("&tgt_freq_cd", dt_temp, 0, 'end'),date9.);
        output;
      end;
    run;
	%bankfdn_rcset(&syserr);

    proc sql noprint;
      select dt into :all_abt_dts separated by ','
      from longrun_dts;
    quit;
	%bankfdn_rcset(&sqlrc);

  %end; /* END - longrun loop */

  %else %do; /* START - Estimated Method */

*--------------------------------------------------;
* creating name of ABTS for all models ;
*--------------------------------------------------;

    %let pd_scr_abt = _est_&pool_scheme_no._&pd_model_key._scr_ds ;
    %let pd_mdl_abt = _est_&pool_scheme_no._&pd_model_key._mdl_ds ;

*--------------------------------------------------;
* finding date of ABTS for all models ;
*--------------------------------------------------;
    data _null_;
      dt = put(&abt_dt,date9.);
      call symput('all_abt_dts',dt);
    run;
	%bankfdn_rcset(&syserr);

  %end; /* END - Estimated Method */

*--------------------------------------------------;
* inserting row in build_pool_abt_status table ;
*--------------------------------------------------;

  proc sql;
    insert into  &csb_apdm_libref..build_pool_abt_status
      (POOL_SCHEME_NO, METHOD_TYPE_CD, LONGRUN_FLAG, MDL_ABT_DS, SCR_ABT_DS,
       MODEL_RK, ABT_DTS, STATUS_CD)
            VALUES
      (&pool_scheme_no,"&pd_method",&longrun_flag, "&pd_mdl_abt",  "&pd_scr_abt",
       &pd_model_key, "&all_abt_dts", '');
  quit;
  %bankfdn_rcset(&sqlrc);

%end;

*==================================================;
* calling the macro to build the ABTS for the;
* pool_scheme;
*==================================================;

%csbmva_build_abt_pool_scheme(pool_scheme_no = &pool_scheme_no);

*==================================================;
* Joining abts of diff model to create o/p data ;
*==================================================;

*--------------------------------------------------;
* creating o/p for pd lgd ccf scheme ;
*--------------------------------------------------;

%if &pool_scheme_type_cd = &PD_LGD_CCF_RANGE_TYPE %then %do;

  proc sql;
    create table out_abt_ds as
      select a.*,b.actual_lgd,b.estimated_lgd,
             c.actual_good_bad, c.score_points,c.estimated_pd
      from poolscr.&ccf_scr_abt a,
           poolscr.&lgd_scr_abt b,
           poolscr.&pd_scr_abt c
      where a.&level_var_lgd = b.&level_var_lgd
      and   a.year_dt = b.year_dt
      and   a.&level_var_pd = c.&level_var_pd
      and   a.year_dt = c.year_dt;
  quit;
  %bankfdn_rcset(&sqlrc);

  data pool._&pool_scheme_no._build_pool_in_ds;
    length &LEVEL_VAR_CCF SCORE_POINTS ESTIMATED_PD ESTIMATED_LGD ESTIMATED_CCF
           ACTUAL_GOOD_BAD ACTUAL_LGD ACTUAL_CCF YEAR_DT 8;
    set out_abt_ds;

    pool_scheme_no = &pool_scheme_no;
	score_points = round(score_points);
    keep &level_var_ccf score_points estimated_pd estimated_lgd estimated_ccf
         actual_good_bad actual_lgd actual_ccf year_dt;
    rename &level_var_ccf = RECORD_RK;
  run;
  %bankfdn_rcset(&syserr);

%end;

*--------------------------------------------------;
* creatinf o/p for pd lgd scheme ;
*--------------------------------------------------;

%if &pool_scheme_type_cd = &PD_LGD_RANGE_TYPE %then %do;

  proc sql;
    create table out_abt_ds as
      select a.*,b.actual_good_bad, b.score_points,b.estimated_pd
      from poolscr.&lgd_scr_abt a,
           poolscr.&pd_scr_abt b
      where a.&level_var_pd = b.&level_var_pd
      and a.year_dt = b.year_dt;
  quit;
  %bankfdn_rcset(&sqlrc);

  data pool._&pool_scheme_no._build_pool_in_ds;
    length &LEVEL_VAR_LGD SCORE_POINTS ESTIMATED_PD ESTIMATED_LGD ESTIMATED_CCF
           ACTUAL_GOOD_BAD ACTUAL_LGD ACTUAL_CCF YEAR_DT 8;
    set out_abt_ds;

    pool_scheme_no = &pool_scheme_no;
	score_points = round(score_points);
    estimated_ccf =.;
    actual_ccf =.;
    keep &level_var_lgd score_points estimated_pd estimated_lgd estimated_ccf
         actual_good_bad actual_lgd actual_ccf year_dt;
    rename &level_var_lgd = RECORD_RK;
  run;
  %bankfdn_rcset(&syserr);

%end;

*--------------------------------------------------;
* creatinf o/p for others scheme ;
*--------------------------------------------------;

%if &pool_scheme_type_cd = &PD_RANGE_TYPE or &pool_scheme_type_cd = &MAP_PD_MASTER_TYPE or
    &pool_scheme_type_cd = &SCORE_RANGE_TYPE
%then %do;

  data pool._&pool_scheme_no._build_pool_in_ds;
    length &LEVEL_VAR_PD SCORE_POINTS ESTIMATED_PD ESTIMATED_LGD ESTIMATED_CCF
           ACTUAL_GOOD_BAD ACTUAL_LGD ACTUAL_CCF YEAR_DT 8;
    set poolscr.&pd_scr_abt;

    pool_scheme_no = &pool_scheme_no;
	score_points = round(score_points);
    estimated_lgd =.;
    actual_lgd =.;
    estimated_ccf =.;
    actual_ccf =.;
    keep &level_var_pd score_points estimated_pd estimated_lgd estimated_ccf
         actual_good_bad actual_lgd actual_ccf year_dt;
    rename &level_var_pd = record_rk;
  run;
  %bankfdn_rcset(&syserr);

%end;

%mend csbmva_build_abt_pooling;
