%macro csbmva_cal_measures_ccf(model_key      = ,time_key       = ,
                               model_type     = ,feed_lib       = ,
					           feed_ds_pool   = ,feed_ds_record = ,
					           measure_ds_lib = ,measure_ds_nm  = ,
							   isDevelopmentCycle = ,
							   isPoolBased = );

*==============================================================;
* Declaring local macro variables ;
*==============================================================;

%local 	temp_model_type measure_sk_lst
        measure_cd_lst  measure_cnt
		default_code    m_measure_sk
        m_measure_cd    m_measure_val
		i ;

*==============================================================;
* Create table to store measure values ;
*==============================================================;

proc sql;
  create table &measure_ds_lib..&measure_ds_nm(
    MODEL_RK num(8),
	SCORE_TIME_SK num(8),
    MEASURE_SK num(8),
    MEASURE_VALUE num(8)
  );
quit;

*==============================================================;
* Derive model type code ;
*==============================================================;

%if &model_type = &PD_ONG_MODEL %then %do;
  %let temp_model_type  = 'PDO';
%end;
%else %if &model_type = &PD_APP_MODEL %then %do;
  %let temp_model_type  = 'PDA';
%end;
%else %if &model_type = &LGD_MODEL %then %do;
  %let temp_model_type  = 'LGD';
%end;
%else %if &model_type = &CCF_MODEL %then %do;
  %let temp_model_type  = 'CCF';
%end;

*==================================================================;
* Mapping time_sk to datetime value ;
*==================================================================;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;

*==============================================================;
*  Find associated Pool scheme Sk;
*==============================================================;

%if &isPoolBased = true %then %do;

	%if &isDevelopmentCycle = true %then %do;
		proc sql noprint;
			select pool_scheme_sk into :m_pool_sch_sk
			from   &csb_apdm_libref..Model_x_pool_scheme
					where current_indicator=1
					and model_rk = &model_key		
			;
		quit;
	%end;
	%else %do;
		proc sql noprint;
			select pool_scheme_sk into :m_pool_sch_sk
			from   csbridge.analytical_model_x_pool_scheme
					where valid_start_dttm <= &period_end_dttm <= valid_end_dttm
					and model_rk = &model_key					
			;
		quit;
	%end;

	proc sql noprint;
		select rank_ordered_ind into :m_rank_flg
		from &csb_apdm_libref..pool_scheme
		where pool_scheme_sk = &m_pool_sch_sk;
	quit;

%end;

*==============================================================;
*  Find measure list;
*==============================================================;

proc sql noprint;

  select a.measure_sk , lowcase(a.measure_cd) , count(*)  
    into :measure_sk_lst separated by '#',
         :measure_cd_lst separated by '#',
		 :measure_cnt
  from &cs_dim_lib..measure_dim as a, &cs_bridge..model_type_x_measure as b
  where a.measure_sk = b.measure_sk 
  		%if &isPoolBased = true %then %do;
			and a.rank_order_reqd_ind = a.rank_order_reqd_ind * &m_rank_flg
		%end;
  and Kupcase(b.model_type_cd) = %kupcase(&temp_model_type);

quit;
%bankfdn_rcset(&sqlrc);

proc sql noprint;
  create table other_msr as
  select a.measure_cd
  from &cs_dim_lib..measure_dim as a, &cs_bridge..model_type_x_measure as b
  where a.measure_sk = b.measure_sk   		 
  		%if &isPoolBased = true %then %do;
			and a.rank_order_reqd_ind = a.rank_order_reqd_ind * &m_rank_flg
		%end;
  and kupcase(measure_level_cd) = 'OTHE'
  and Kupcase(b.model_type_cd) = %kupcase(&temp_model_type);

quit;
%bankfdn_rcset(&sqlrc);

/* Set measure value to NULL for OTHER Reports */
data _null_;
  set other_msr;
  call symput(measure_cd,'.');
run;


	/* Calculate SYSTEM STABILITY INDEX */
	%if %sysfunc(indexw(&measure_cd_lst,sysstbindx,#)) > 0 %then %do;

		%csbmva_sysstbindx(input_ds = &feed_lib..&feed_ds_pool,output_ds = ssi,flag=2);
  		proc sql noprint;
    		select sum(sysstbindx) into :sysstbindx 
			from work.ssi;
  		quit;

	%end;

	/* Calculate Mean Squared Error (MSE), Mean Absolute Deviation (MAD) */
	%if %sysfunc(indexw(&measure_cd_lst,mse,#)) > 0
				or
		%sysfunc(indexw(&measure_cd_lst,mad,#)) > 0
	%then %do;

		%csbmva_errormeasures(input_ds = &feed_lib..&feed_ds_pool,input_ds_estvar = Estimated_CCF,input_ds_actvar = Actual_CCF,output_ds = work.mse_mad);
  		proc sql noprint;
    		select mse, mad into :mse, :mad
			from work.mse_mad;
  		quit;

	%end;

	/* Calculate Mean Absolute Percent Error (MAPE) */
	%if %sysfunc(indexw(&measure_cd_lst,mape,#)) > 0 %then %do;

		%csbmva_mape(input_ds=&feed_lib..&feed_ds_pool,output_ds=mape1,flag=2,model_type =&model_type);
   		proc sql noprint;
    		select mape into :mape
			from work.mape1;
  		quit;

	%end;

    /* Calculate Chi-Square Test (p-value) */
	%if %sysfunc(indexw(&measure_cd_lst,chsqp,#)) > 0 %then %do;

		%csbmva_chsqp(input_ds = &feed_lib..&feed_ds_pool,output_ds = WORK.chsqp,flag=2,model_type =&model_type);
  		proc sql noprint;
    		select chsqp into :chsqp
			from work.chsqp;
  		quit;

	%end;

	/* Calculate Correlation Analysis */
	%if %sysfunc(indexw(&measure_cd_lst,corr,#)) > 0 %then %do;

		%csbmva_corr(input_ds = &feed_lib..&feed_ds_pool,output_ds = WORK.corr,flag=2,model_type =&model_type);
  		data _null_;
    		set corr;
			call symput(_type_,measure_value);
  		run;

	%end;

    /*  Insert measure values in MEASURE_DS */
    %do i = 1 %to &measure_cnt;
    	%let m_measure_sk = %kscan(&measure_sk_lst,&i,%str(#));
		%let m_measure_cd = %kscan(&measure_cd_lst,&i,%str(#));
		%let m_measure_val = .;
    	%let m_measure_val = &&&m_measure_cd;

    	proc sql;
	  		insert into &measure_ds_lib..&measure_ds_nm
	    		(model_rk, score_time_sk, measure_sk, measure_value)
		  	values
				(&model_key, &time_key, &m_measure_sk, &m_measure_val);
		quit;
		%bankfdn_rcset(&sqlrc);
	%end;



%mend csbmva_cal_measures_ccf;
