/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_feed_pd_mip.sas
Function       : This macro creates the feed table for PD Model for Model Input Reports.

Authors        :  CSB Team
Date           :  29Oct2010
SAS            :  9.2

Called-by      :   
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  SCORE_CARD_DIM (CSBDIM library)
                  SCORE_CARD_VARIABLE_DIM (CSBDIM library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_PD_FACT (CSB FACT library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_SC_FACT (CSB FACT library)
                  _<MODEL_RK>_SC_RANGE_FACT (CSB FACT library)
                  
Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_PD_ATTR_FD_MIP (CSB FACT library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_PD_ATTR_FD_MIP (CSB FACT library)

Logic          :  

Parameters     :  MODEL_KEY         -> Model_RK of the model.
                  TIME_KEY          -> Score_time_sk of the model.
                  OUT_LIB_NM        -> Library where the feed dataset will be created.
                  OUT_LIB_NM_DIM    -> Library where Dimensions are present.
                  OUT_LIB_NM_FACT   -> Library where Facts are present.
                  OUT_TABLE_NM_POOL -> Name of the attribute level feed table.
                  OUT_TABLE_NM_ACCT -> Name of the score range level feed table.

**********************************************************************************************/
%macro csbmva_feed_pd_mip(model_key         = ,time_key          = ,
                          out_lib_nm        = ,out_lib_nm_dim    = ,
                          out_lib_nm_fact   = ,out_table_nm_pool = , 
	                      out_table_nm_acct = ,isDevelopmentCycle = );

%local modelsk levelsk time_sk_exist_flg sc_range_fact_version_no range_datail_version_no model_type;

*==================================================================;
* Finding the model_key from model_master_extension ;
*==================================================================;

%if &isDevelopmentCycle = true %then %do;
	%let mm_ext_key = &model_key;
%end;
%else %do;
	proc sql noprint;
		select source_model_master_key, level_sk
			into :mm_ext_key, :levelsk
		from dim.analytical_model_dim
		where model_rk = &model_key;
	quit;
	%let mm_ext_key = &mm_ext_key;
%end;

proc sql noprint;
	select model_type_cd into :model_type
	from &csb_apdm_libref..MODEL_MASTER_EXTENSION b
	where model_key = &model_key
	;
quit;

%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

*==============================================================;
/* START - Find VERSION_NO for score_card_dim corresponding to current time_sk */
*==============================================================;

%let dev_version_no=;
%let score_card_dim_version_no=;
proc sql noprint;
	select dev_measure_version_no into :dev_version_no
	from csbfact.model_score_summary
	where model_rk = &model_key and score_time_sk = &time_key;
quit;

%if &dev_version_no ne %then %do;
	proc sql noprint;
		select  SCORECARD_ATTRIB_PROP_VERSION into :score_card_dim_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where dev_measure_version_no = &dev_version_no
		and model_rk = &mm_ext_key.;
	quit;
%end;
%else %do;
	proc sql noprint;
		select max(SCORECARD_ATTRIB_PROP_VERSION) into :score_card_dim_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where model_rk = &mm_ext_key.;
	quit;
%end;

*==============================================================;
/* END - Find VERSION_NO corresponding to current time_sk */

*==============================================================;

*==============================================================;
* Creating Attribute level feed ;
*==============================================================;

/* To populate developemnt data for all respective deminsions*/
proc sql;
  create table &debug_lib..all_dev_attribute_levels as
    select  
      a.model_rk as model_master_rk,
      a.variable_sk,
	  a.variable_group_label as attr_name length = 3000,
      a.variable_group_no as seq_no , 
      b.variable_name as var_name length = 200,
      max(a.score_card_points) as scorecard_points, 
      max(a.no_of_records) as no_of_records_dev, 
      max(a.no_of_defaults) as no_of_defaults_dev, 
      max(a.average_score) as avg_score_points
    from &out_lib_nm_dim..score_card_dim as a,
           &out_lib_nm_dim..score_card_variable_dim as b
    where a.variable_sk = b.variable_sk
	and a.variable_group_no ne -2
        and a.version_no=&score_card_dim_version_no
        and a.model_rk = &mm_ext_key
    group by a.model_rk,a.variable_sk,var_name,attr_name, seq_no;
quit;
%bankfdn_rcset(&sqlrc);

/* To populate actual data for all respective deminsions */

proc sql;
  create table &debug_lib..&out_table_nm_pool._sql as
    select 
	  a.model_rk,
 	  a.record_rk,
	  a.predicted_good_bad as predicted_good_bad,
      a.actual_good_bad as actual_good_bad,
      a.score_time_sk,
	  a.level_sk,
      c.variable_sk,
	  c.variable_group_label as attr_name length = 3000, 
	  c.variable_group_no as seq_no,
      d.variable_name as var_name length = 200
   from &out_lib_nm_fact.._&model_key._&time_key._pd_fact as a
                 inner join
       &out_lib_nm_fact.._&model_key._&time_key._sc_fact as b
   on (a.record_rk =  b.record_rk)
	             inner join
       &out_lib_nm_dim..score_card_dim as c
   on ( b.score_card_sk = c.score_card_sk and c.version_no=&score_card_dim_version_no) 
                 inner join
       &out_lib_nm_dim..score_card_variable_dim as d
   on (c.variable_sk = d.variable_sk)
  ;

quit;
%bankfdn_rcset(&sqlrc);

%if &model_type=%str(&PD_APP_MODEL) %then %do;
	proc sql;
	  create table &debug_lib..&out_table_nm_pool._app as
	    select model_rk,
	        score_time_sk,
	        level_sk,
	        variable_sk,
	        var_name, 
	        attr_name, 
		    seq_no,
	      	count(a.record_rk) as no_of_records_actual_all
		  	from &debug_lib..&out_table_nm_pool._sql a
			group by model_rk,score_time_sk,level_sk,variable_sk,var_name,attr_name,seq_no;
	quit;
%end;

/* grouping at attribute level */
proc sql;
  create table &debug_lib..&out_table_nm_pool._tmp as
    select a.model_rk,
      a.score_time_sk,
      a.level_sk,
      a.variable_sk,
      a.var_name, 
      a.attr_name, 
	  a.seq_no,
	  count(a.record_rk) as no_of_records_actual,
	  sum(a.actual_good_bad) as no_of_defaults_actual
  from &debug_lib..&out_table_nm_pool._sql a
  %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
  		on (a.record_rk = b.application_rk)

  %end;
  group by model_rk,score_time_sk,level_sk,variable_sk,var_name,attr_name,seq_no;
quit;
%bankfdn_rcset(&sqlrc);

%if &model_type=%str(&PD_APP_MODEL) %then %do;
	%let alias = b;
%end;
%else %do;
	%let alias = a;
%end;

proc sql;
  create table &debug_lib..&out_table_nm_pool. as
  	select
      &alias..model_rk,
      &alias..score_time_sk,
      &alias..level_sk,
      &alias..variable_sk,
      &alias..var_name, 
      &alias..attr_name, 
	  &alias..seq_no,
	  a.no_of_records_actual,
	  a.no_of_defaults_actual 
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			, b.no_of_records_actual_all
		%end;
		from &debug_lib..&out_table_nm_pool._tmp a 
			%if &model_type=%str(&PD_APP_MODEL) %then %do;
				right join &debug_lib..&out_table_nm_pool._app b
				on a.variable_sk = b.variable_sk 
					and a.seq_no = b.seq_no
			%end;
		;
quit;
%bankfdn_rcset(&sqlrc);

/* Merge development data with the actual data for all respective deminsions */
proc sort data  = &debug_lib..all_dev_attribute_levels;
  by variable_sk seq_no;
run;

proc sort data  = &debug_lib..&out_table_nm_pool.;
  by variable_sk seq_no;
run;

data  &debug_lib..&out_table_nm_pool.;
  merge &debug_lib..all_dev_attribute_levels (in=a ) &debug_lib..&out_table_nm_pool. (in = b);
  by  variable_sk seq_no;
  if a =1 and  b =1 then output &debug_lib..&out_table_nm_pool.;
  if a =1 and b =0 then output &debug_lib..&out_table_nm_pool.;
  if a =0 and b = 1 then output &debug_lib..&out_table_nm_pool.;
run;
%bankfdn_rcset(&syserr);

proc sql noprint;
  select max(level_sk) into
    :levelsk
  from &debug_lib..&out_table_nm_pool.;
quit;
%bankfdn_rcset(&sqlrc);

data  &out_lib_nm..&out_table_nm_pool.(drop = model_master_rk);
  length MODEL_RK  SCORE_TIME_SK LEVEL_SK VARIABLE_SK
         SEQ_NO 8
		 VARIABLE_NAME $200  
		 SCORECARD_POINTS
		 NO_OF_RECORDS_DEV NO_OF_DEFAULTS_DEV AVG_SCORE_POINTS
         NO_OF_RECORDS_ACTUAL NO_OF_DEFAULTS_ACTUAL 8;
  set &debug_lib..&out_table_nm_pool.(rename  = (attr_name = attribute_name
                                                 var_name  = variable_name));

  if score_time_sk = . then score_time_sk =  &time_key;
  if model_rk  = . then model_rk = &model_key;
  if level_sk = . then level_sk  = &levelsk;
  if no_of_records_actual =. then no_of_records_actual = 0;
  %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	if no_of_records_actual_all =. then no_of_records_actual_all = 0;
  %end;
  if no_of_records_dev = . then no_of_records_dev =0;
  if no_of_defaults_actual  = . then no_of_defaults_actual =0;
  if no_of_defaults_dev =. then no_of_defaults_dev = 0;

run;
%bankfdn_rcset(&syserr);

proc sql;
  create unique index xpk_attr_feed_mip on 
    &out_lib_nm..&out_table_nm_pool.(model_rk,score_time_sk,variable_sk,seq_no);
quit;
%bankfdn_rcset(&sqlrc);

*==============================================================;
* Creating Score Range level feed ;
*==============================================================;

/* START - Find VERSION_NO corresponding to current time_sk */
%let time_sk_exist_flg = 0;
proc sql noprint;
	select 1, dev_measure_version_no into :time_sk_exist_flg, :dev_version_no
	from csbfact.model_score_summary
	where model_rk = &model_key and score_time_sk = &time_key;
quit;

%if &time_sk_exist_flg = 1 %then %do;
	proc sql noprint;
		select sc_range_fact_version_no, range_group_version_no
				into :sc_range_fact_version_no, :range_datail_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where dev_measure_version_no = &dev_version_no
		and model_rk = &mm_ext_key.;
	quit;
%end;
%else %do;
	proc sql noprint;
		select max(sc_range_fact_version_no), max(range_group_version_no)
			into :sc_range_fact_version_no, :range_datail_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where model_rk = &mm_ext_key.;
	quit;
%end;

/* END - Find VERSION_NO corresponding to current time_sk */

/*Subset SC_RANGE_FACT for corresponding version*/
proc sql;
	create table &debug_lib.._&mm_ext_key._&time_key._sc_range_ext as
		select * from csbfact._&mm_ext_key._sc_range_fact
		where version_no = &sc_range_fact_version_no;
quit;

/*Subset MODEL_RANGE_DETAIL for corresponding version*/
proc sql;
	create table &debug_lib..range_detail_ext as
		select * from &csb_apdm_libref..model_range_detail
		where range_group_version_no = &range_datail_version_no
                and model_key=&mm_ext_key;
quit;

/* Apply bins on PD fact */
/* Call DABT macro to apply bins on account level dataset */

/*Get path for debug library*/
%local debug_path;
%let debug_path=%sysfunc(pathname(&debug_lib));

%let success_flg=1;

%csbmva_dabt_intrfc_apply_bins(m_in_bin_info_tbl_libref   = &debug_lib, m_in_bin_info_tbl_nm = range_detail_ext,
							   m_in_bin_unique_id_clmn_nm = range_sk,
							   m_in_dtl_tbl_libref  = csbfact, m_in_dtl_tbl_nm  = _&model_key._&time_key._pd_fact,
							   m_out_dtl_tbl_libref = &debug_lib, m_out_dtl_tbl_nm   = _&model_key._&time_key._pd_fact_bin,
							   m_scr_log_file_path  =&debug_path ,
                               m_scr_lib_ref        = csbscr, m_model_sk = &mm_ext_key,m_return_success_flg=success_flg);

*=============================================================;
* If interfacing  macro not completed successfully flag error ;
*=============================================================;
%IF &success_flg ne 0 %then %do;
	%let msg1= ERROR: %sysfunc(sasmsg(smd_ds.bismsg,CSMART.APPLY_BIN_ERROR.1, noquote));
		  		%let msg2= &model_key;
				%put &msg1 &msg2;  
            %ABORT ABEND;
%end;
%bankfdn_rcset(&syserr);
/* Actual Sample - Account level*/
proc sql;
  create table &debug_lib..&out_table_nm_acct._sql as
  	select 
	     SC_FACT.score_card_sk,
		 PD_FACT.range_sk,
	     PD_FACT.record_rk as record_rk,
		 PD_FACT.actual_good_bad as actual_good_bad

  	from &out_lib_nm_fact.._&model_key._&time_key._sc_fact SC_FACT
                  inner join 
			&debug_lib.._&model_key._&time_key._pd_fact_bin PD_FACT
	on (SC_FACT.record_rk = PD_FACT.record_rk);

quit;
%bankfdn_rcset(&sqlrc);
 
/* Actual sample - Group by to get the no of accounts */
proc sql;
  create table &debug_lib..&out_table_nm_acct._agact_t as
  select 
      score_card_sk, 
	  range_sk,
	  count(record_rk) as no_of_records_actual, 
      sum(actual_good_bad) as no_of_defaults_actual
  from &debug_lib..&out_table_nm_acct._sql a
 /* Subsetting the records for Applications that are ACCEPTED */
   %if &model_type=%str(&PD_APP_MODEL) %then %do;
	   inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
	   on (a.record_rk = b.application_rk)
   %end;
  group by score_card_sk, range_sk;
quit;
%bankfdn_rcset(&sqlrc);

%if &model_type=%str(&PD_APP_MODEL) %then %do;
	proc sql;
	  create table &debug_lib..&out_table_nm_acct._agact_ap as
	  select 
	      score_card_sk, 
		  range_sk,
		  count(record_rk) as no_of_records_actual_all
	  from &debug_lib..&out_table_nm_acct._sql
	  group by score_card_sk, range_sk;
	quit;
	%bankfdn_rcset(&sqlrc);
%end;


%if &model_type=%str(&PD_APP_MODEL) %then %do;
	%let alias = b;
%end;
%else %do;
	%let alias = a;
%end;


proc sql;
  create table &debug_lib..&out_table_nm_acct._agg_act as
   	select &alias..score_card_sk, 
		  &alias..range_sk,
		  a.no_of_records_actual, 
	          a.no_of_defaults_actual
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		, b.no_of_records_actual_all
	%end;
	from &debug_lib..&out_table_nm_acct._agact_t a
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			right join &debug_lib..&out_table_nm_acct._agact_ap b
				on a.range_sk = b.range_sk and 
					a.score_card_sk = b.score_card_sk
		%end;
	;
quit;

/* DEV sample  */

proc sql;
  create table &debug_lib..&out_table_nm_acct._dev as
  	select 
	    score_card_sk, 
		range_sk, 
		range_name,
		to_score, 
		no_of_records, 
		no_of_defaults 
  	from  &debug_lib.._&mm_ext_key._&time_key._sc_range_ext ;
quit;
%bankfdn_rcset(&sqlrc);
 
/* Merging DEV sample & Actual Sample  */
proc sql;
  create table &debug_lib..&out_table_nm_acct._mrg as
  	select 
		DEV.score_card_sk,
		DEV.range_sk,
		DEV.range_name as score_range,
		DEV.to_score,
		SC_DIM.variable_sk,
		SC_VAR_DIM.variable_name,
		SC_DIM.variable_group_no as seq_no,
	 	SC_DIM.variable_group_label as attribute_name,
	 	DEV.no_of_records as no_of_records_dev, 
		DEV.no_of_defaults as no_of_defaults_dev, 
		ACT.no_of_records_actual,
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			ACT.no_of_records_actual_all, 
		%end;
		ACT.no_of_defaults_actual
  	from &debug_lib..&out_table_nm_acct._dev DEV
	              	left join
       		&debug_lib..&out_table_nm_acct._agg_act ACT
  	on (DEV.score_card_sk = ACT.score_card_sk  and
	  		DEV.range_sk = ACT.range_sk)
					inner join
			csbdim.score_card_dim SC_DIM
	on (DEV.score_card_sk = SC_DIM.score_card_sk and SC_DIM.version_no=&score_card_dim_version_no)
					inner join
			csbdim.score_card_variable_dim SC_VAR_DIM
	on (SC_DIM.variable_sk = SC_VAR_DIM.variable_sk);
quit;
%bankfdn_rcset(&sqlrc);

data  &out_lib_nm..&out_table_nm_acct.(drop = score_card_sk);
  length MODEL_RK  SCORE_TIME_SK LEVEL_SK VARIABLE_SK
         SEQ_NO 8
		 VARIABLE_NAME $200 ATTRIBUTE_NAME $3000 SCORE_RANGE $3000
         TO_SCORE  
		 NO_OF_RECORDS_DEV NO_OF_DEFAULTS_DEV
         NO_OF_RECORDS_ACTUAL NO_OF_DEFAULTS_ACTUAL 8;
  set  &debug_lib..&out_table_nm_acct._mrg;

  if model_rk  = . then model_rk = &model_key;
  if score_time_sk = . then score_time_sk =  &time_key;
  if level_sk = . then level_sk  = &levelsk;

  if no_of_records_actual =. then no_of_records_actual = 0;
  %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	if no_of_records_actual_all =. then no_of_records_actual_all = 0;
  %end;
  if no_of_records_dev = . then no_of_records_dev =0;
  if no_of_defaults_actual  = . then no_of_defaults_actual =0;
  if no_of_defaults_dev =. then no_of_defaults_dev = 0;

run;
%bankfdn_rcset(&syserr);

proc sql;
  create unique index xpk_scr_range_feed_mip on 
    &out_lib_nm..&out_table_nm_acct.(model_rk,score_time_sk,variable_sk,seq_no,to_score);
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_feed_pd_mip;
