/*----Hosmer Lemeshow test-----
Macro Name	: csbmva_hlp
Function	: Finds value of Hosmer Lemeshow statistic and it's p-value
Created by	: BIS Team
Date		: Aug , 2010
SAS Version	: 9.2

	
Parameters	: 

input_ds		: Name of the input data set
output_ds		: Name of the data set which stores data for graph of events vs non events.
flag:       It takes 1 or 2 .the flag =1 means that output_ds  stores data for graphs on the screen. 
                 the flag =2 means that output_ds stores data for measure values.
 

Input Dataset(s) Format:

--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
estimated_PD	 				Number		Default Formatting
Max_Score_Points	 	Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------

Output Dataset(s) Format:(when storing data for measure value)
-------------------------------------------------------------------------
Name		Type		Format
------------------------------------------------------------------------
hlp		Numeric 	Default Formatting
hlstat 	Numeric 	Default Formatting
------------------------------------------------------------------------

Output Dataset(s) Format:(when storing data for measure value)
-------------------------------------------------------------------------
Name		Type		Format
------------------------------------------------------------------------
pool_seq_no         Numeric 	Default Formatting
Max_Score_Points    Numeric 	Default Formatting
prop_records_actual Numeric 	Default Formatting
prop_actual_bad     Numeric 	Default Formatting
prop_est_bad        Numeric 	Default Formatting
hlp                 Numeric 	Default Formatting
stat                Numeric 	Default Formatting





Logic:
1. Using the input data default rate is calcualted.
2. Using ESTIMATED_PD Hosmer Lemeshow statistic is calculated .

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
options mlogic mprint;
%macro csbmva_hlp(input_ds=,output_ds=,flag =);
%if &flag = 1 %then %do;
data _NULL_;

name=put(time(),mmss8.1);

name1=kscan(name,1,":.");

name2=kscan(name,2,":.");

name3=kscan(name,3,":.");

name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));

call symput('log_name',"&LOG_PATH_CS"||"&sysmacroname"||ktrim(kleft(name4))||".log");

run;



data temp;
set &input_ds;
var_exp_bads = ESTIMATED_PD*NO_OF_RECORDS_ACTUAL;
run;
%let err_code=;
/*****************************************************************/
%csbmva_alldefaults_nodefaults(temp,NO_OF_RECORDS_ACTUAL,var_exp_bads, err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler1;

proc sql noprint;
select Sum(NO_OF_RECORDS_ACTUAL), Sum(No_Of_Actual_Bads),sum(estimated_PD* NO_OF_RECORDS_ACTUAL)
into :Total_records, :Total_bads,:total_estbads from &input_ds;
quit;
data &output_ds;
set &input_ds;
prop_records_actual = NO_OF_RECORDS_ACTUAL/&total_records;
prop_actual_bad = No_Of_Actual_Bads/&Total_bads;
prop_est_bad = (estimated_PD* NO_OF_RECORDS_ACTUAL)/&total_estbads;
run;

data temp;
set &input_ds;
var_exp_bads = ESTIMATED_PD*NO_OF_RECORDS_ACTUAL;/*CS4.3 HF2 */
run;
data Chisq_x;
	retain CHI_SQ 0;
	set &input_ds end = last;
	/* Hosmer Lemeshow statistic follows CHI_SQUARE distribtion with n degrees of freedom*/
	CHI_SQ = sum(CHI_SQ,(No_Of_Actual_Bads-ESTIMATED_PD*NO_OF_RECORDS_ACTUAL)**2/((ESTIMATED_PD*NO_OF_RECORDS_ACTUAL)*(1-ESTIMATED_PD)));
run;

proc sql noprint;
	select count(pool_seq_no) into:DF from chisq_x;

	create table chisq as
	  select Max(CHI_SQ) as Chi_sq
	  from Chisq_x;

quit;

data output_ds;
	set chisq;
	hlp = 1 - CDF('CHISQUARE', Chi_sq, &DF); 
run;

/*****************************************************************/
%csbmva_errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler1;

proc sql noprint;
select hlp,chi_sq into:hlp,:chi_sq
from output_ds;
quit;

data &output_ds;
set &output_ds;
hlp =&hlp;
stat = &chi_sq;
run;



%return;
/*****************************************************************/
%Error_Handler1:
%put WARNING: &err_code;
%csbmva_defdata(input_ds=&input_ds,stat='HL',
		out_graph_ds =&output_ds,output_ds = ,out_stat_ds=);
			


%end;



%if &flag = 2 %then %do;



data temp;
set &input_ds;
var_exp_bads = ESTIMATED_PD*NO_OF_RECORDS_ACTUAL;/*CS4.3 HF2 */
run;
%let err_code=;
/*****************************************************************/
%csbmva_alldefaults_nodefaults(temp,NO_OF_RECORDS_ACTUAL,var_exp_bads, err_code);
%put &err_code;
%if (&err_code ne 'NONE')%then %goto Error_Handler2;
/*****************************************************************/

data Chisq_x;
	retain CHI_SQ 0;
	set &input_ds end = last;
	/* Hosmer Lemeshow statistic follows CHI_SQUARE distribtion with n degrees of freedom*/
	CHI_SQ = sum(CHI_SQ,(No_Of_Actual_Bads-ESTIMATED_PD*NO_OF_RECORDS_ACTUAL)**2/((ESTIMATED_PD*NO_OF_RECORDS_ACTUAL)*(1-ESTIMATED_PD)));
run;

proc sql noprint;
	select count(pool_seq_no) into:DF from chisq_x;
	create table chisq as
	select Max(CHI_SQ) as Chi_sq
	from chisq_x;
quit;

/*Calculating the P value of Chi square stats*/
data &output_ds;
	set chisq;
	chsqp = 1 - CDF('CHISQUARE', Chi_sq, &DF); 
run;


/*****************************************************************/
%csbmva_errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler2;
/*****************************************************************/
data &output_ds;
  set &output_ds;

  rename chsqp = hlp CHI_SQ = hlstat;
run;
%return;
%Error_Handler2:
%put WARNING: &err_code;
%csbmva_defdata(input_ds=&input_ds,stat='HL',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);


%end;

%mend csbmva_hlp;
