%macro csbmva_insert_measure_val(model_key        = ,time_key        = ,
                                 measure_fact_lib = ,measure_fact_ds = ,
                                 measure_ds_lib   = ,measure_ds_nm   = ,
                                 dev_cnt          = ,isDevelopmentCycle = ,
				 				 );

*==================================================;
* Defining local macro variables ;
*==================================================;

%local  m_model_rk  m_level_sk    
        m_time_key  m_measure_sk     m_measure_value 
        totalobs    period_end_dttm  m_target_time_sk i
		m_version_no m_processed_dttm;

%let m_processed_dttm = %sysfunc(datetime(),datetime.);

*==================================================;
* Create MEASURE_FACT if does not exist ;
*==================================================;

%if %sysfunc(exist(&measure_fact_lib..&measure_fact_ds)) eq 0 %then %do;
	proc sql;
		CREATE TABLE &measure_fact_lib..&measure_fact_ds(
		       MEASURE_SK           NUMERIC(8),
		       SCORE_TIME_SK        NUMERIC(8),
			   TARGET_TIME_SK       NUMERIC(8), 
               DEV_CNT              NUMERIC(8),
               MODEL_RK             NUMERIC(8),
               MEASURE_VALUE        NUMERIC(8),
		       LEVEL_SK             NUMERIC(8),
			   CLR_MEASURE          NUMERIC(8),
			   CLR_LEVEL            NUMERIC(8),
			   VERSION_NO			NUMERIC(8),
			   PROCESSED_DTTM		NUMERIC(8)	format = datetime.
		);

		CREATE UNIQUE INDEX XPK_MEASURE_FACT ON &measure_fact_lib..&measure_fact_ds
		(
               MODEL_RK,
               MEASURE_SK,
		       SCORE_TIME_SK,
		       DEV_CNT,
			   VERSION_NO
		);
	quit;
	%bankfdn_rcset(&sqlrc);
%end;

*==================================================;
* Get MODEL parameters ;
*==================================================;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);


proc sql noprint;
  select model_type_cd, level_sk, target_period_cnt, target_period_time_freq_cd
    into  :m_model_type, :m_level_sk, :m_tgt_prd, :m_tgt_freq_cd
  from %if &isDevelopmentCycle = true %then &dabt_lib..model_master_extension ;
	%else dim.analytical_model_dim; 
  where %if &isDevelopmentCycle  = true %then model_key ; %else model_rk; = &model_key
  %if &isDevelopmentCycle = true %then  ; %else and valid_start_dttm <= &period_end_dttm <= valid_end_dttm;;
quit;

%let m_model_type = &m_model_type;
%let m_model_type = %str(%')&m_model_type%str(%');
%let m_tgt_freq_cd = &m_tgt_freq_cd;

%let target_prd_adj_cnt = 0;
%if &m_model_type eq &LGD_MODEL %then %do;
	%let target_prd_adj_cnt = &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR;
%end;

data _null_;
	scr_dttm = &period_end_dttm.;
	scr_dt = datepart(scr_dttm);
	tgt_dt = intnx("&m_tgt_freq_cd",scr_dt, &m_tgt_prd + &target_prd_adj_cnt , 'end');
    tgt_dttm = dhms(tgt_dt,23,59,59);
    call symput('m_tgt_dttm',tgt_dttm);

 run;

 %let m_target_time_sk = .;

proc sql noprint;
	select time_sk into :m_target_time_sk
	from dim.time_dim
	where PERIOD_FIRST_DTTM <= &m_tgt_dttm <= PERIOD_LAST_DTTM;
quit;

*==================================================;
* Derive VERSION_NO ;
*==================================================;

%let m_version_no = 1;

%if &dev_cnt = 1 %then %do;
	proc sql noprint;
		select coalesce(max(VERSION_NO),0) + 1 into :m_version_no
		from &measure_fact_lib..&measure_fact_ds
		where dev_cnt = 1;
	quit;
%end;

proc sql noprint;
  select count(*) into :totalobs
  from &measure_ds_lib..&measure_ds_nm;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================;
* Insert data in MEASURE_FACT ;
*==================================================;

%do i = 1 %to &totalobs;

	data _null_;
	  pointer = &i;
	  set &measure_ds_lib..&measure_ds_nm point = pointer;

	  call symput('m_model_rk',model_rk);
	  call symput('m_time_key',score_time_sk);
	  call symput('m_measure_sk',measure_sk);
	  call symput('m_measure_value',measure_value);
	  stop;
	run;

	proc sql;
	  insert into &measure_fact_lib..&measure_fact_ds
	    (MEASURE_SK, SCORE_TIME_SK, TARGET_TIME_SK, DEV_CNT, 
		 MODEL_RK,MEASURE_VALUE, LEVEL_SK, VERSION_NO, PROCESSED_DTTM)
	      values
		(&m_measure_sk, &m_time_key, &m_target_time_sk, &dev_cnt,
		 &m_model_rk, &m_measure_value, &m_level_sk, &m_version_no, "&m_processed_dttm."dt );
	quit;
	%bankfdn_rcset(&sqlrc);

%end;

%mend csbmva_insert_measure_val;
