%macro csbmva_repool_feed_pd_scr (	Model_key	 	=  , Time_key			=  ,
									input_ds_lib 	=  , input_ds_nm     	=  ,
									out_feed_lib 	=  , feed_ds_nm_pool 	=  ,
									method_pd	 	=  , method_lgd			=  ,
									method_ccf	 	=  , method_scr			=  ,
									range_ds_nm		=  , range_ds_lib		= 	);

*==================================================================;
* declaring local macro variables ;
*==================================================================;

%local period_end_dttm  m_level_sk scheme_type mm_ext_key range_table;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);

*==================================================================;
* capturing few values from source into macro variables ;
*==================================================================;
%let model_type =;
proc sql noprint;
  select kupcase(model_type_cd) into
    :model_type
  from dim.analytical_model_dim
  where model_rk = &model_key;
quit;

%let model_type =&model_type;
%let model_type = %str(%')&model_type%str(%');
proc sql noprint;
  select  level_sk
    into  :m_level_sk
  from &input_ds_lib..&input_ds_nm;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Subsetting data if model type is PDA and getting ;
* few values from ANALYTICAL_MODEL_DIM ;
*==================================================================;
proc sql;
  create table feed_input_prep as
    select a.*
	from &input_ds_lib..&input_ds_nm as a
  	     %if &model_type = &PD_APP_MODEL  %then %do;
           inner join dim.application_dim(keep = application_rk outcome_cd valid_end_dttm
										  where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
		   on (a.record_rk = b.application_rk )
		 %end;
    ;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Process range table ;
*==================================================================;

proc sql;
	create table range_table_ext as
		select 
			pool_seq_no, pool_name,
			from_score, coalesce(from_score, -99999) as from_score_imput,
			to_score, coalesce(to_score, 99999) as to_score_imput
		from &range_ds_lib..&range_ds_nm;
quit;

*==================================================================;
* Finding the model_key value for name of model_dev_fact ;
*==================================================================;

proc sql noprint;
	select source_model_master_key into :mm_ext_key
	from dim.analytical_model_dim
	where model_rk = &model_key;
quit;
%let mm_ext_key = &mm_ext_key;

*==================================================================;
* generating Pool level feed ;
*==================================================================;

/* START - Calculating Actual data */

proc sql;
  create table feed_repool_temp1 as 
    select a.*,b.pool_name,b.pool_seq_no
	from feed_input_prep as a
           right outer join
         range_table_ext as b
	on ( b.from_score_imput < a.score_points <= b.to_score_imput )
	where pool_seq_no ne .;
quit;
%bankfdn_rcset(&sqlrc);


/* Grouping the actual data at POOL */
proc sql;
  create table feed_repool_temp2 as
    select distinct pool_name, pool_seq_no,
      sum(case when record_rk ne . then 1 else 0 end) as no_of_records_actual,
	  coalesce(sum(actual_good_bad),0) as no_of_actual_bads,
	  &model_key as model_rk,
	  &time_key as score_time_sk,
      &m_level_sk as level_sk
	from feed_repool_temp1 
  	group by pool_seq_no;
quit;
%bankfdn_rcset(&sqlrc);


/* calculating no_of_records_actual_all */
%if &model_type= &PD_APP_MODEL  %then %do;
	proc sql;
	  create table feed_repool_act_all as 
	    select
			pool_seq_no,
			sum(case when record_rk ne . then 1 else 0 end) as no_of_records_actual_all
		from &input_ds_lib..&input_ds_nm a
	           right outer join
	         range_table_ext as b
		on ( b.from_score_imput < a.score_points <= b.to_score_imput )
		group by pool_seq_no;
	quit;
%end;

/* END - Calculating Actual data */

/* START - Calculating DEV data */

proc sql;
  create table feed_repool_temp1_dev as 
    select  a.*,
			b.pool_name, b.pool_seq_no
	from &cs_fact_lib.._&mm_ext_key._dev_fact as a
           right outer join
         range_table_ext as b
	on ( b.from_score_imput < a.score_points <= b.to_score_imput )
    where pool_seq_no ne .;
quit;
%bankfdn_rcset(&sqlrc);

/* Grouping the dev data at POOL */
proc sql;
  create table feed_repool_temp2_dev as
    select pool_name, pool_seq_no ,
           sum(case when record_rk ne . then 1 else 0 end) as no_of_records_dev,
		   max(score_points) as max_score_points,
	  	   avg(estimated_pd) as estimated_pd
    from feed_repool_temp1_dev
	group by pool_name,pool_seq_no;
quit;
%bankfdn_rcset(&sqlrc);


/*replace the code below by looking up into csbmva_feed_pd, and create a unique index on POOL_SEQ_NO*/
proc sql;
  create table &out_feed_lib..&feed_ds_nm_pool as
    select  a.*,
		    b.estimated_pd,
      		case when b.no_of_records_dev = . then 0 
				 else b.no_of_records_dev
            end as no_of_records_dev,
			case 
				when to_score = . then coalesce(b.max_score_points,from_score + 200)
				else to_score 
			end as max_score_points
			%if &model_type= &PD_APP_MODEL  %then %do;
				
				,y.no_of_records_actual_all
			%end;
    from range_table_ext c,
		 feed_repool_temp2_dev as b,
	     feed_repool_temp2 as a 
		 %if &model_type= &PD_APP_MODEL  %then %do;
			 
			 ,feed_repool_act_all y
		%end;

	where a.pool_seq_no = b.pool_seq_no
	and b.pool_seq_no = c.pool_seq_no 
    %if &model_type= &PD_APP_MODEL  %then %do;
		
    	and c.pool_seq_no = y.pool_seq_no 
	%end;
    order by a.pool_seq_no;  
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_repool_feed_pd_scr;
