%macro csbmva_repool_feed_pool( Model_key		= , Time_key		=  ,
								input_ds_lib 	= , input_ds_nm     =  ,
								out_feed_lib 	= , feed_ds_nm_pool =  ,
								method_pd		= , method_lgd		=  ,
								method_ccf		= , method_scr		=  ,
								range_ds_nm		= ,	range_ds_lib	=  );
								
								

*==================================================================;
* declaring local macro variables ;
*==================================================================;

%local period_end_dttm  m_level_sk scheme_type mm_ext_key ;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);

*==================================================================;
* Find model type, level sk and MM ext key ;
*==================================================================;

%let m_level_sk=;
proc sql noprint;
	select  level_sk, model_type_cd, source_model_master_key
		into  :m_level_sk, :model_type, :mm_ext_key
	from dim.analytical_model_dim
	where model_rk=&model_key;
quit;
%let m_level_sk=&m_level_sk;
%let model_type =&model_type;
%let model_type = %str(%')&model_type%str(%');
%let mm_ext_key = &mm_ext_key;
%bankfdn_rcset(&sqlrc);


*==================================================================;
* Find Pool scheme sk and no ;
*==================================================================;

%let pool_scheme_sk=;
data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;

proc sql;
	select pool_scheme_sk into :pool_scheme_sk
	from csbridge.analytical_model_x_pool_scheme
	where model_rk = &model_key
	and valid_start_dttm <= &period_end_dttm <= valid_end_dttm;
quit;
%let pool_scheme_sk=&pool_scheme_sk;

%let pool_scheme_no=&pool_scheme_sk;

*==================================================================;
* Allocating  Pools to Development data ;
*==================================================================;

%let input_ds_lib = pool;
%let input_ds_nm  = _&pool_scheme_no._build_pool_in_ds;
%let out_ds_dev   = _alloc_dev_pool;
%let out_feed_ds_dev = feed_repool_dev;

%if %sysfunc(exist(rdebug.&out_feed_ds_dev)) eq 0 %then %do;

	%csbmva_allocate_pools_repool ( Model_key		= &model_key   ,Time_key		= &time_key,
									report_type		= POOL         ,pool_data_type	= DEV,
									method_pd		= &method_pd   ,method_lgd		= &method_lgd,
									method_ccf		= &method_ccf  ,method_scr		= &method_scr,
									input_ds_lib  	= &input_ds_lib,input_ds_nm 	= &input_ds_nm ,
									range_ds_nm		= &range_ds_nm ,range_ds_lib	= &range_ds_lib,
									out_ds_nm		= &out_ds_dev);


	%csbmva_repool_grp_dev_data_pool (  model_key		= &model_key,
										pool_scheme_sk	= &pool_scheme_sk,
										source_Ds		= &out_ds_dev,
										range_ds_lib	= &range_ds_lib,
										range_ds_nm		= &range_ds_nm,
										output_ds_lib	= RDEBUG,
										output_ds_nm	= &out_feed_ds_dev);

%end;

%if &syscc > 4 %then %do;
	%return;
%end;

*==================================================================;
* Allocating  Pools for Act;
*==================================================================;

%let input_ds_lib	= rdebug;
%let pool_scheme_no = &pool_scheme_no;
%let input_ds_nm	= _&pool_scheme_no._&time_key._POOL_FACT;
%let out_ds_act		= _alloc_act_pool_&time_key.;

%csbmva_allocate_pools_repool ( Model_key		= &model_key   , Time_key		= &time_key,
								report_type		= POOL         , pool_data_type	= ACT,
								method_pd		= &method_pd   , method_lgd		= &method_lgd,
								method_ccf		= &method_ccf  , method_scr		= &method_scr,
								input_ds_lib  	= &input_ds_lib, input_ds_nm 	= &input_ds_nm ,
								range_ds_nm		= &range_ds_nm , range_ds_lib	= &range_ds_lib,
								out_ds_nm		= &out_ds_act);


/*Associating Actuals and Grouping data at pool*/
%let out_feed_ds_act = feed_repool_act&time_key.;
proc sql;
	create table rdebug.&out_feed_ds_act as
		select 
			a.pool_seq_no,
			sum(case when a.record_rk ne . then 1 else 0 end) as no_of_records_actual,
			%if &model_type=&PD_APP_MODEL or &model_type=&PD_ONG_MODEL %then %do;
				coalesce(sum(b.actual_good_bad),0) as no_of_actual_bads,
			%end;
			%if &model_type=&LGD_MODEL %then %do;
  				avg(b.actual_lgd) as actual_lgd,
				sum(case when b.actual_lgd > a.estimated_lgd then 1 else 0 end) as no_of_actual_bads,
			%end;
			%if &model_type=&CCF_MODEL %then %do;
  				avg(b.actual_ccf) as actual_ccf,
				sum(case when b.actual_ccf > a.estimated_ccf then 1 else 0 end) as no_of_actual_bads,
			%end;
			&model_key as model_rk,
			&time_key as score_time_sk,
		    &m_level_sk as level_sk
		from 
			rdebug.&out_ds_act a,
			%if &model_type= &PD_APP_MODEL  %then %do;
           		dim.application_dim(keep = application_rk outcome_cd valid_end_dttm
									where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm )) as app_dim ,
			%end;
			%if &model_type=&PD_APP_MODEL or &model_type=&PD_ONG_MODEL %then %do;
				&input_ds_lib.._&model_key._&time_key._PD_FACT b
			%end;
			%if &model_type=&LGD_MODEL %then %do;
				&input_ds_lib.._&model_key._&time_key._LGD_FACT b
			%end;
			%if &model_type=&CCF_MODEL %then %do;
				&input_ds_lib.._&model_key._&time_key._CCF_FACT b
			%end;
		where
			a.record_rk = b.record_rk
			%if &model_type= &PD_APP_MODEL  %then %do;
           		and a.record_rk = app_dim.application_rk 
			%end;
		group by pool_seq_no;
quit;
%bankfdn_rcset(&sqlrc);

%if &model_type= &PD_APP_MODEL  %then %do;
	proc sql;
		create table feed_ds_act_all as
			select 
				pool_seq_no ,
				count(record_rk) as no_of_records_actual_all
			from rdebug.&out_ds_act
			group by pool_seq_no;
	quit;
%end;

*==================================================================;
* Create FEED table;
*==================================================================;

proc sql;

	CREATE TABLE &out_feed_lib..&feed_ds_nm_pool
	(
	POOL_SEQ_NO NUMERIC(8) NOT NULL,
	POOL_NAME CHARACTER(3000) FORMAT = $3000. INFORMAT = $3000.  ,
	SCORE_TIME_SK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	NO_OF_RECORDS_DEV NUMERIC(8)  ,
	NO_OF_RECORDS_ACTUAL NUMERIC(8)  ,
	%if &model_type= &PD_APP_MODEL  %then %do;
		
		NO_OF_RECORDS_ACTUAL_ALL NUMERIC(8)  ,
	%end;
	NO_OF_ACTUAL_BADS NUMERIC(8)  ,
	%if &model_type=&PD_APP_MODEL or &model_type=&PD_ONG_MODEL %then %do;
		MAX_SCORE_POINTS NUMERIC(8) /*NOT NULL*/,
		ESTIMATED_PD  NUMERIC(8) ,
	%end;
	%if &model_type=&LGD_MODEL %then %do;
		ESTIMATED_LGD  NUMERIC(8) ,
		ACTUAL_LGD  NUMERIC(8) ,
	%end;
	%if &model_type=&CCF_MODEL %then %do;
		ESTIMATED_CCF  NUMERIC(8) ,
		ACTUAL_CCF  NUMERIC(8) ,
	%end;
	MODEL_RK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	LEVEL_SK NUMERIC(8) NOT NULL 
    );

    create unique index xpk_feed_pd on &out_feed_lib..&feed_ds_nm_pool(model_rk,score_time_sk,pool_seq_no);

quit;
%bankfdn_rcset(&sqlrc);

proc sql;
	insert into &out_feed_lib..&feed_ds_nm_pool
		select
			a.pool_seq_no,
			a.pool_name,
			&time_key as score_time_sk,
			coalesce(b.no_of_records_dev, 0 ) as no_of_records_dev,
			coalesce(c.no_of_records_actual, 0 ) as no_of_records_actual,
     		coalesce(c.no_of_actual_bads, 0 ) as no_of_actual_bads,
			%if &model_type= &PD_APP_MODEL  %then %do;
				
				coalesce(d.no_of_records_actual_all, 0 ) as no_of_records_actual_all,
			%end;
     		%if &model_type=&PD_APP_MODEL or &model_type=&PD_ONG_MODEL %then %do;
				%if &method_scr ne %then %do;
					case 
						when to_score = . then coalesce(b.max_score_points,from_score + 200)
						else to_score
					end as max_score_points,
				%end;
				%else %do;
					b.max_score_points,
				%end;
				b.assigned_pd as estimated_pd,
			%end;
			%if &model_type=&LGD_MODEL %then %do;
				b.assigned_lgd as estimated_lgd,
				c.actual_lgd,
			%end;
			%if &model_type=&CCF_MODEL %then %do;
				b.assigned_ccf as estimated_ccf,
				c.actual_ccf,
			%end;
			&model_key as model_rk,
			&level_sk as level_sk
    	from 
			&range_ds_lib..&range_ds_NM a
					left join
			rdebug.&out_feed_ds_dev as b 
		on (a.pool_seq_no = b.pool_seq_no)
					left join
			rdebug.&out_feed_ds_act as c
		on (a.pool_seq_no = c.pool_seq_no)
		%if &model_type= &PD_APP_MODEL  %then %do;
						left join
				feed_ds_act_all d
			on (a.pool_seq_no = d.pool_seq_no)
		%end;
/*    	order by a.pool_seq_no*/
;  
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_repool_feed_pool;
