/*********************************************************************
       Module:  shift_indices

     Function:  This stored process creates the table of Score and event shift indices.


      Authors:  BIS Team
         Date:  05 April 2006
          SAS:  9.1
    Called-by:	shift_index
        Calls:  NA

Datasets used:  Supplied as the parameter
                Input datset is passed which is output of Macro generate_feed_mip.
				otput dataste where you will collect the value.
                
   		Logic: 1.Computaion of shift of score cardspoints for attribute proportion 
				 per attribute for a variable.
			   2.Computaion of shift of score cardspoints for event proportion 
				 per attribute for a variable.
				
   Parameters:  INPUT (From Report Screen when link is used for navigation) 
				1.input_ds   :Dataset passed is  
				2.output_ds  :out put library. where we have our dataset.
	Notes:
				1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
				2. It is assumed that data set of specified as input_ds has valid specified fileds.
				3. For all output datasets (Viz output_ds),it is assumend that
					a. it has valid name.
					b. Macro has necessary required rights to create the datasets with specified name @ specified location.
					c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.
***************************************************************************/
options mlogic mprint;

%macro csbmva_scrshftind(input_ds=,output_ds=);


/*Start: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for score shift index calculation */
%local input_col_act_tmp input_col_dev_tmp cnt_stab_col;

%let cnt_stab_col=0;
proc sql noprint;
select count(*) into :cnt_stab_col from dictionary.columns 
where  strip(kupcase(memname))=%kupcase("&input_ds") and kupcase(name)='NO_OF_RECORDS_ACTUAL_ALL';
quit;
%let cnt_stab_col=&cnt_stab_col;

%if &cnt_stab_col gt 0 %then %do; /*Additional condition for hot_fix related to score shift index calculation*/
	%let input_col_act_tmp = NO_OF_RECORDS_ACTUAL_ALL;
	%let input_col_dev_tmp = NO_OF_RECORDS_DEV;
%end;
%else %do;
	%let input_col_act_tmp = NO_OF_RECORDS_ACTUAL;
	%let input_col_dev_tmp = NO_OF_RECORDS_DEV;
%end;

/*End: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for SSI calculation */


data work.totals;
	set &input_ds end=last;

	prop_act_tot + &input_col_act_tmp.;
	prop_dev_tot + &input_col_dev_tmp.;

	prop_evt_act_tot + NO_OF_DEFAULTS_ACTUAL;
	prop_evt_dev_tot + NO_OF_DEFAULTS_DEV;

	if last then do;
	 	call symput('PAT',prop_act_tot);
	 	call symput('PDT',prop_dev_tot);
	 	call symput('PEAT',prop_evt_act_tot);
	 	call symput('PEDT',prop_evt_dev_tot);	
	end;
run;

data &output_ds;
	set work.totals (drop=prop_act_tot prop_dev_tot prop_evt_act_tot prop_evt_dev_tot);
	
	
	format scorecard_points &CS_DEC_FMT. prop_actual &CS_DEC_FMT. prop_dev &CS_DEC_FMT. 
			prop_evt_act &CS_DEC_FMT. prop_evt_dev &CS_DEC_FMT. score_shift_index &CS_DEC_FMT. 
			event_shift_index &CS_DEC_FMT.;
	prop_actual=&input_col_act_tmp./&PAT;
	prop_dev=&input_col_dev_tmp./&PDT;
	prop_evt_act=NO_OF_DEFAULTS_ACTUAL/&PEAT;
	prop_evt_dev=NO_OF_DEFAULTS_DEV/&PEDT;

	score_shift_index = ((prop_actual*100)  - (prop_dev*100))    * scorecard_points;
	event_shift_index = ((prop_evt_act*100) - (prop_evt_dev*100))* scorecard_points;
run;
data &output_ds;
set &output_ds(keep =ATTRIBUTE_NAME
SEQ_NO
SCORECARD_POINTS
prop_actual
prop_dev
prop_evt_act
prop_evt_dev
score_shift_index
event_shift_index);
run;


%mend csbmva_scrshftind;
