/*
Macro Name	: csbmva_sysstbindx
Function	: Generates data for system stability index (SSI)
Created by	: BIS Team
Date		: Aug , 2010
SAS Version	: 9.2

Called-by	:SSI screen  
			 
Calls		: None 

Parameters	: 

input_ds		: Name of the input data set
output_ds	: Name of the data set which stores data for graph graph of Observed Default rate estimated PD.

Input Dataset(s) Format:

For PD 
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
estimated_PD	 				Number		Default Formatting
Max_Score_Points	 	Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------


For CCF
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
actual_ccf	 				Number		Default Formatting
ESTIMATED_ccf            Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------
For LGD
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
actual_lgd	 				Number		Default Formatting
ESTIMATED_LGD            Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds
-------------------------------------------------------------------------
Name					Type		Format
------------------------------------------------------------------------
Pool_Name				Character 	Default Formatting	
Pool_Seq_no	 			Number		Default Formatting
max_Score_Points*	 		Number		Default Formatting
Per_Of_record_Actual		Numeric		percent7.3
Per_of_record_Dev 			Numeric		percent7.3
Cum_Per_Of_record_Actual	Numeric		percent7.3
Cum_Per_of_record_Dev		Numeric		percent7.3
Diff_AD					Numeric		percent7.3
Ratio_AD				Numeric		7.3
ln_Ratio_AD				Numeric		7.3
sysstbindx					Numeric		7.3
------------------------------------------------------------------------
* Score_points will only appear in case of PD and Not for LGD and CCF
Logic:
1. First cumulate the number of recordsounts in development and no recordsounts in actual sample
2. Compute cumulative percents for the same. 
3. Compute the difference between Actual_cumulative percentage and Development_cumulative percentage
4. Compute natual log for the same
5. SSI for a pool = Diff * Natual log.
6. Sum of SSI per pool will give you SSI for all pooling scheme.


Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/

%macro csbmva_sysstbindx(input_ds=,output_ds=,flag=);


%if &flag = 1 %then %do;
data _NULL_;

name=put(time(),mmss8.1);

name1=kscan(name,1,":.");

name2=kscan(name,2,":.");

name3=kscan(name,3,":.");

name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));

call symput('log_name',"&LOG_PATH_CS"||"&sysmacroname"||ktrim(kleft(name4))||".log");

run;


%end;

/*Start: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for score shift index calculation */
%local input_col_act_tmp input_col_dev_tmp cnt_stab_col;

%let cnt_stab_col=0;
proc sql noprint;
select count(*) into :cnt_stab_col from dictionary.columns 
where strip(kupcase(libname)) || "." || strip(kupcase(memname))=%kupcase("&input_ds") and kupcase(name)='NO_OF_RECORDS_ACTUAL_ALL';
quit;
%let cnt_stab_col=&cnt_stab_col;

%if &cnt_stab_col gt 0 %then %do; /*Additional condition for hot_fix related to score shift index calculation*/
	%let input_col_act_tmp = NO_OF_RECORDS_ACTUAL_ALL;
	%let input_col_dev_tmp = NO_OF_RECORDS_DEV;
%end;
%else %do;
	%let input_col_act_tmp = NO_OF_RECORDS_ACTUAL;
	%let input_col_dev_tmp = NO_OF_RECORDS_DEV;
%end;

/*End: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for SSI calculation */


%let err_code=;
data &output_ds;
	set &input_ds;
	retain 
		   Cum_A .
		   Cum_D .;	
	cum_A = Sum(cum_A, &input_col_act_tmp.);
	cum_D = Sum (cum_D, &input_col_dev_tmp.);
run;

proc sql noprint;
	select max(Cum_A), max(Cum_D) into :Tot_Act, :Tot_dev from &output_ds;
quit;

%if %sysevalf(&Tot_act eq 0) %then %goto Error_Handler; 




data &output_ds;
set &output_ds;
	
	Per_Of_records_Actual = (&input_col_act_tmp./&Tot_Act)*100;
	Per_of_records_Dev  = (&input_col_dev_tmp./&Tot_Dev)*100;


    Cum_Per_Of_records_Actual = (Cum_A/&Tot_Act)*100;
	Cum_Per_of_records_Dev  = (Cum_D/&Tot_Dev)*100;	


	Diff_AD = Per_Of_records_Actual - Per_of_records_Dev;

	if Per_of_records_Dev > 0 and Per_Of_records_Actual > 0 then do;
	  Ratio_AD = Per_Of_records_Actual/Per_of_records_Dev;
	  ln_Ratio_AD = log (Ratio_AD);
    end;
	else if Per_of_records_Dev ne 0 and Per_Of_records_Actual =0 then do;
	  Ratio_AD = Per_Of_records_Actual/Per_of_records_Dev;
      ln_Ratio_AD =.;
	end;
    else  do;
       Ratio_AD =.;
       ln_ratio_ad =.;
	end;
	sysstbindx = Diff_AD * ln_Ratio_AD ;

run;
%csbmva_errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;

data &output_ds;
     set &output_ds (keep = POOL_NAME POOL_SEQ_NO LEVEL_SK Per_Of_records_Actual Per_of_records_Dev
     Cum_Per_Of_records_Actual Cum_Per_Of_records_dev Diff_AD Ratio_AD ln_Ratio_AD sysstbindx);
run;
%return;

%Error_Handler:
%put WARNING: &err_code;
%csbmva_defdata(input_ds=&input_ds,stat='sysstbindx',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);


%mend csbmva_sysstbindx;
 
