/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_ui_get_feed_mip.sas
Function       : This macro creates the output dataset for displaying feed for 
				 MIP reports

Authors        :  CSB Team
SAS            :  9.3

Called-by      :  Mid Tier
Calls          :  None

Input Datasets :  _<MODEL_RK>_<SCORE_TIME_SK>_PD_SRANGE_FD
				  _<MODEL_RK>_<SCORE_TIME_SK>_PD_ATTR_FD
				   
Output Datasets:  Dataset for displaying feed

Logic          :  creates the output dataset displaying feed

Parameters     :  model_key  	  	 -> Key of the model
                  score_time_sk      -> Score time point Key
                  measure_cd    	 -> Code for the measure
                  output_ds			 -> Output dataset

**********************************************************************************************/

%macro csbmva_ui_get_feed_mip(model_key = ,score_time_sk = ,
							  measure_cd = ,output_ds = );

%let model_key = &model_key;
%let score_time_sk = &score_time_sk;
%let measure_cd = %lowcase(&measure_cd);

*==================================================;
* Defining local macro variables ;
*==================================================;

%local feed_ds analytical_model_key;

*==================================================;
* Getting the model_key from ANALYTICAL_MODEL_DIM ;
*==================================================;

proc sql noprint;
  select b.model_rk into :analytical_model_key
  from &csb_apdm_libref..model_master_extension a, dim.analytical_model_dim b
  where a.model_key = b.source_model_master_key
  and a.model_key = &model_key;
quit;
%let analytical_model_key = &analytical_model_key;

*==================================================;
* Generating feed dataset name ;
*==================================================;

%if &measure_cd = attribprps or &measure_cd = attribevnt %then %do;
	%let feed_ds = csbfact._&analytical_model_key._&score_time_sk._pd_srange_fd;
%end;
%else %do;
	%let feed_ds = csbfact._&analytical_model_key._&score_time_sk._pd_attr_fd;
%end;

*==================================================;
* Creating output dataset ;
*==================================================;
%local input_col_act_tmp input_col_dev_tmp cnt_stab_col;

%let cnt_stab_col=0;
proc sql noprint;
select count(*) into :cnt_stab_col from dictionary.columns 
where strip(kupcase(libname)) || "." || strip(kupcase(memname))=%kupcase("&feed_ds.") and kupcase(name)='NO_OF_RECORDS_ACTUAL_ALL';
quit;
%let cnt_stab_col=&cnt_stab_col;

%if &cnt_stab_col gt 0 %then %do; /*Additional condition for hot_fix related to score shift index calculation*/
	data &output_ds;
	set &feed_ds. 
	%if "&measure_cd" eq "wtgscr"  or "&measure_cd" eq "scrshftind" or "&measure_cd" eq "attribprps" or "&measure_cd" eq"varstbindx" %then %do;
		(drop= NO_OF_RECORDS_ACTUAL )
	%end;
	%else %do;
		(drop= NO_OF_RECORDS_ACTUAL_ALL )
	%end;
	;
	
	%if "&measure_cd" eq "wtgscr"  or "&measure_cd" eq "scrshftind" or "&measure_cd" eq "attribprps" or "&measure_cd" eq"varstbindx" %then %do;
		rename NO_OF_RECORDS_ACTUAL_ALL=NO_OF_RECORDS_ACTUAL;
	%end;
	run;
   
%end;
%else %do;
	proc sql;
	  create table &output_ds as
	    select * from &feed_ds.;
	quit;

	
%end;
%mend csbmva_ui_get_feed_mip;
