/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_update_pd_actuals.sas
Function       : This macro updates individual _<MODEL_SK>_<SCORE_TIME_SK>_PD_FACT
                 from taking the values from PD_ACTUALS table.

Authors        :  CSB Team
Date           :  17AUG2010
SAS            :  9.2
Called-by      :  UPDATE_PD_FACT job
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  PD_ACTUALS (CSB Scratch library)
                  _<MODEL_SK>_<SCORE_TIME_SK>_PD_FACT (CSB Fact library)

Output Datasets:  _<MODEL_SK>_<SCORE_TIME_SK>_PD_FACT (CSB Fact library)

Logic          :  1. Take the distinct combination of MODEL_SK & SCORE_TIME_SK
                     in a dataset.
                  2. For each combination of MODEL_SK & SCORE_TIME_SK, update 
                     the ACTUALS in corresponding fact table.

Parameters     :  INDATA  -> name of input dataset containing the ACTUALS value
                             (with libref)
                  FACTLIB -> name of the library where PD fact tables are present
**********************************************************************************************/

%macro csbmva_update_pd_actuals(indata = ,factlib = );

%if %sysfunc(exist(&indata)) eq 0 %then %do;
  %return;
%end;

proc sql;
  create table csbscr.pd_updatetime_list as
  	select distinct model_rk, score_time_sk
	from &indata;
quit;

%let dsid=%sysfunc(open(csbscr.pd_updatetime_list));
%let totalobs=%sysfunc(attrn(&dsid,nobs));
%let rc=%sysfunc(close(&dsid));

%do i = 1 %to &totalobs;

data _null_;
  pointer = &i;
  set csbscr.pd_updatetime_list point = pointer;

  call symput('m_model_key',model_rk);
  call symput('m_time_key',score_time_sk);
  stop;
run;

%let m_model_key = &m_model_key;
%let m_time_key = &m_time_key;

data &factlib.._&m_model_key._&m_time_key._pd_fact;
  set &indata(where = (model_rk = &m_model_key and score_time_sk = &m_time_key)
              rename = (actual_good_bad = t_actual_good_bad)
			  idxname = idx_model_scoretime
             );
  modify &factlib.._&m_model_key._&m_time_key._pd_fact key = xpk_pd_fact / unique;

  if t_actual_good_bad not in ( 1 . ) then t_actual_good_bad = 0;

  actual_good_bad = t_actual_good_bad;

  if _iorc_ = %sysrc(_SOK) then replace;

  _iorc_ = 0;
  _error_ = 0;

run;

%bankfdn_rcset(&syserr);

%end;

Proc append base=csbscr.pd_updatetime_list data=csbscr.Pd_model_scoretime_list(drop=model_type_cd);
run;

%bankfdn_rcset(&syserr);

proc sort data=csbscr.pd_updatetime_list nodup;
by _all_;
run;

%bankfdn_rcset(&syserr);

%mend csbmva_update_pd_actuals;
