/*****************************Comment added by sinsin on 13Apr2011********************************* 
	This macro will create a copy of APDM data required for building the ABT, in a seperate area
    (staging area) and from now onwards, only staging area will be accessed for building the ABT. 
***************************************************************************************************/

%macro dabt_create_stg_area(m_abt_sk=, m_abt_type=)/ des='dabt_create_stg_area';

	%let m_abt_sk = &m_abt_sk;
	%let m_abt_type = &m_abt_type;

	/* This Macro is used for creating staging tables required for building 
		modeling, backtesting, scoring and actual ABT.*/
	%let BEH_VAR_TYPE = BEH;
	%let RNT_VAR_TYPE = RNT;
	%let SPM_VAR_TYPE = SPM;
	%let DER_VAR_TYPE = DER;
	%let EXT_VAR_TYPE = EXT; /*sinsin 15 May 2013:Added new variable type for External variables*/
	%let SPL_DER_VAR_TYPE = SPL;
	%let CPX_DER_VAR_TYPE = CPX;
	
	/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
	/* Setting the default value as N, will be replaced by the actual flg value */
	%let OC_BSD_PST_ABT_IMPL_SBST_FLG = N;
	/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

	/* Deleting the staging datasets, if any, that were created during the last build
		of the abt */

	/*****************************Comment added by sinsin on 13Apr2011*********************************
	    This code for deleting the temporary datasets can be removed as the previous macro has already 
	    deleted them.
	***************************************************************************************************/

	proc datasets library=&m_stg_lib kill nodetails nolist;
	run;
	quit;
	%dabt_err_chk(type=DATA);

	/* Obtain the list of libraries and add a library for the current ABT. */

	/*****************************Comment added by sinsin on 13Apr2011*********************************
	The staging area will contain a copy of library master and one extra record for current ABT library
    ***************************************************************************************************/

	proc sql noprint;
		create table &m_stg_lib..library_master as
		select library_sk, library_reference, libname_statement as libname_statement, 
			case when kupcase(strip(library_type_cd)) IN ('SRC_TBL','SRC_IM') 
			then 'SRC' 
			else library_type_cd 
			end as library_type_cd 
		from &m_src_lib..library_master;
		%dabt_err_chk(type=SQL);
	quit;


	/* Depending on the type of abt built (modeling, scoring or actual), the temporary
		dataset is created which contains the abt information from the respective area
		of the apdm */

	/* If the abt that is built is scoring abt, then the following code will get executed*/
	%if &m_abt_type = SCR %then %do;
	
		%let m_abt_type_first = s;

		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
		proc sql noprint;
			select apply_oc_bsd_pst_impl_sbst_flg into :OC_BSD_PST_ABT_IMPL_SBST_FLG
			from &m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr 
				inner join &m_src_lib..scoring_model as sm
				on (scr.scoring_template_sk = sm.scoring_template_sk)
				inner join &m_src_lib..model_master mm
				on (sm.model_sk = mm.model_sk);
		quit;
		%dabt_err_chk(type=SQL);
		%let OC_BSD_PST_ABT_IMPL_SBST_FLG = %kupcase(&OC_BSD_PST_ABT_IMPL_SBST_FLG);
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

		/* This table will be later used in the common code, to get the abt and its variable information.
			Hence the structure of this table should be same in all types of ABT Build. */
		proc sql noprint;
			create table &m_tmp_lib..var_req_info_&m_abt_sk as
			select scr.scoring_template_sk as abt_sk,mm.event_sk as event_sk,
					mv.variable_sk, 'Y' as part_of_abt_flg, 'N' as outcome_variable_flg,
					mv.implicit_variable_flg as implicit_variable_flg
			from &m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr 
				inner join &m_src_lib..scoring_model as sm
				on (scr.scoring_template_sk = sm.scoring_template_sk)
				inner join &m_src_lib..model_master mm
				on (sm.model_sk = mm.model_sk)
				inner join &m_src_lib..model_x_scr_input_variable as mv
				on (mm.model_sk = mv.model_sk);
		quit;
		%dabt_err_chk(type=SQL);

		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
		%if &OC_BSD_PST_ABT_IMPL_SBST_FLG eq Y %then %do;
			proc sql noprint;
				create table &m_tmp_lib..abt_info_out_&m_abt_sk as
				select scr.scoring_template_sk as abt_sk,
				mm.event_sk as event_sk, mv.variable_sk, 'N' as part_of_abt_flg, 
				mv.target_variable_flg as outcome_variable_flg,
				mv.implicit_variable_flg as implicit_variable_flg
				from 
				&m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr 
				inner join &m_src_lib..scoring_model as sm
				on (scr.scoring_template_sk = sm.scoring_template_sk)
				inner join &m_src_lib..model_master mm
				on (sm.model_sk = mm.model_sk)
				inner join &m_src_lib..model_x_act_outcome_var as mv
				on (mm.model_sk = mv.model_sk and upcase(mv.target_variable_flg) = 'Y');
			quit;
			%dabt_err_chk(type=SQL);

			proc sql noprint;
				create table &m_tmp_lib..abt_info_all_&m_abt_sk as
				select coalesce(t1.abt_sk, t2.abt_sk) as abt_sk,
						coalesce(t1.event_sk, t2.event_sk) as event_sk,
						coalesce(t1.variable_sk, t2.variable_sk) as variable_sk,
						coalesce(t1.part_of_abt_flg, t2.part_of_abt_flg) as part_of_abt_flg,
						coalesce(t1.outcome_variable_flg, t2.outcome_variable_flg) as outcome_variable_flg,
						coalesce(t1.implicit_variable_flg, t2.implicit_variable_flg) as implicit_variable_flg
				from &m_tmp_lib..var_req_info_&m_abt_sk as t1 full join &m_tmp_lib..abt_info_out_&m_abt_sk as t2
				on (t1.variable_sk = t2.variable_sk);
			quit; 
			%dabt_err_chk(type=SQL);

			/* Clear Temporary Tables - Debug check is not done since the table var_req_info is created again in the next sql */
			proc sql noprint;
				drop table &m_tmp_lib..abt_info_out_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..var_req_info_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;

			/* This table will be later used in the common code, to get the abt and its variable information.
				Hence the structure of this table should be same in all types of ABT Build. */
			proc sql noprint;
				create table &m_tmp_lib..var_req_info_&m_abt_sk as
				select * from &m_tmp_lib..abt_info_all_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			run;
			
			/* Clear Temporary Tables */
			%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
				proc sql noprint;
					drop table &m_tmp_lib..abt_info_all_&m_abt_sk;
					%dabt_err_chk(type=SQL);
				quit;
			%end;
		
		%end;
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
		
		/* This table contains the abt information, regarding the abt table name, outcome period, abt grain,
			purpose and level */
		proc sql noprint;
			create table &m_stg_lib..abt_info as
			select distinct scr.scoring_template_sk label="" as var_target_table_sk,
				scr.scoring_template_id label="" as abt_id,
				scr.scoring_abt_table_nm label="" as var_target_table_nm,
				"&DABT_SCORING_ABT_LIBREF." as abt_library_ref len=8, 
				. as target_period_cnt,
				"" as target_period_time_freq_cd,
				"" as target_period_int_date_exp,
				"" as target_period_int_dttm_exp,
				tfm.time_frequency_cd label="" as source_data_time_grain_cd,
				tfm.date_interval_expression label="" as source_data_time_int_date_exp,
				tfm.dttm_interval_expression label="" as source_data_time_int_dttm_exp,
				tfm.time_align label="" as source_data_time_align,
				mm.apply_bld_dt_cap_outcm_var_flg label="" as bld_dt_cap_flg,
				/* Start: Added abt_type to be used in event macro for hot fix:S1109569 */
				"&m_abt_type" as abt_type_cd len=8,  /* i18nOK:Line */
				/* END: Added abt_type to be used in event macro for hot fix:S1109569 */
				mm.level_sk,
				prp.purpose_cd
			from &m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr
			inner join &m_src_lib..scoring_model as sm
			on (scr.scoring_template_sk = sm.scoring_template_sk)
			inner join &m_src_lib..model_master mm
			on (sm.model_sk = mm.model_sk)
			inner join &m_src_lib..level_master as lvl
			on (mm.level_sk = lvl.level_sk)
			inner join &m_src_lib..modeling_abt_master abt
			on (mm.project_sk = abt.project_sk)
			inner join &m_src_lib..project_master pm
			on (mm.project_sk = pm.project_sk)
			inner join &m_src_lib..time_frequency_master tfm
			on (abt.source_data_time_grain_sk = tfm.time_frequency_sk)
			inner join &m_src_lib..purpose_master prp
			on (pm.purpose_sk = prp.purpose_sk);
		quit;
		%dabt_err_chk(type=SQL);
		
		/* the actual staging table build_date_parameter is created in dabt_build_scr_abt macro.
			This build_date_parameter is created here so that the parameter_list_deploy_DABT table
			is populated properly. */
		proc sql noprint;
			create table &m_stg_lib..build_date_parameter as
			select "" as parameter_nm length=32, /* i18nOK:Line */
					. as parameter_value,
					0 as run_no
			from &m_stg_lib..abt_info;
		quit;
		%dabt_err_chk(type=SQL);


	%end;
	/* else if the abt that is built is actual abt, then the following code will get executed*/
	%else %if &m_abt_type = ACT %then %do;
		
		%let m_abt_type_first = a;
		
		proc sql noprint;
			/* This table will be later used in the common code, to get the abt and its variable information.
				Hence the structure of this table should be same in all types of ABT Build. */
			create table &m_tmp_lib..var_req_info_&m_abt_sk as
			select scr.scoring_template_sk as abt_sk, mm.event_sk as event_sk,
					mv.variable_sk, 'Y' as part_of_abt_flg, 'Y' as outcome_variable_flg,
					mv.implicit_variable_flg as implicit_variable_flg
			from &m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr 
			inner join &m_src_lib..scoring_model as sm
			on (scr.scoring_template_sk = sm.scoring_template_sk)
			inner join &m_src_lib..model_master mm
			on (sm.model_sk = mm.model_sk)
			inner join &m_src_lib..model_x_act_outcome_var as mv
			on (mm.model_sk = mv.model_sk);
			%dabt_err_chk(type=SQL);

			create table &m_tmp_lib..abt_info_tmp_&m_abt_sk as
			select scr.scoring_template_sk label="" as var_target_table_sk,
				scr.scoring_template_id label="" as abt_id,
				sm.outcome_period_cnt as target_period_cnt,
				tfm.time_frequency_cd label="" as target_period_time_freq_cd,
				tfm.date_interval_expression label="" as target_period_int_date_exp,
				tfm.dttm_interval_expression label="" as target_period_int_dttm_exp,
				abt.source_data_time_grain_sk,
				mm.apply_bld_dt_cap_outcm_var_flg label="" as bld_dt_cap_flg,
				scr.scoring_abt_table_nm label="" as var_target_table_nm, 
				mm.level_sk,
				prp.purpose_cd
			from &m_src_lib..scoring_template_detail(where = (scoring_template_sk = &m_abt_sk)) as scr 
			inner join &m_src_lib..scoring_model as sm
			on (scr.scoring_template_sk = sm.scoring_template_sk)
			inner join &m_src_lib..model_master mm
			on (sm.model_sk = mm.model_sk)
			inner join &m_src_lib..project_master pm
			on (mm.project_sk = pm.project_sk)
			inner join &m_src_lib..purpose_master prp
			on (pm.purpose_sk = prp.purpose_sk)
			inner join &m_src_lib..modeling_abt_master abt
			on (pm.project_sk = abt.project_sk)
			left join &m_src_lib..time_frequency_master as tfm
			on (sm.outcome_period_time_freq_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);

			/* This table contains the abt information, regarding the abt table name, outcome period, abt grain,
				purpose and level */
			create table &m_stg_lib..abt_info as
			select var_target_table_sk, abt_id, var_target_table_nm,
					"&DABT_ACTUAL_CALC_ABT_LIBREF." as abt_library_ref len=8, 
					target_period_cnt, target_period_time_freq_cd, 
					target_period_int_date_exp, target_period_int_dttm_exp,
					time_frequency_cd label="" as source_data_time_grain_cd,
					date_interval_expression label="" as source_data_time_int_date_exp,
					dttm_interval_expression label="" as source_data_time_int_dttm_exp,
					time_align label="" as source_data_time_align,
					bld_dt_cap_flg,
					/* Start: Added abt_type to be used in event macro for hot fix:S1109569 */
					"&m_abt_type" as abt_type_cd len=8,  /* i18nOK:Line */
					/* END: Added abt_type to be used in event macro for hot fix:S1109569 */
					level_sk,
					purpose_cd
			from &m_tmp_lib..abt_info_tmp_&m_abt_sk as sm
			inner join &m_src_lib..time_frequency_master as tfm
			on (sm.source_data_time_grain_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);
		quit;
		
		/* the actual staging table build_date_parameter is created in dabt_build_act_abt macro
			This build_date_parameter is created here so that the parameter_list_deploy_DABT table
			is populated properly. */
		proc sql noprint;
			create table &m_stg_lib..build_date_parameter as
			select "" as parameter_nm length=32, /* i18nOK:Line */
					. as parameter_value,
					0 as run_no
			from &m_stg_lib..abt_info;
		quit;
		%dabt_err_chk(type=SQL);
		
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..abt_info_tmp_&m_abt_sk;
			quit;
			%dabt_err_chk(type=SQL);
		%end;

	%end;
	/* else if the abt that is built is back testing/pooling abt, then the following code will get executed*/
	/*Start: Preetha: Modified on 11 Feb 2011, to cater to back testing/pooling abt */
	%else %if &m_abt_type = BCK or &m_abt_type = PLG %then %do;
		%local m_bck_plg_bld_dt_cap_flg m_bck_plg_event_sk m_refer_abt_type;
		%if &m_abt_type = BCK %then %do;
			%let DABT_ABT_LIBREF = &DABT_BACKTESTING_ABT_LIBREF.;
			%let m_abt_type_first = b;
			%let m_refer_abt_type = mdl_spec;
			%if %upcase(&DABT_REF_OBJECT_TYPE_FOR_BCKTEST) eq MDL_ABT %then %do;
				%let m_refer_abt_type = mdl_abt;
			%end; 
		%end;
		%else %do;
			%let m_abt_type_first = p;
			%let DABT_ABT_LIBREF = &DABT_POOLING_ABT_LIBREF.;
			%let m_refer_abt_type = mdl_spec;
			%if %upcase(&DABT_REF_OBJECT_TYPE_FOR_POOLING) eq MDL_ABT %then %do;
				%let m_refer_abt_type = mdl_abt;
			%end; 
		%end;

		%let m_bck_plg_bld_dt_cap_flg = Y;
		%let m_bck_plg_event_sk = .;
		%if &m_refer_abt_type eq mdl_abt %then %do;
			proc sql noprint;
				select abt.apply_oc_bsd_pst_impl_sbst_flg, abt.event_sk, abt.apply_bld_dt_cap_outcm_var_flg
						 into :OC_BSD_PST_ABT_IMPL_SBST_FLG, :m_bck_plg_event_sk, :m_bck_plg_bld_dt_cap_flg
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) mm
					inner join &m_src_lib..model_x_modeling_abt mma
					on (mm.model_sk = mma.model_sk)
					inner join &m_src_lib..modeling_abt_master abt
					on (mma.abt_sk = abt.abt_sk);
			quit;
			%dabt_err_chk(type=SQL);
		%end;
		%else %do;
			proc sql noprint;
				select apply_oc_bsd_pst_impl_sbst_flg, event_sk, apply_bld_dt_cap_outcm_var_flg
						 into :OC_BSD_PST_ABT_IMPL_SBST_FLG, :m_bck_plg_event_sk, :m_bck_plg_bld_dt_cap_flg
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk));
			quit;
			%dabt_err_chk(type=SQL);
		%end;
		
		%let OC_BSD_PST_ABT_IMPL_SBST_FLG = %kupcase(&OC_BSD_PST_ABT_IMPL_SBST_FLG);
		%let m_bck_plg_bld_dt_cap_flg = &m_bck_plg_bld_dt_cap_flg;
		%let m_bck_plg_event_sk = &m_bck_plg_event_sk;
		
		proc sql noprint;
			create table &m_tmp_lib..var_req_info1_&m_abt_sk as
			select mma.abt_sk as abt_sk, &m_bck_plg_event_sk as event_sk,
					mv.variable_sk, 'Y' as part_of_abt_flg, 'Y' as outcome_variable_flg,
					mv.implicit_variable_flg as implicit_variable_flg
			from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
			inner join &m_src_lib..model_x_modeling_abt mma
			on (mm.model_sk = mma.model_sk)
			inner join &m_src_lib..model_x_act_outcome_var as mv
			on (mm.model_sk = mv.model_sk)
			
			union 
			
			select mma.abt_sk as abt_sk, &m_bck_plg_event_sk as event_sk,
					mv.variable_sk, 'Y' as part_of_abt_flg, 'N' as outcome_variable_flg,
					mv.implicit_variable_flg as implicit_variable_flg
			from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
			inner join &m_src_lib..model_x_modeling_abt mma
			on (mm.model_sk = mma.model_sk)
			inner join &m_src_lib..model_x_scr_input_variable as mv
			on (mm.model_sk = mv.model_sk);
		quit;
		%dabt_err_chk(type=SQL);
		
		proc sql noprint;
		/* This table will be later used in the common code, to get the abt and its variable information.
				Hence the structure of this table should be same in all types of ABT Build. */
			create table &m_tmp_lib..var_req_info_&m_abt_sk as
			select abt_sk, event_sk, variable_sk, part_of_abt_flg, outcome_variable_flg, implicit_variable_flg
			from &m_tmp_lib..var_req_info1_&m_abt_sk
			
			union 
			
			/* To have implicit variable (specifically for LGD, variables like EAD) */
			select mma.abt_sk as abt_sk, &m_bck_plg_event_sk as event_sk,
					mv.variable_sk, 'Y' as part_of_abt_flg, mv.outcome_variable_flg as outcome_variable_flg,
					mv.implicit_variable_flg as implicit_variable_flg
			from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
			inner join &m_src_lib..model_x_modeling_abt mma
			on (mm.model_sk = mma.model_sk)
			inner join &m_src_lib..modeling_abt_x_variable as mv
			on (mma.abt_sk = mv.abt_sk)
			where mv.implicit_variable_flg = 'Y' and 
				mv.variable_sk not in (select t1.variable_sk from &m_tmp_lib..var_req_info1_&m_abt_sk t1)
			;
		quit;
		%dabt_err_chk(type=SQL);
		
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..var_req_info1_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
		
		proc sql noprint;
			create table &m_tmp_lib..abt_info_tmp_&m_abt_sk as
			select abt.abt_sk label="" as var_target_table_sk, 
				mm.model_id label="" as abt_id,
				abt.target_period_cnt label="" as target_period_cnt,
				tfm.time_frequency_cd label="" as target_period_time_freq_cd,
				tfm.date_interval_expression label="" as target_period_int_date_exp,
				tfm.dttm_interval_expression label="" as target_period_int_dttm_exp,
				abt.source_data_time_grain_sk,
				"&m_bck_plg_bld_dt_cap_flg" as bld_dt_cap_flg,
				abt.abt_table_nm label="" as var_target_table_nm, 
				abt.level_sk,
				pm.purpose_cd
			from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
			inner join &m_src_lib..model_x_modeling_abt as mma
			on (mm.model_sk = mma.model_sk)
			inner join &m_src_lib..modeling_abt_master as abt
			on (mma.abt_sk = abt.abt_sk)
			inner join &m_src_lib..project_master prj
			on (abt.project_sk = prj.project_sk)
			inner join &m_src_lib..purpose_master pm
			on (prj.purpose_sk = pm.purpose_sk)
			left join &m_src_lib..time_frequency_master as tfm
			on (abt.target_period_time_freq_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);

			/* This table contains the abt information, regarding the abt table name, outcome period, abt grain,
				purpose and level */
			create table &m_stg_lib..abt_info as
			select sm.var_target_table_sk,sm.abt_id, sm.var_target_table_nm,
					"&DABT_ABT_LIBREF." as abt_library_ref len=8, 
					sm.target_period_cnt, sm.target_period_time_freq_cd,
					sm.target_period_int_date_exp, sm.target_period_int_dttm_exp,
					tfm.time_frequency_cd label="" as source_data_time_grain_cd,
					tfm.date_interval_expression label="" as source_data_time_int_date_exp,
					tfm.dttm_interval_expression label="" as source_data_time_int_dttm_exp,
					tfm.time_align label="" as source_data_time_align,
					sm.bld_dt_cap_flg,
					/* Start: Added abt_type to be used in event macro for hot fix:S1109569 */
					"&m_abt_type" as abt_type_cd len=8,  /* i18nOK:Line */
					/* END: Added abt_type to be used in event macro for hot fix:S1109569 */
					sm.level_sk,
					sm.purpose_cd
			from &m_tmp_lib..abt_info_tmp_&m_abt_sk as sm
			inner join &m_src_lib..time_frequency_master as tfm
			on (sm.source_data_time_grain_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);
		quit;

		/* the actual staging table build_date_parameter is created in dabt_build_abt macro
			This build_date_parameter is created here so that the parameter_list_deploy_DABT table
			is populated properly. */
		proc sql noprint;
			create table &m_stg_lib..build_date_parameter as
			select "" as parameter_nm length=32, /* i18nOK:Line */
					. as parameter_value,
					0 as run_no
			from &m_stg_lib..abt_info;
		quit;
		%dabt_err_chk(type=SQL);
		
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..abt_info_tmp_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;	/* else if the abt that is built is back testing/pooling abt, then the following code will get executed*/
	/*End: Preetha: Modified on 11 Feb 2011, to cater to back testing/pooling abt */
	%else %if &m_abt_type = CHRSTC %then %do;	
			%let m_abt_type_first = c;
			
			%let m_chrstc_bld_dt_cap_flg = Y;
			%let m_chrstc_event_sk = .;
			proc sql noprint;
				select apply_oc_bsd_pst_impl_sbst_flg, event_sk, apply_bld_dt_cap_outcm_var_flg
						 into :OC_BSD_PST_ABT_IMPL_SBST_FLG, :m_chrstc_event_sk, :m_chrstc_bld_dt_cap_flg
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk));
			quit;
			%dabt_err_chk(type=SQL);
			
			%let OC_BSD_PST_ABT_IMPL_SBST_FLG = %kupcase(&OC_BSD_PST_ABT_IMPL_SBST_FLG);
			%let m_chrstc_bld_dt_cap_flg = &m_chrstc_bld_dt_cap_flg;
			%let m_chrstc_event_sk = &m_chrstc_event_sk;
			
			proc sql noprint;
			/* This table will be later used in the common code, to get the abt and its variable information.
					Hence the structure of this table should be same in all types of ABT Build. */
				create table &m_tmp_lib..var_req_info_&m_abt_sk as
				select mma.abt_sk as abt_sk, &m_chrstc_event_sk as event_sk,
						mv.variable_sk, 'Y' as part_of_abt_flg, 'Y' as outcome_variable_flg,
						mv.implicit_variable_flg as implicit_variable_flg
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
				inner join &m_src_lib..model_x_modeling_abt mma
				on (mm.model_sk = mma.model_sk)
				inner join &m_src_lib..model_x_act_outcome_var as mv
				on (mm.model_sk = mv.model_sk)
				where mv.target_variable_flg = 'Y'
				
				union 
				
				select mma.abt_sk as abt_sk, &m_chrstc_event_sk as event_sk,
						mv.variable_sk, 'Y' as part_of_abt_flg, 'N' as outcome_variable_flg,
						mv.implicit_variable_flg as implicit_variable_flg
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
				inner join &m_src_lib..model_x_modeling_abt mma
				on (mm.model_sk = mma.model_sk)
				inner join &m_src_lib..model_x_scr_input_variable as mv
				on (mm.model_sk = mv.model_sk)
				where mv.significant_variable_flg = 'Y'
				;
			quit;
			%dabt_err_chk(type=SQL);
			
			proc sql noprint;
				create table &m_tmp_lib..abt_info_tmp_&m_abt_sk as
				select abt.abt_sk label="" as var_target_table_sk, 
					mm.model_id label="" as abt_id,
					abt.target_period_cnt label="" as target_period_cnt,
					tfm.time_frequency_cd label="" as target_period_time_freq_cd,
					tfm.date_interval_expression label="" as target_period_int_date_exp,
					tfm.dttm_interval_expression label="" as target_period_int_dttm_exp,
					abt.source_data_time_grain_sk,
					"&m_chrstc_bld_dt_cap_flg" as bld_dt_cap_flg,
					abt.abt_table_nm label="" as var_target_table_nm, 
					abt.level_sk,
					pm.purpose_cd
				from &m_src_lib..model_master(where=(model_sk = &m_abt_sk)) as mm
				inner join &m_src_lib..model_x_modeling_abt as mma
				on (mm.model_sk = mma.model_sk)
				inner join &m_src_lib..modeling_abt_master as abt
				on (mma.abt_sk = abt.abt_sk)
				inner join &m_src_lib..project_master prj
				on (abt.project_sk = prj.project_sk)
				inner join &m_src_lib..purpose_master pm
				on (prj.purpose_sk = pm.purpose_sk)
				left join &m_src_lib..time_frequency_master as tfm
				on (abt.target_period_time_freq_sk = tfm.time_frequency_sk);
				%dabt_err_chk(type=SQL);
	
				/* This table contains the abt information, regarding the abt table name, outcome period, abt grain,
					purpose and level */
				create table &m_stg_lib..abt_info as
				select sm.var_target_table_sk,sm.abt_id, sm.var_target_table_nm,
						"&DABT_MODELING_ABT_LIBREF." as abt_library_ref len=8, 
						sm.target_period_cnt, sm.target_period_time_freq_cd,
						sm.target_period_int_date_exp, sm.target_period_int_dttm_exp,
						tfm.time_frequency_cd label="" as source_data_time_grain_cd,
						tfm.date_interval_expression label="" as source_data_time_int_date_exp,
						tfm.dttm_interval_expression label="" as source_data_time_int_dttm_exp,
						tfm.time_align label="" as source_data_time_align,
						sm.bld_dt_cap_flg,
						/* Start: Added abt_type to be used in event macro for hot fix:S1109569 */
						"&m_abt_type" as abt_type_cd len=8,  /* i18nOK:Line */
						/* END: Added abt_type to be used in event macro for hot fix:S1109569 */
						sm.level_sk,
						sm.purpose_cd
				from &m_tmp_lib..abt_info_tmp_&m_abt_sk as sm
				inner join &m_src_lib..time_frequency_master as tfm
				on (sm.source_data_time_grain_sk = tfm.time_frequency_sk);
				%dabt_err_chk(type=SQL);
			quit;
	
			/* the actual staging table build_date_parameter is created in dabt_build_abt macro
				This build_date_parameter is created here so that the parameter_list_deploy_DABT table
				is populated properly. */
			proc sql noprint;
				create table &m_stg_lib..build_date_parameter as
				select "" as parameter_nm length=32, /* i18nOK:Line */
						. as parameter_value,
						0 as run_no
				from &m_stg_lib..abt_info;
			quit;
			%dabt_err_chk(type=SQL);
			
			/* Clear Temporary Tables */
			%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
				proc sql noprint;
					drop table &m_tmp_lib..abt_info_tmp_&m_abt_sk;
					%dabt_err_chk(type=SQL);
				quit;
			%end;
	%end;	/* else if the abt that is built is for proportion calculation then the following code will get executed*/
	%else %do;
	
  		/*****************************Comment added by sinsin on 13Apr2011********************************* 
	      		else if the abt that is built is modelling abt, then the following code will get executed.
	     	**************************************************************************************************/
		%let m_abt_type_first = m;  
		
		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
		proc sql noprint;
			select apply_oc_bsd_pst_impl_sbst_flg into :OC_BSD_PST_ABT_IMPL_SBST_FLG
			from &m_src_lib..modeling_abt_master(where=(abt_sk = &m_abt_sk));
		quit;
		%dabt_err_chk(type=SQL);
		%let OC_BSD_PST_ABT_IMPL_SBST_FLG = %kupcase(&OC_BSD_PST_ABT_IMPL_SBST_FLG);
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

		proc sql noprint;
			/* This table will be later used in the common code, to get the abt and its variable information.
				Hence the structure of this table should be same in all types of ABT Build. */
			create table &m_tmp_lib..var_req_info_&m_abt_sk as
			select abt.abt_sk as abt_sk, abt.event_sk as event_sk, av.variable_sk,	
					av.part_of_abt_flg, av.outcome_variable_flg

		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
					,av.implicit_variable_flg
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

			from &m_src_lib..modeling_abt_master(where=(abt_sk = &m_abt_sk)) as abt 
			inner join &m_src_lib..modeling_abt_x_variable as av
			on (abt.abt_sk = av.abt_sk);
			%dabt_err_chk(type=SQL);

			create table &m_tmp_lib..abt_info_tmp_&m_abt_sk as
			select abt_sk label="" as var_target_table_sk,
				prj.project_id as abt_id, 
				prj.project_short_nm as project_short_nm,
				lm.level_short_nm as level_short_nm,
				pm.purpose_short_nm as purpose_short_nm,
				target_period_cnt label="" as target_period_cnt,
				tfm.time_frequency_cd label="" as target_period_time_freq_cd,
				date_interval_expression label="" as target_period_int_date_exp,
				dttm_interval_expression label="" as target_period_int_dttm_exp,
				source_data_time_grain_sk,
				abt.apply_bld_dt_cap_outcm_var_flg label="" as bld_dt_cap_flg,
				abt_table_nm label="" as var_target_table_nm, 
				abt.level_sk,
				pm.purpose_cd
			from &m_src_lib..modeling_abt_master(where=(abt_sk = &m_abt_sk)) as abt
			inner join &m_src_lib..project_master prj
			on (abt.project_sk = prj.project_sk)
			inner join &m_src_lib..purpose_master pm
			on (prj.purpose_sk = pm.purpose_sk)
			inner join &m_src_lib..level_master lm
			on (prj.level_sk = lm.level_sk)
			left join &m_src_lib..time_frequency_master as tfm
			on (abt.target_period_time_freq_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);
			
			/*****************************Comment added by sinsin on 13Apr2011********************************* 
			  This will build a dataset in staging area which will contain all the ABT related informations 
			  like abt_sk, time_grain and level.
             **************************************************************************************************/

			/* This table contains the abt information, regarding the abt table name, outcome period, abt grain,
				purpose and level */
			create table &m_stg_lib..abt_info as
			select var_target_table_sk,var_target_table_nm,abt_id,
					project_short_nm, level_short_nm, purpose_short_nm,
					"&DABT_MODELING_ABT_LIBREF." as abt_library_ref len=8, 
					target_period_cnt, target_period_time_freq_cd, 
					target_period_int_date_exp, target_period_int_dttm_exp,
					time_frequency_cd label="" as source_data_time_grain_cd,
					date_interval_expression label="" as source_data_time_int_date_exp,
					dttm_interval_expression label="" as source_data_time_int_dttm_exp,
					time_align label="" as source_data_time_align,
					bld_dt_cap_flg,
					/* Start: Added abt_type to be used in event macro for hot fix:S1109569 */
					"&m_abt_type" as abt_type_cd len=8,  /* i18nOK:Line */
					/* END: Added abt_type to be used in event macro for hot fix:S1109569 */
					level_sk,
					purpose_cd
			from &m_tmp_lib..abt_info_tmp_&m_abt_sk as sm
			inner join &m_src_lib..time_frequency_master as tfm
			on (sm.source_data_time_grain_sk = tfm.time_frequency_sk);
			%dabt_err_chk(type=SQL);
		quit;
		
		/* i18nOK:Begin */

		/*****************************Comment added by sinsin on 13Apr2011********************************* 
		 This will build a dataset containg abt build date and outcome period end date and datetime. For each run of the
		 ABT, there will be four records. Hence, there will be minimum four records in the dataset and can 
		 contain more than four records in case of multiple runs of the ABT (stacked abt). The abt_sk and 
		 run_no will form the primary key for the table.
		 **************************************************************************************************/

		/* the actual staging table build_date_parameter is created in dabt_build_abt macro
			This build_date_parameter is created here so that the parameter_list_deploy_DABT table
			is populated properly. */
		proc sql noprint;
			create table &m_stg_lib..build_date_parameter as
			select "" as parameter_nm length=32, /* i18nOK:Line */
					. as parameter_value,
					0 as run_no
			from &m_stg_lib..abt_info;
		quit;
		%dabt_err_chk(type=SQL);

		/* i18nOK:End */

		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..abt_info_tmp_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;
	
	/*Start: Preetha 14thOct2013: Added code to consider even those variables that are not part of abt, 
		but are required for calculating derived variable. As further down in the code it is assumed, that this
		table abt_info_&m_abt_sk contains the list of all variables required for building the abt. */
	%let max_var_lvl_no = 0;
	proc sql noprint;
		select max(variable_level_no) into :max_var_lvl_no
		from &m_src_lib..derived_variable dv inner join
			&m_tmp_lib..var_req_info_&m_abt_sk as ivs
			on (dv.variable_sk = ivs.variable_sk);
	quit;
	%dabt_err_chk(type=SQL);
	%let max_var_lvl_no = &max_var_lvl_no;

	%if &max_var_lvl_no gt 0 %then %do;
		proc sql noprint;
			create table &m_tmp_lib..all_var_list_&m_abt_sk as
			select msv.abt_sk, msv.variable_sk
			from &m_tmp_lib..var_req_info_&m_abt_sk as msv
			%do un_cnt = 0 %to &max_var_lvl_no;
				union

				select ivs.abt_sk, dev&un_cnt..EXPRESSION_VARIABLE_SK as variable_sk
				from &m_tmp_lib..var_req_info_&m_abt_sk as ivs
					inner join &m_src_lib..DERIVED_VAR_X_EXPRESSION_VAR  as dev0  
				on (ivs.VARIABLE_SK = dev0.DERIVED_VARIABLE_SK ) 
					
				%if &un_cnt gt 0 %then %do;
					%do jn_cnt = 1 %to &un_cnt;
					%let prev = %eval(&jn_cnt - 1);
					inner join 
						&m_src_lib..DERIVED_VAR_X_EXPRESSION_VAR as dev&jn_cnt  
					on 
						(dev&prev..EXPRESSION_VARIABLE_SK = dev&jn_cnt..DERIVED_VARIABLE_SK) 
					%end;
				%end;
			%end;
			;
		quit;
				
		/* Get all the list of all variables with appropriate part of abt flag. */
		proc sql noprint;
			create table &m_tmp_lib..var_part_info_&m_abt_sk as
			select coalesce(vri.abt_sk,avl.abt_sk) as abt_sk, coalesce(vri.variable_sk, avl.variable_sk) as variable_sk,	
					 coalesce(vri.part_of_abt_flg,'N') as part_of_abt_flg,
					 coalesce(vri.outcome_variable_flg, 'N') as outcome_variable_flg,
					 coalesce(vri.implicit_variable_flg, 'N') as implicit_variable_flg
			from &m_tmp_lib..var_req_info_&m_abt_sk vri
				full join &m_tmp_lib..all_var_list_&m_abt_sk avl
				on (vri.abt_sk = avl.abt_sk and vri.variable_sk=avl.variable_sk);
		quit;
		
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..all_var_list_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
					
	%end;	
	%else %do;
		/* Get all the list of all variables with appropriate part of abt flag. */
		proc sql noprint;
			create table &m_tmp_lib..var_part_info_&m_abt_sk as
			select abt_sk, variable_sk, part_of_abt_flg, outcome_variable_flg, implicit_variable_flg
			from &m_tmp_lib..var_req_info_&m_abt_sk;
		quit;	
	%end;
		
	proc sql noprint;	
		create table &m_tmp_lib..abt_info_&m_abt_sk as
		select msv.abt_sk as abt_sk, vm.level_sk , msv.variable_sk,	
				av.variable_short_nm, msv.part_of_abt_flg, msv.outcome_variable_flg,
				msv.implicit_variable_flg, vtm.variable_type_cd, vm.variable_column_nm,
				vm.source_table_sk label="" as var_source_table_sk,
				cdtm.column_data_type_cd as variable_column_data_type_cd
		from &m_tmp_lib..var_part_info_&m_abt_sk msv
			inner join &m_src_lib..variable_master as vm
			on (msv.variable_sk = vm.variable_sk)
			inner join &m_src_lib..modeling_abt_x_variable as av
			on (msv.variable_sk = av.variable_sk)
			inner join &m_src_lib..variable_type_master as vtm
			on (vm.variable_type_sk = vtm.variable_type_sk)
			inner join &m_src_lib..column_data_type_master cdtm
			on (vm.column_data_type_sk = cdtm.column_data_type_sk);
		%dabt_err_chk(type=SQL);
	quit;
	
	/*End: Preetha 14thOct2013: Added code to consider even those variables that are not part of abt, 
	but are required for calculating derived variable.*/
	
	/*Start: Preetha:StarSchemaSupport: added code to get the level key column name from level_master */

	/* Get abt event info. That is check whether the abt is event based or not. */
	%let abt_event_sk = .;
	proc sql noprint;
		select event_sk into :abt_event_sk
		from &m_tmp_lib..var_req_info_&m_abt_sk(obs=1);
	quit;
	%dabt_err_chk(type=SQL);
	%let abt_event_sk = &abt_event_sk;

	%let event_date_nm = ;
	%let event_date_data_type_cd = ;
	/* Get event column name and data type */
	proc sql noprint;
		select em.event_date_column_nm, cdtm.column_data_type_cd
					into :event_date_nm ,  :event_date_data_type_cd
		from &m_src_lib..event_master em 
			inner join &m_src_lib..event_mapping_dtl as emd
			on (em.event_sk = emd.event_sk)
			inner join &m_src_lib..column_data_type_master as cdtm
	 		on (em.event_date_column_data_type_sk = cdtm.column_data_type_sk)
			where em.event_sk = &abt_event_sk.;
	quit;
	%dabt_err_chk(type=SQL);
	%let event_date_nm = &event_date_nm;
	%let event_date_data_type_cd = &event_date_data_type_cd;

	/* For each level get the level code, short name, subset rk/sk table name,
		master table info and avg calc info. */
	proc sql noprint;
		create table &m_tmp_lib..lvl_n_evt_&m_abt_sk as
		select lm.level_sk, lm.level_cd,lm.level_short_nm, lm.master_source_table_sk, lm.date_for_avg_calc_column_sk, lm.subset_rk_table_nm,
				lm.subset_sk_table_nm, "&event_date_nm" as event_date_nm len=32, "&event_date_data_type_cd" as event_date_data_type_cd len=3
		from &m_src_lib..level_master as lm;
	quit;
	%dabt_err_chk(type=SQL);

	/* For all levels get the avg column name and data type and all other level information present in level master table*/
	/* This table will be used later while creating required_columns for getting the level key column names */
	proc sql noprint;
		create table &m_tmp_lib..lvl_mstr_&m_abt_sk as
		select lvlm.level_sk, lvlm.level_short_nm, scm.column_data_type_sk as level_data_type_sk,
				scm.source_column_nm as level_column_nm, lvlm.level_cd, lm.key_type_cd, lvlm.master_source_table_sk,
				(strip(lbm.library_reference) || "." || strip(stm.source_table_nm)) as master_source_table_nm label="" len=70,
				lbm.library_sk as master_source_table_lib_sk label="",
				sdcm.source_column_nm as date_column_nm_for_avg_calc,
				cdtm.column_data_type_cd as avg_calc_data_type_cd, lvlm.event_date_nm, lvlm.event_date_data_type_cd,
				lvlm.subset_rk_table_nm, lvlm.subset_sk_table_nm
		from &m_tmp_lib..lvl_n_evt_&m_abt_sk as lvlm
		inner join &m_src_lib..level_key_column_dtl as lm
		on (lvlm.level_sk = lm.level_sk)
		inner join &m_src_lib..source_column_master as scm
		on (lm.key_column_sk = scm.source_column_sk)
		left join &m_src_lib..source_table_master as stm
		on (lvlm.master_source_table_sk = stm.source_table_sk)
		left join &m_src_lib..library_master as lbm
		on (stm.library_sk = lbm.library_sk)
		left join &m_src_lib..source_column_master as sdcm
		on (lvlm.date_for_avg_calc_column_sk = sdcm.source_column_sk)
		left join &m_src_lib..column_data_type_master as cdtm
		on (sdcm.column_data_type_sk = cdtm.column_data_type_sk)
		order by level_sk;
	quit;
	%dabt_err_chk(type=SQL);

	/* For all levels get the level_key_column_nm as comma separated column name (in case multiple columns are involved in the level) for _RK and _SK */
	data &m_tmp_lib..lvl_mstr_sep_&m_abt_sk;
		set &m_tmp_lib..lvl_mstr_&m_abt_sk;
		by level_sk;
		length level_key_column_nm level_sk_column_nm $200.;
		retain level_key_column_nm level_sk_column_nm;
		keep level_sk level_short_nm level_key_column_nm level_sk_column_nm level_cd date_column_nm_for_avg_calc avg_calc_data_type_cd;
		keep master_source_table_sk master_source_table_nm master_source_table_lib_sk subset_rk_table_nm subset_sk_table_nm event_date_nm event_date_data_type_cd;
		if first.level_sk then do;
			level_key_column_nm = "";
			level_sk_column_nm = "";
		end;
		if kupcase(strip(key_type_cd)) = "RETAIN_KEY" then do;
			level_key_column_nm = catx(', ', level_key_column_nm, level_column_nm);
		end;
		else do;
			level_sk_column_nm = catx(', ', level_sk_column_nm, level_column_nm);
		end;
		if last.level_sk then do;
			output;
		end;
	run;
	%dabt_err_chk(type=DATA);

	/* Clear Temporary Tables */
	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..lvl_n_evt_&m_abt_sk;
			%dabt_err_chk(type=SQL);
		quit;	
	%end;
	/*End: Preetha:StarSchemaSupport: added code to get the level key column name from level_master */

	/* For each variable get the level information, i.e. level code, level short name, level key column name. 
		Also get the two level abt table name (<libref>.<table_nm>)*/
	proc sql noprint;
		create table &m_tmp_lib..all_vars_bld_&m_abt_sk as
		select abt.variable_sk, abt.level_sk, lvl.level_cd, lvl.level_short_nm,
				lvl.level_key_column_nm, abt.variable_type_cd,
				abt.part_of_abt_flg, abt.outcome_variable_flg, abt.variable_column_nm,
				abt.variable_short_nm, abt.var_source_table_sk,
				abt.variable_column_data_type_cd,
				abt.implicit_variable_flg
		from &m_tmp_lib..abt_info_&m_abt_sk as abt
		left join &m_tmp_lib..lvl_mstr_sep_&m_abt_sk as lvl
		on (abt.level_sk = lvl.level_sk);
	quit;
	%dabt_err_chk(type=SQL);

	/* For each variable get the variable information, i.e. variable name, short name, column data type,
		source table from which the variable is to be calculated. */
	proc sql noprint;
		
		/*****************************Comment added by sinsin on 13Apr2011*********************************
		  This will build a dataset in the staging area which will contain information about all the 
		  constituting variables of the abt.
		**************************************************************************************************/

		create table &m_stg_lib..variable_list_all as
		select 	variable_sk label="" as variable_sk, 
				variable_column_nm label="" as derived_var_nm,
				variable_short_nm label="" as derived_var_lbl,
				part_of_abt_flg label="" as part_of_abt_flg,
				outcome_variable_flg label="" as outcome_variable_flg,
				level_sk, level_cd, level_key_column_nm,
				variable_type_cd, implicit_variable_flg
		from &m_tmp_lib..all_vars_bld_&m_abt_sk;
		%dabt_err_chk(type=SQL);

		/* For each variable, get the selection criteria information. This table won't contain any records,
			if no selection criteria is applied in defining any of the abt variables, in the given abt.*/
		create table &m_tmp_lib..var_info_bld_&m_abt_sk as
		select av.variable_sk label="", am.source_column_nm label="" as dim_column_nm,
				avm.dim_attribute_value label="",
				am.source_column_short_nm label="" as dim_attribute_short_nm,
				am.source_column_sk label="" as dim_attribute_sk,
				av.variable_type_cd label="", av.variable_column_data_type_cd label="",
				av.part_of_abt_flg label="", av.outcome_variable_flg label="",
				level_sk, level_cd
		from &m_tmp_lib..all_vars_bld_&m_abt_sk as av
		inner join &m_src_lib..variable_dim_attribute_filter as vf
		on (av.variable_sk = vf.variable_sk)
		inner join &m_src_lib..source_dim_attrib_value_master as avm
		on (vf.dim_attribute_value_sk = avm.dim_attribute_value_sk)
		inner join &m_src_lib..source_column_master as am
		on (avm.source_column_sk = am.source_column_sk) 
		order by av.variable_sk;
		%dabt_err_chk(type=SQL);
	quit;
	
	/* For each variable get the source table information, from whcih it is to be calcualted.
		In case of derived variables, the source table sk is blank, hence left join is performed. */
	proc sql noprint;
		create table &m_tmp_lib..var_info_tmp_&m_abt_sk as
		select av.variable_sk label="" as variable_sk,
			av.level_cd label="" as abt_lvl_cd,
			av.level_sk label="" as abt_lvl_sk,
			av.level_key_column_nm label="" as abt_lvl_key,
			av.level_short_nm label="" as abt_lvl_short_nm,
			av.variable_column_nm label="" as derived_var_nm,
			av.variable_short_nm label="" as derived_var_lbl,
			av.var_source_table_sk,
			stm.source_table_cd as var_source_table_cd label="",
			stm.source_table_short_nm as var_source_table_short_nm label="",
			av.variable_type_cd label="" as variable_type_cd,
			av.part_of_abt_flg label="" as part_of_abt_flg,
			av.outcome_variable_flg label="" as outcome_variable_flg,
			av.implicit_variable_flg label="" as implicit_variable_flg,
			av.variable_column_data_type_cd label="" as variable_column_data_type_cd
		from &m_tmp_lib..all_vars_bld_&m_abt_sk as av
				left join &m_src_lib..source_table_master as stm
				on (av.var_source_table_sk = stm.source_table_sk);
	quit;
	%dabt_err_chk(type=SQL);

	proc sql noprint;
		/* Get the distinct list of source tables at each level, that will be required for variable calculation */
		create table &m_tmp_lib..var_source_info_tmp_&m_abt_sk as
		select distinct var_source_table_sk, level_sk, level_cd
		from &m_tmp_lib..all_vars_bld_&m_abt_sk as av;
		%dabt_err_chk(type=SQL);

		/* For each source table, obtain the select and alias of level key column, to be passed to IM API.
			Also a column with complete column defifition is created ' <col_def> as <col_nm>', to be used in case of base tables. */
		create table &m_tmp_lib..lvl_mstr_src_col_&m_abt_sk as
		select lvl.var_source_table_sk, lvl.level_sk, lvl.level_cd,
			/* If base source column is same as level column and it is not configured in table_x_level_key_column
						then the select and alias name is same as that of key_column_sk.
						Else the select name is that of corrsp_source_column_sk and alias of key_column_sk as given in table_x_level_key_column */
					coalesce(sacm.source_column_nm,slcm.source_column_nm)
					as select_src_level_col_nm len=70,
					coalesce(sacm2.source_column_nm,slcm.source_column_nm) as alias_src_level_col_nm len=40,
					/* This is only used in case of source as physical table/view. Hence IM case is ignored. */
					/* This column select_alias_src_level_col_nm contains the complete column defintion including 'as <col_name>' */
					catx(' as ',catx('.',coalesce(tacm.source_table_nm,stm.source_table_nm),
									coalesce(sacm.source_column_nm,slcm.source_column_nm)),
							calculated alias_src_level_col_nm) as select_alias_src_level_col_nm len=120
		from &m_tmp_lib..var_source_info_tmp_&m_abt_sk as lvl
			inner join &m_src_lib..source_table_master stm
				on (lvl.var_source_table_sk = stm.source_table_sk)
			inner join &m_src_lib..level_key_column_dtl(where=(kupcase(strip(key_type_cd)) = "RETAIN_KEY")) lkcd
				on (lvl.level_sk = lkcd.level_sk)
			inner join &m_src_lib..source_column_master as slcm
				on (lkcd.key_column_sk = slcm.source_column_sk)
			inner join &m_src_lib..source_table_master tbl_assoc
				on (lvl.var_source_table_sk = tbl_assoc.source_table_sk)
			left join &m_src_lib..table_x_level_key_column as tlc
				on (lvl.var_source_table_sk = tlc.source_table_sk and
					lkcd.level_key_column_dtl_sk = tlc.level_key_column_dtl_sk)
			left join &m_src_lib..source_column_master as sacm
				on (tlc.corrsp_source_column_sk = sacm.source_column_sk)
			left join &m_src_lib..source_table_master as tacm
				on (sacm.source_table_sk = tacm.source_table_sk)
			left join &m_src_lib..level_key_column_dtl as lkdt2
				on (lkdt2.level_key_column_dtl_sk = tlc.level_key_column_dtl_sk)
			left join &m_src_lib..source_column_master as sacm2
				on (lkdt2.key_column_sk = sacm2.source_column_sk)
			order by lvl.var_source_table_sk, lvl.level_sk;
			%dabt_err_chk(type=SQL);
	quit;

	/* For a given source table and level, get the select and alias list as comma separated value. */
	data &m_tmp_lib..lvl_mstr_com_src_col_&m_abt_sk;
		set &m_tmp_lib..lvl_mstr_src_col_&m_abt_sk;
		by var_source_table_sk level_sk;
		length select_src_lvl_col_nm alias_src_lvl_col_nm select_alias_src_lvl_col_nm $32767.;
		retain select_src_lvl_col_nm alias_src_lvl_col_nm select_alias_src_lvl_col_nm;
		drop select_src_level_col_nm alias_src_level_col_nm select_alias_src_level_col_nm;
		if first.level_sk then do;
			select_src_lvl_col_nm = "";
			alias_src_lvl_col_nm = "";
			select_alias_src_lvl_col_nm = "";
		end;
		select_alias_src_lvl_col_nm = catx(', ', select_alias_src_lvl_col_nm, select_alias_src_level_col_nm);
		select_src_lvl_col_nm = catx(', ', select_src_lvl_col_nm, select_src_level_col_nm);
		alias_src_lvl_col_nm = catx(', ', alias_src_lvl_col_nm, alias_src_level_col_nm);
		if last.level_sk then do;
			output;
		end;
	run;
	%dabt_err_chk(type=DATA);

	/* Staging area table containing the source table information, for each source at each level, required
		for calculation of abt variable. */
	proc sql noprint;
		create table &m_stg_lib..source_table_info as
		select var_source_table_sk label="", 
			stm.library_sk label="" as source_library_sk,
			(strip(lm.library_reference)||"."||strip(stm.source_table_nm)) as base_source_table_nm label="" len=70,
			av.level_sk, av.level_cd,
			lm.library_type_cd label="" as var_source_type_cd,
			stm.source_table_cd label="" as var_source_table_cd,
			stl.has_retained_key_flg label="" as has_retained_key_flg,
			case when kupcase(strip(stl.has_retained_key_flg)) = 'Y' then catx('',"&IM_SUBSET_TABLE_LIBREF..",lvl.subset_rk_table_nm)
					else catx('',"&IM_SUBSET_TABLE_LIBREF..",lvl.subset_sk_table_nm) end label="" as dummy_subset_table_nm len=100,
			select_src_lvl_col_nm, alias_src_lvl_col_nm, select_alias_src_lvl_col_nm
		from &m_tmp_lib..lvl_mstr_com_src_col_&m_abt_sk as av inner join
		&m_src_lib..source_table_master as stm
		on (av.var_source_table_sk = stm.source_table_sk) inner join
		&m_src_lib..library_master as lm
		on (stm.library_sk = lm.library_sk)
		inner join &m_src_lib..source_table_x_level as stl
		on (av.var_source_table_sk = stl.source_table_sk and av.level_sk = stl.level_sk)
		inner join &m_tmp_lib..lvl_mstr_sep_&m_abt_sk as lvl
		on (av.level_sk = lvl.level_sk);
		%dabt_err_chk(type=SQL);
	quit;

	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..var_source_info_tmp_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..lvl_mstr_com_src_col_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..lvl_mstr_src_col_&m_abt_sk;
			%dabt_err_chk(type=SQL);
		quit;
	%end;

	/* Start: Obtain the valid from and valid to dates for master table. */

	/* Get the list of distinct levels required for building the given ABT. */
	proc sql noprint;
			create table &m_tmp_lib..dstnct_abt_level_&m_abt_sk as
			select distinct coalesce(t1.level_sk,t2.level_sk) as level_sk
			from &m_stg_lib..variable_list_all t1 full join &m_stg_lib..abt_info t2
			 	on (t1.level_sk = t2.level_sk)
			where coalesce(t1.level_sk,t2.level_sk) is not null;
	quit;
	%dabt_err_chk(type=SQL);

	/* For each level, get the master table information for valid_fr and valid_to usage. */
	proc sql noprint;
		create table &m_tmp_lib..mstr_usg_dts_&m_abt_sk as
		select  dlvl.level_sk, 
			catx('.',tbl_assoc.source_table_nm,scm.source_column_nm) as date_column_nm,
			case kupcase(column_usage_cd) when "VALID_FR" then "date1"
			else "date2" end as column_usage, column_data_type_cd
		from 
				&m_tmp_lib..dstnct_abt_level_&m_abt_sk dlvl
				inner join &m_tmp_lib..lvl_mstr_sep_&m_abt_sk lvl_assoc
					on (dlvl.level_sk = lvl_assoc.level_sk)
				inner join &m_src_lib..SOURCE_TABLE_MASTER tbl_assoc
					on (lvl_assoc.MASTER_SOURCE_TABLE_SK = tbl_assoc.SOURCE_TABLE_SK)
				inner join &m_src_lib..source_column_master as scm
					on (tbl_assoc.source_table_sk = scm.source_table_sk)
				inner join &m_src_lib..source_column_x_usage as scxu
					on (scm.source_column_sk = scxu.source_column_sk)
				inner join &m_src_lib..column_usage_master
					(where=(kupcase(strip(column_usage_cd)) in ("VALID_FR","VALID_TO"))) as cum
					on (scxu.column_usage_sk = cum.column_usage_sk)		
				inner join &m_src_lib..column_data_type_master cdtm
					on (scm.column_data_type_sk = cdtm.column_data_type_sk)	
		order by dlvl.level_sk, column_usage, column_data_type_cd, date_column_nm;
	quit;
	%dabt_err_chk(type=SQL);

	%let mstr_vld_dt_cnt = 0;
	proc sql noprint;
		select count(*) into :mstr_vld_dt_cnt
		from &m_tmp_lib..mstr_usg_dts_&m_abt_sk;
	quit;
	%dabt_err_chk(type=SQL);
	%let mstr_vld_dt_cnt= &mstr_vld_dt_cnt;

	%if &mstr_vld_dt_cnt gt 0 %then %do;

		/* If more than one pair of valid from and valid to are configured, then first take transpose for each usage. */
		proc transpose data=&m_tmp_lib..mstr_usg_dts_&m_abt_sk out=&m_tmp_lib..mstr_all_dts_&m_abt_sk(drop=_NAME_) prefix=dabtdate;
			by level_sk column_usage column_data_type_cd;
		 	var date_column_nm;
		run;
		%dabt_err_chk(type=DATA);

		/* Get the number of pair of usage configured for a given source */
		%let date_tot = .;
		proc sql noprint;
			select count(*) into :date_tot from
			dictionary.columns as dt
			where dt.libname = %sysfunc(kupcase("&m_tmp_lib"))
			and dt.memname = %sysfunc(kupcase("mstr_all_dts_&m_abt_sk"))
			and kupcase(dt.name) like %sysfunc(kupcase(("dabtdate_")));
		quit;
		%dabt_err_chk(type=SQL);

		%let date_tot = &date_tot;

		/* Transpose to get valid from and valid to dates in two columns for each source table and each pair configured.  */
		proc transpose data=&m_tmp_lib..mstr_all_dts_&m_abt_sk out=&m_tmp_lib..mstr_vld_dts_&m_abt_sk(drop=_NAME_);
			by level_sk column_data_type_cd;
			 var 
			%do dabt_cnt = 1 %to &date_tot;
				dabtdate&dabt_cnt
			%end;
			;
			 id column_usage;
		run;
		%dabt_err_chk(type=DATA);

		/* Remove records for source tables coming with null values, as only pair was configured.
			This will happen only if for some source more than one pair is configured. */
		data &m_tmp_lib..mstr_vld_tr_&m_abt_sk;
			set &m_tmp_lib..mstr_vld_dts_&m_abt_sk;
			by level_sk;
			length table_where $10000.;
			retain table_where;
			keep level_sk table_where;
			where date1 ne "" and date2 ne "";
			if first.level_sk then do;
				table_where = "";
			end;
			if upcase(column_data_type_cd) = 'DAT' then do;
				table_where = catx(' AND ', table_where, catx(' ','(&DABT_LOAD_USER_INPUT_DATE >= ',date1,'and &DABT_LOAD_USER_INPUT_DATE <=',date2,')' ));
			end;
			else do;
				table_where = catx(' AND ', table_where, catx(' ','(&DABT_LOAD_USER_INPUT_DTTM >= ',date1,'and &DABT_LOAD_USER_INPUT_DTTM <=',date2,')' ));
			end;
			if last.level_sk then do;
				output;
			end;
		run;
		%dabt_err_chk(type=DATA);
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..mstr_all_dts_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..mstr_vld_dts_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;
	%else %do;
		data &m_tmp_lib..mstr_vld_tr_&m_abt_sk;
			set &m_tmp_lib..dstnct_abt_level_&m_abt_sk;
			keep level_sk table_where;
			length table_where $200.;
			table_where = "";
		run;
		%dabt_err_chk(type=DATA);
	%end;
	/* Clear Temporary Tables */
	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..mstr_usg_dts_&m_abt_sk;
			%dabt_err_chk(type=SQL);
		quit;
	%end;

	/* End: Obtain the valid from and valid to dates for master table. */

	/****start of changes for cons var**/

	/*****************************Comment added by sinsin on 13Apr2011********************************* 
	  Logic to find all the levels apart from the main levels, i.e, if the abt level is 1 and 
	  there is any other abt variable which is of level 2, then it will get the level 2 information .
	***************************************************************************************************/

	%let m_abt_level_sk = ;
	proc sql noprint;
		select level_sk into :m_abt_level_sk
		from &m_stg_lib..abt_info;
	quit;
	%dabt_err_chk(type=SQL);

	%let m_abt_level_sk = &m_abt_level_sk;

	/* Get the list of levels on which the variables are created, apart from abt level. */
	%let m_other_lvl_cnt = ;
	proc sql noprint;
		select count(distinct level_sk) into :m_other_lvl_cnt
		from &m_stg_lib..variable_list_all
		where level_sk ne &m_abt_level_sk.;
	quit;
	%dabt_err_chk(type=SQL);


	%let m_other_lvl_cnt = &m_other_lvl_cnt;

	/* If cons level variable exists */
	%if &m_other_lvl_cnt gt 0 %then %do;
		proc sql noprint;
				create table &m_tmp_lib..dstnct_non_abt_level_&m_abt_sk as
				select level_sk
				from &m_tmp_lib..dstnct_abt_level_&m_abt_sk
				where level_sk ne &m_abt_level_sk;
		quit;
		%dabt_err_chk(type=SQL);

		/* Start: get the columns names for mapping table name corresponding to master table and set master table as alias */
		proc sql noprint;
			create table &m_tmp_lib..lvl_map_tbl_sel_col_&m_abt_sk as
			select  dlvl.level_sk, 
					/* If mapping sourc column is same as level column and it is not configured in table_x_level_key_column
						then the select and alias name is same as that of key_column_sk.
						Else the select name is that of corrsp_source_column_sk and alias of key_column_sk as given in table_x_level_key_column */
					catx(' as ', catx('.',tbl_assoc.source_table_nm,coalesce(sacm.source_column_nm,slcm.source_column_nm)),
								coalesce(sacm2.source_column_nm,slcm.source_column_nm)) as select_map_level_col_nm
			from 
					&m_tmp_lib..dstnct_non_abt_level_&m_abt_sk dlvl
					inner join &m_src_lib..LEVEL_ASSOC lvl_assoc
						on (dlvl.level_sk = lvl_assoc.assoc_level_sk and lvl_assoc.level_sk = &m_abt_level_sk)
					inner join &m_src_lib..level_key_column_dtl(where=(kupcase(strip(key_type_cd)) = "RETAIN_KEY")) lkcd
						on (dlvl.level_sk = lkcd.level_sk or lkcd.level_sk = &m_abt_level_sk)
					inner join &m_src_lib..source_column_master as slcm
						on (lkcd.key_column_sk = slcm.source_column_sk)
					inner join &m_src_lib..SOURCE_TABLE_MASTER tbl_assoc
						on (lvl_assoc.MAPPING_SOURCE_TABLE_SK = tbl_assoc.SOURCE_TABLE_SK)
					left join &m_src_lib..table_x_level_key_column as tlc
						on (lvl_assoc.mapping_source_table_sk = tlc.source_table_sk and
							lkcd.level_key_column_dtl_sk = tlc.level_key_column_dtl_sk)
					left join &m_src_lib..source_column_master as sacm
						on (tlc.corrsp_source_column_sk = sacm.source_column_sk)
					left join &m_src_lib..level_key_column_dtl as lkdt2
						on (lkdt2.level_key_column_dtl_sk = tlc.level_key_column_dtl_sk)
					left join &m_src_lib..source_column_master as sacm2
						on (lkdt2.key_column_sk = sacm2.source_column_sk)
					order by dlvl.level_sk;
		quit;
		%dabt_err_chk(type=SQL);

		/* For level, get the comma separated select/alias list (this is required in case there are more than one columns
			in a given level */
		data &m_tmp_lib..lvl_map_com_sel_col_&m_abt_sk;
			set &m_tmp_lib..lvl_map_tbl_sel_col_&m_abt_sk;
			by level_sk;
			length select_map_lvl_col_nm $32767.;
			retain select_map_lvl_col_nm;
			drop select_map_level_col_nm;
			if first.level_sk then do;
				select_map_lvl_col_nm = "";
			end;
			select_map_lvl_col_nm = catx(', ', select_map_lvl_col_nm, select_map_level_col_nm);
			if last.level_sk then do;
				output;
			end;
		run;
		%dabt_err_chk(type=DATA);

		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..lvl_map_tbl_sel_col_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
		/* End: get the columns names for mapping table name corresponding to master table and set master table as alias */
		
		/* Start: Obtain valid from and valid to for mapping table */

		/*Start: Preetha: Modified on 12 Apr 2011 : date1 will now contain the column name for valid_fr
			and date2 for valid_to, this is done to avoid the defect that comes in RDMS,
			while using between operator for the two dates. */

		proc sql noprint;
			create table &m_tmp_lib..map_usg_dts_&m_abt_sk as
			select  dlvl.level_sk, 
				catx('.',tbl_assoc.source_table_nm,scm.source_column_nm) as date_column_nm,
				case kupcase(column_usage_cd) when "VALID_FR" then "date1"
				else "date2" end as column_usage, column_data_type_cd
			from 
					&m_tmp_lib..dstnct_non_abt_level_&m_abt_sk dlvl
					inner join &m_src_lib..LEVEL_ASSOC lvl_assoc
						on (dlvl.level_sk = lvl_assoc.assoc_level_sk and lvl_assoc.level_sk = &m_abt_level_sk)
					inner join &m_src_lib..SOURCE_TABLE_MASTER tbl_assoc
						on (lvl_assoc.MAPPING_SOURCE_TABLE_SK = tbl_assoc.SOURCE_TABLE_SK)
					inner join &m_src_lib..source_column_master as scm
						on (tbl_assoc.source_table_sk = scm.source_table_sk)
					inner join &m_src_lib..source_column_x_usage as scxu
						on (scm.source_column_sk = scxu.source_column_sk)
					inner join &m_src_lib..column_usage_master
						(where=(kupcase(strip(column_usage_cd)) in ("VALID_FR","VALID_TO"))) as cum
						on (scxu.column_usage_sk = cum.column_usage_sk)		
					inner join &m_src_lib..column_data_type_master cdtm
						on (scm.column_data_type_sk = cdtm.column_data_type_sk)	
			order by dlvl.level_sk, column_usage, column_data_type_cd, date_column_nm;
		quit;
		%dabt_err_chk(type=SQL);

		/*End: Preetha: Modified on 12 Apr 2011*/

		%let map_vld_dt_cnt = 0;
		proc sql noprint;
			select count(*) into :map_vld_dt_cnt
			from &m_tmp_lib..map_usg_dts_&m_abt_sk;
		quit;
		%dabt_err_chk(type=SQL);
		%let map_vld_dt_cnt= &map_vld_dt_cnt;

		%if &map_vld_dt_cnt gt 0 %then %do;
			/* If more than one pair of valid from and valid to are configured, then first take transpose for each usage. */
			proc transpose data=&m_tmp_lib..map_usg_dts_&m_abt_sk out=&m_tmp_lib..map_all_dts_&m_abt_sk(drop=_NAME_) prefix=dabtdate;
				by level_sk column_usage column_data_type_cd;
			 	var date_column_nm;
			run;
			%dabt_err_chk(type=DATA);

			/* Get the number of usage pair configured. */
			%let date_tot = .;
			proc sql noprint;
				select count(*) into :date_tot from
				dictionary.columns as dt
				where dt.libname = %sysfunc(kupcase("&m_tmp_lib"))
				and dt.memname = %sysfunc(kupcase("map_all_dts_&m_abt_sk"))
				and kupcase(dt.name) like %sysfunc(kupcase(("dabtdate_")));
			quit;
			%dabt_err_chk(type=SQL);

			%let date_tot = &date_tot;

			/* Transpose to get valid from and valid to dates in two columns for each source table and each pair configured.  */
			proc transpose data=&m_tmp_lib..map_all_dts_&m_abt_sk out=&m_tmp_lib..map_vld_dts_&m_abt_sk(drop=_NAME_);
				by level_sk column_data_type_cd;
				 var 
				%do date_cnt = 1 %to &date_tot;
					dabtdate&date_cnt
				%end;
				;
				 id column_usage;
			run;
			%dabt_err_chk(type=DATA);

			/* Remove records for source tables coming with null values, as only pair was configured.
				This will happen only if for some source more than one pair is configured. */
			data &m_tmp_lib..map_vld_dts_tr_&m_abt_sk;
				set &m_tmp_lib..map_vld_dts_&m_abt_sk;
				by level_sk;
				where date1 ne "" and date2 ne "";
				length table_where $10000.;
				retain table_where;
				keep level_sk table_where;
				if first.level_sk then do;
					table_where = "";
				end;
				if upcase(column_data_type_cd) = 'DAT' then do;
					table_where = catx(' AND ', table_where, catx(' ','(&DABT_LOAD_USER_INPUT_DATE >= ',date1,'and &DABT_LOAD_USER_INPUT_DATE <=',date2,')' ));
				end;
				else do;
					table_where = catx(' AND ', table_where, catx(' ','(&DABT_LOAD_USER_INPUT_DTTM >= ',date1,'and &DABT_LOAD_USER_INPUT_DTTM <=',date2,')' ));
				end;
				if last.level_sk then do;
					output;
				end;
			run;
			%dabt_err_chk(type=DATA);	
		%end;
		%else %do;
			data &m_tmp_lib..map_vld_dts_tr_&m_abt_sk;
				set &m_tmp_lib..dstnct_non_abt_level_&m_abt_sk;
				keep level_sk table_where;
				length table_where $200.;
				table_where = "";
			run;
			%dabt_err_chk(type=DATA);	
		%end;

		/* End: Obtain valid from and valid to for mapping table */
	
		/* Staging are to store the level information for each level required for variable calculation. */
		proc sql noprint;
			create table &m_stg_lib..level_details as
			select 
				dlvl.level_sk as level_sk label="",
				'N' as main_level_flg label="",
				lvl.level_short_nm, LEVEL_KEY_COLUMN_NM len=200, level_sk_column_nm len=200,
				(strip(library_reference)||"."||strip(source_table_nm)) as MAPPING_SOURCE_TABLE_NM label="" len=70,
				 MAPPING_SOURCE_TABLE_SK as mapping_source_table_sk,
				 tbl_assoc.LIBRARY_SK as MAPPING_SOURCE_TABLE_LIB_SK label="",
				 select_map_lvl_col_nm,
				tr_valid_dts.table_where as valid_dts_where_clause label="" len=200,
				mstr_valid_dts.table_where as valid_mstr_dts_where_clause label="" len=200,
				lvl.level_cd,
				lvl.master_source_table_nm, lvl.master_source_table_lib_sk, lvl.event_date_nm, lvl.event_date_data_type_cd 
			from 	
				 	&m_tmp_lib..dstnct_non_abt_level_&m_abt_sk dlvl inner join
					&m_tmp_lib..lvl_mstr_sep_&m_abt_sk lvl
					on (dlvl.level_sk = lvl.level_sk) inner join
					&m_src_lib..LEVEL_ASSOC lvl_assoc
					on (dlvl.level_sk = lvl_assoc.assoc_level_sk and
						lvl_assoc.level_sk = &m_abt_level_sk) inner join
					&m_src_lib..SOURCE_TABLE_MASTER tbl_assoc
					on (lvl_assoc.MAPPING_SOURCE_TABLE_SK = tbl_assoc.SOURCE_TABLE_SK) inner join
					&m_src_lib..LIBRARY_MASTER lib
					on (tbl_assoc.LIBRARY_SK = lib.LIBRARY_SK) left join
					&m_tmp_lib..map_vld_dts_tr_&m_abt_sk tr_valid_dts
					on (tr_valid_dts.level_sk = dlvl.level_sk) left join
					&m_tmp_lib..mstr_vld_tr_&m_abt_sk mstr_valid_dts 
					on (mstr_valid_dts.level_sk = dlvl.level_sk) left join
					&m_tmp_lib..lvl_map_com_sel_col_&m_abt_sk map_sel_col
					on (map_sel_col.level_sk = dlvl.level_sk);
		quit;

		/*****************************Comment added by sinsin on 13Apr2011********************************* 
		        End of logic to find all the levels apart from the main levels.
        ***************************************************************************************************/

		%dabt_err_chk(type=SQL);

		/* Insert record in staging area table containing level information for abt level */
		proc sql noprint;
			insert into &m_stg_lib..level_details
			select dlvl.level_sk, 'Y' as main_level_flg, lvl.level_short_nm,
					LEVEL_KEY_COLUMN_NM, level_sk_column_nm, '' as MAPPING_SOURCE_TABLE_NM,. as MAPPING_SOURCE_TABLE_SK, 
					. as MAPPING_SOURCE_TABLE_LIB_SK, '' as select_map_lvl_col_nm, '' as valid_dts_where_clause,
					mstr_valid_dts.table_where as valid_mstr_dts_where_clause label="" len=200, lvl.level_cd,
					lvl.master_source_table_nm, lvl.master_source_table_lib_sk, lvl.event_date_nm, lvl.event_date_data_type_cd
			from &m_stg_lib..abt_info dlvl inner join 
				 &m_tmp_lib..lvl_mstr_sep_&m_abt_sk lvl
				 on (dlvl.level_sk = lvl.level_sk)left join
				 &m_tmp_lib..mstr_vld_tr_&m_abt_sk mstr_valid_dts
				 on (mstr_valid_dts.level_sk = dlvl.level_sk);
		quit;
		%dabt_err_chk(type=SQL);

		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..dstnct_non_abt_level_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..map_usg_dts_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..map_all_dts_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..map_vld_dts_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..map_vld_dts_tr_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..lvl_map_com_sel_col_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;

	%end;
	%else %do;  /* if m_other_lvl_cnt eq 0, i.e. if no cons level variable exists */
		data &m_stg_lib..level_details;
			attrib level_sk length=8.;
			attrib main_level_flg length=$1.;
			attrib level_short_nm length=$250.;
			attrib LEVEL_KEY_COLUMN_NM length=$200.;
			attrib level_sk_column_nm length=$200.;
			attrib MAPPING_SOURCE_TABLE_NM length=$70.;
			attrib MAPPING_SOURCE_TABLE_SK length=8.;
			attrib MAPPING_SOURCE_TABLE_LIB_SK length=8.;
			attrib select_map_lvl_col_nm length=$32767.;
			attrib valid_dts_where_clause length=$200.;
			attrib valid_mstr_dts_where_clause length=$200.;
			attrib level_cd length=$3.;
			attrib master_source_table_nm  length=$70.;
			attrib master_source_table_lib_sk length=8.;
			attrib event_date_nm length=$32.;
			attrib event_date_data_type_cd length=$3.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
		
		/*****************************Comment added by sinsin on 13Apr2011********************************* 
		  Inserting the level information for the main level,i.e., the level on which the abt is built.
        ***************************************************************************************************/

		/* Insert record in staging area table containing level information for abt level */
		proc sql noprint;
			insert into &m_stg_lib..level_details
			select dlvl.level_sk, 'Y' as main_level_flg, lvl.level_short_nm,
					LEVEL_KEY_COLUMN_NM,level_sk_column_nm, '' as MAPPING_SOURCE_TABLE_NM,. as MAPPING_SOURCE_TABLE_SK, 
					. as MAPPING_SOURCE_TABLE_LIB_SK, '' as select_map_lvl_col_nm, '' as valid_dts_where_clause, 
					mstr_valid_dts.table_where as valid_mstr_dts_where_clause label="" len=200, lvl.level_cd,
					lvl.master_source_table_nm, lvl.master_source_table_lib_sk,	lvl.event_date_nm, lvl.event_date_data_type_cd 
			from &m_stg_lib..abt_info dlvl inner join
				 &m_tmp_lib..lvl_mstr_sep_&m_abt_sk lvl
				 on (dlvl.level_sk = lvl.level_sk) left join
				 &m_tmp_lib..mstr_vld_tr_&m_abt_sk mstr_valid_dts
				 on (mstr_valid_dts.level_sk = dlvl.level_sk);
		quit;
		%dabt_err_chk(type=SQL);
	%end;

	/****end of changes for cons var**/

	/* Clear Temporary Tables */
	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..dstnct_abt_level_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..mstr_vld_tr_&m_abt_sk;
			%dabt_err_chk(type=SQL);
		quit;
	%end;

	/* Macro create staging area table for storing subset information. */
	%let m_sub_whr_tmp_tbl_nm = &m_tmp_lib..subset_where_info_&m_abt_sk.;
	%let m_sub_val_tmp_tbl_nm = &m_tmp_lib..subset_valid_info_&m_abt_sk.;
	%dabt_build_subset_wrapper(m_abt_sk = &m_abt_sk, m_abt_type=&m_abt_type, m_sub_whr_tbl_nm = &m_sub_whr_tmp_tbl_nm., m_sub_val_tbl_nm=&m_sub_val_tmp_tbl_nm.);


	/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

	/*****************************Comment added by sinsin on 13Apr2011****************************************** 
	  This will assure that the OC_BSD_PST_ABT_IMPL_SBST_FLG is applied only on the explicit outcome variables
	************************************************************************************************************/

	%if &OC_BSD_PST_ABT_IMPL_SBST_FLG eq Y %then %do;
		data &m_stg_lib..post_abt_subset_criteria;
			keep post_abt_subset_query_sk where_clause;
			set &m_tmp_lib..var_info_tmp_&m_abt_sk(keep=variable_sk derived_var_nm
														outcome_variable_flg implicit_variable_flg);
			where kupcase(outcome_variable_flg) = "Y" and kupcase(implicit_variable_flg) = "N";
			length where_clause $200.;
			post_abt_subset_query_sk = variable_sk;
			where_clause = strip(derived_var_nm) || ' IN (&OC_BSD_PST_IMPL_SBST_VALUE)';
		run;
		%dabt_err_chk(type=DATA);
	%end;
	%else %do;
		data &m_stg_lib..post_abt_subset_criteria;
			attrib post_abt_subset_query_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
	%end;
	/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

	proc sql noprint;

		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

	/*****************************Comment added by sinsin on 13Apr2011********************************* 
	 Obtain the list of input variables and part of the final abt. If the OC_BSD_PST_ABT_IMPL_SBST_FLG
	 equals Y then the outcome_variable_flg should be "Y" and the implicit_variable_flg should be "N",
	 as we need to calculate the outcome variable as of build abt date and the variable should be 
	 explicitly defined by the user.
	***************************************************************************************************/

		create table &m_tmp_lib..var_info_input_&m_abt_sk as
				select variable_sk
				from &m_tmp_lib..var_info_tmp_&m_abt_sk
				where kupcase(part_of_abt_flg) = "Y" 
					%if &OC_BSD_PST_ABT_IMPL_SBST_FLG eq Y %then %do;
						or (
							kupcase(outcome_variable_flg) = "Y"
							and kupcase(implicit_variable_flg) = "N"
							);
					%end;
					%else %do;
						and kupcase(outcome_variable_flg) ^= "Y";
					%end;
		%dabt_err_chk(type=SQL);
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
	quit;

	proc append base=&m_tmp_lib..var_info_all_dup_&m_abt_sk data=&m_tmp_lib..var_info_input_&m_abt_sk force;
	run;
	%dabt_err_chk(type=DATA);

	%let max_var_lvl_no = 0;
	proc sql noprint;
		select max(variable_level_no) into :max_var_lvl_no
		from &m_src_lib..derived_variable dv inner join
			&m_tmp_lib..var_info_input_&m_abt_sk t2
			on (dv.variable_sk = t2.variable_sk);
	quit;
	%dabt_err_chk(type=SQL);
	%let max_var_lvl_no = &max_var_lvl_no;

	%if &max_var_lvl_no gt 0 %then %do;
		%do i = 0 %to &max_var_lvl_no;
			%if &i eq 0 %then %do;
				%let in_tmp_ds_nm = &m_tmp_lib..var_info_input_&m_abt_sk;
			%end;
			%else %do;
				%let ds_lst = %eval(&i - 1);
				%let in_tmp_ds_nm = &m_tmp_lib..var_info_exp_&ds_lst._&m_abt_sk;
			%end;
			%let out_tmp_ds_nm = &m_tmp_lib..var_info_exp_&i._&m_abt_sk;
			proc sql noprint;
				/* Get the list of variables, that are used in the expression of some derived variable*/
				create table &out_tmp_ds_nm. as
					select expression_variable_sk as variable_sk
					from &in_tmp_ds_nm. as t1
					inner join &m_src_lib..derived_var_x_expression_var as t2
					on (t1.variable_sk = t2.derived_variable_sk);
				%dabt_err_chk(type=SQL);
			quit;

			/* Get the complete list of variables to be calculated. */
			proc append base=&m_tmp_lib..var_info_all_dup_&m_abt_sk data=&out_tmp_ds_nm. force;
			run;
			%dabt_err_chk(type=DATA);

			/* Clear Temporary Tables */
			%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
				proc sql noprint;
					drop table &in_tmp_ds_nm.;
					%dabt_err_chk(type=SQL);
					%if &i eq &max_var_lvl_no %then %do;
						drop table &out_tmp_ds_nm.;
						%dabt_err_chk(type=SQL);
					%end;
				quit;
			%end;
		%end;
	%end;
	%else %do;
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..var_info_input_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;

	/* Complete distinct list of variables that are to be computed as of build abt date.
	   This is independent of whether or not they will be present in final ABT.
	   There might be few variables which are not present in final ABT but used in
	   deriving some final input variable */
	proc sql noprint;
		create table &m_tmp_lib..var_info_all_&m_abt_sk as
		select distinct variable_sk
		from &m_tmp_lib..var_info_all_dup_&m_abt_sk;
	quit;
	%dabt_err_chk(type=SQL);
	
	proc sql noprint;
		/* Get the variable information for the required variables */
		create table &m_tmp_lib..var_info_msr_&m_abt_sk(drop=part_of_abt_flg
											outcome_variable_flg implicit_variable_flg) as
		select t1.*
		from &m_tmp_lib..var_info_tmp_&m_abt_sk as t1 inner join
			&m_tmp_lib..var_info_all_&m_abt_sk as vb
			on (t1.variable_sk = vb.variable_sk);
		%dabt_err_chk(type=SQL);

		/* Get the variable selection criteria information for the required variables */
	/* Note, if none of the variables have any filter on dim attribute,
		then this table won't have any record  */
		create table &m_tmp_lib..var_info_dim_&m_abt_sk(drop=part_of_abt_flg
											outcome_variable_flg) as
		select t1.*
		from &m_tmp_lib..var_info_bld_&m_abt_sk as t1 inner join
			&m_tmp_lib..var_info_all_&m_abt_sk as vb
			on (t1.variable_sk = vb.variable_sk);
		%dabt_err_chk(type=SQL);
	quit;


	/**************************************************************************/
	/******************Changes done by sinsin on 04 July 2011******************/
	/*The changes have been done in order to enhance the performance of the  
	/*processing in the existing DABT application. This will create a SAS 
	/*dataset containing only the columns which are needed to be kept in level4
	/*job. This is done so as to reduce the size of scratch and the sas dataset 
	/*being generated at level4.
	
	/*Defining a dummy dataset in which the column attributes will be inserted.*/

	/* This table will also be referred in dabt_get_required_columns for inserting records */
		data &m_tmp_lib..required_columns;
				attrib var_source_table_sk length=8;
				attrib level_sk length=8;
				attrib outcome_flg length=$1.;
				attrib column_info_type length=$12.;
				attrib keep_column_name length=$32.;
				attrib alias_column_name length=$32.;
				attrib keep_usage_column_name length=$10000.;
				attrib keep_column_data_type length=$3.;
				attrib is_calc_flg length=$1.;
				attrib var_time_sk length=8;
				attrib var_src_type_cd length=$32.;
				stop;
		run;

		%dabt_err_chk(type=DATA);

		/* Macro gets the list of source columns required for abt variable calculation.
			The information regarding where this column will be used is also stored.
			This is for each source at each level and for input/outcome calculation, as required. */
		%dabt_get_required_columns(m_abt_sk=&m_abt_sk., out_ds_nm=&m_tmp_lib..required_columns, 
						in_msr_ds_nm=&m_tmp_lib..var_info_msr_&m_abt_sk,
						in_dim_ds_nm=&m_tmp_lib..var_info_dim_&m_abt_sk,
						in_lvl_ds_nm=&m_tmp_lib..lvl_mstr_&m_abt_sk,
						outcome_flg=N);
	/***********************************************************************/


	/* Begins: Population of variable list tables for Behavioral variables */

	%dabt_create_beh_var_data(in_msr_data=&m_tmp_lib..var_info_msr_&m_abt_sk,
						in_dim_data=&m_tmp_lib..var_info_dim_&m_abt_sk, var_type=beh);

	/* Ends: Population of variable list table for Behavioral variables */

	/* Begins: Population of variable list table for Recent variables */
	%dabt_create_rnt_var_data(in_msr_data=&m_tmp_lib..var_info_msr_&m_abt_sk,
						in_dim_data=&m_tmp_lib..var_info_dim_&m_abt_sk, var_type=rnt);

	/* Ends: Population of variable list table for Recent variables */

	/* Begins: Population of variable list table for Supplementary(Refresh) variables */
	%dabt_create_spm_var_data(in_msr_data=&m_tmp_lib..var_info_msr_&m_abt_sk, var_type=spm);

	/* Begins: sinsin 15 May 2013: Population of variable list table for External variables */
	%dabt_create_ext_var_data(in_msr_data=&m_tmp_lib..var_info_msr_&m_abt_sk, var_type=ext);

	/* Ends: Population of variable list table for External variables */

	%if &m_abt_type ne SCR %then %do;
		/* Begins: Population of variable list table for Outcome variables */
		%dabt_create_outcome_var_data(in_msr_data=&m_tmp_lib..var_info_tmp_&m_abt_sk,
							in_dim_data=&m_tmp_lib..var_info_bld_&m_abt_sk,
							in_lvl_data=&m_tmp_lib..lvl_mstr_&m_abt_sk, /* Added for passing to dabt_get_required_columns macro*/
							var_type=out);
	%end;
	%else %do; /* Creating empty structures just to ensure that the code doesn't fail in dabt */
		data &m_stg_lib..variable_list_out;
			attrib variable_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
		data &m_stg_lib..variable_list_out_beh;
			attrib variable_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
		data &m_stg_lib..variable_list_out_rnt;
			attrib variable_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
		data &m_stg_lib..variable_list_out_spm;
			attrib variable_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
		/*sinsin 15 May 2013: Changes done for external variables requirement*/
		data &m_stg_lib..variable_list_out_ext;
			attrib variable_sk length=8.;
			attrib abt_lvl_sk length=8.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
	%end;

	/* Ends: Population of variable list table for Outcome variables */	

	/* Begins: Population of variable list table for Computed variables */
		/*Start: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */
		/* The input dataset passed is changed to the filtered dataset */
	%dabt_create_der_in_var_data(in_msr_data=&m_tmp_lib..var_info_msr_&m_abt_sk, var_type=der_inp);
		/*End: Preetha: Modified on 15 Dec 2010, to cater to outcome based post abt implicit filter */

	/* Ends: Population of variable list table for Computed variables */
	
	/* Get the information of whether subset or event exists for the given abt. */
	%let sub_cnt = 0;
	%let evt_dt = ;
	proc sql noprint;
		select count(*) into :sub_cnt
		from &m_stg_lib..subset_all_info;
		%dabt_err_chk(type=SQL);

		select event_date_nm into :evt_dt
		from &m_stg_lib..level_details;
		%dabt_err_chk(type=SQL);
	quit;


	%let sub_im_exst = 0;
	/* If subset exists, then get the select column list for all the subsets present in the given ABT. 
		This is to be included in the keep_required_columns table. */
	%if &sub_cnt gt 0 %then %do;

		/* Get the distinct list of subsets present for the given abt. */
		proc sql noprint;
			create table &m_tmp_lib..subset_info_distnct_&m_abt_sk as
			select distinct subset_query_sk
			from &m_stg_lib..subset_all_info;
		quit;
		%dabt_err_chk(type=SQL);

		/* get the distinct list of source tables used in the business user specified where clause. */
		proc sql noprint;
			create table &m_tmp_lib..subset_whr_join_&m_abt_sk. as
			select distinct subset_query_sk, operand_source_table_nm as source_table_nm
			from &m_sub_whr_tmp_tbl_nm
			where is_calc_flg = 'N';
		quit;
		%dabt_err_chk(type=SQL);

		/* get the list of source tables used in the business user specified where clause and in valid from and valid to.
			In case the tables defined in the subset path have VALID_FR and VALID_TO configured. */
		data &m_tmp_lib..subset_table_exp_join_&m_abt_sk.;
			set &m_sub_val_tmp_tbl_nm &m_tmp_lib..subset_whr_join_&m_abt_sk.;
		run;
		%dabt_err_chk(type=DATA);

		/* get the distinct list of source tables used for each subset criteria */
		proc sql noprint;
			create table &m_tmp_lib..subset_table_exp_&m_abt_sk. as
			select distinct subset_query_sk, source_table_nm
			from &m_tmp_lib..subset_table_exp_join_&m_abt_sk.
			order by subset_query_sk;
		run;
		%dabt_err_chk(type=DATA);

		/* Get comma separated table names for each subset. So that the table will contain
			Number of records = total number of subsets defined (implicit/explicit/subject group) */
		data &m_tmp_lib..subset_exp_join_tbl_&m_abt_sk.;
			set &m_tmp_lib..subset_table_exp_&m_abt_sk.;
			by subset_query_sk;
			keep subset_query_sk explicit_join_tables;
			length explicit_join_tables $700.;
			retain explicit_join_tables;
			if first.subset_query_sk then do;
				explicit_join_tables = '';
			end;
			explicit_join_tables = catx(',',explicit_join_tables,source_table_nm);
			if last.subset_query_sk then do;
				output;
			end;
		run;
		%dabt_err_chk(type=DATA);

		/* Get the list of columns that are used in defining business user specified where clause, and which are
			calculated fields. */
		/* This is required, as in case of calcualted column used in where clause, the table involved needs to be added in the from statement. */
		proc sql noprint;
			create table &m_tmp_lib..subset_column_exp_join_&m_abt_sk. as
			select distinct subset_query_sk, operand_source_table_nm, operand_source_column_nm, is_calc_flg
			from &m_sub_whr_tmp_tbl_nm
			where is_calc_flg = 'Y';
		quit;
		%dabt_err_chk(type=SQL);

		/* Get the list of columns as comma separated value for each subset criteria. */
		data &m_tmp_lib..subset_exp_join_col_&m_abt_sk.;
			set &m_tmp_lib..subset_column_exp_join_&m_abt_sk.;
			by subset_query_sk;
			keep subset_query_sk select_column_calc_list;
			length select_column_calc_list $32767.;
			retain  select_column_calc_list;
			if first.subset_query_sk then do;
				select_column_calc_list = '';
			end;
			select_column_calc_list = catx(',',select_column_calc_list,operand_source_column_nm);
			if last.subset_query_sk then do;
				output;
			end;
		run;
		%dabt_err_chk(type=DATA);

		/* Obtain the list of calcualted columns and tables required */
		proc sql noprint;
			create table &m_tmp_lib..subset_explicit_join_&m_abt_sk. as
			select coalesce(t1.subset_query_sk, t2.subset_query_sk) as subset_query_sk,
					t1.select_column_calc_list, t2.explicit_join_tables
			from &m_tmp_lib..subset_exp_join_col_&m_abt_sk. t1 full join
				&m_tmp_lib..subset_exp_join_tbl_&m_abt_sk. t2
				on (t1.subset_query_sk = t2.subset_query_sk);
		quit;
		%dabt_err_chk(type=SQL);

		/* Obtain the information required for subset. This will be used to insert records in required_columns to 
			get the select column list. */
		proc sql noprint;			
			create table &m_tmp_lib..source_sub_list_&m_abt_sk as
			select qm.subset_query_sk as var_source_table_sk, 'Y' as has_retained_key_flg, lm.im_metadata_path,
					spl.level_sk as level_sk, lvlm.level_cd as level_cd, catx('','S',qm.subset_query_sk) as var_source_table_cd len=12,
					'SUB' as source_usage_cd len=3, '' as dummy_subset_table_nm len=100, sscm.source_column_nm as keep_column_name,
					coalesce(mscm.source_column_nm,sscm.source_column_nm) as alias_column_name, 'SUB_SEL' as column_info_type len=12, 'N' as source_out_flg,
					'DABT_OUTPUT_LIB_NM' as output_lib_nm len=40, 'DABT_OUTPUT_TABLE_NM' as output_ds_nm len=40,
					'&DABT_FILTER_FOR_SUBSET_CRITERIA' as filter_for_time_extract len=40, lm.library_type_cd,
					(strip(lm.library_reference)||"."||strip(stm.source_table_nm)) as base_source_table_nm label="" len=70,
					lkcd.key_column_sk as corrsp_master_tbl_column_sk, sej.explicit_join_tables as explicit_join_path_tables,
					sej.select_column_calc_list, '' as select_src_lvl_col_nm len=32767, '' as alias_src_lvl_col_nm len=32767,
					'' as level_key_column_nm len=200, 'SUB' as var_src_type_cd len=32
			from &m_tmp_lib..subset_info_distnct_&m_abt_sk as qm left join /* Performing left join to cater to the scenario, when there is only root node
												and no valid from column configured */
					&m_tmp_lib..subset_explicit_join_&m_abt_sk. sej
					on (qm.subset_query_sk = sej.subset_query_sk) left join
					&m_src_lib..subset_query_master as sqm
					on (qm.subset_query_sk = sqm.subset_query_sk) left join
					&m_src_lib..subset_from_path_x_level(where=(level_sk = &m_abt_level_sk.)) as spl
					on (sqm.subset_from_path_sk = spl.subset_from_path_sk) left join
					&m_src_lib..source_column_master as sscm
					on (spl.select_source_column_sk = sscm.source_column_sk) left join
					&m_src_lib..source_table_master as stm
					on (sscm.source_table_sk = stm.source_table_sk) left join
					&m_src_lib..library_master as lm
					on (stm.library_sk = lm.library_sk) left join
					&m_src_lib..level_master as lvlm
					on (spl.level_sk = lvlm.level_sk) left join
					&m_src_lib..level_key_column_dtl as lkcd
					on (spl.corrsp_level_key_column_dtl_sk = lkcd.level_key_column_dtl_sk) left join
					&m_src_lib..source_column_master as mscm
					on (lkcd.key_column_sk = mscm.source_column_sk); 
			%dabt_err_chk(type=SQL);
		quit;
		
		/* Insert the level column and any calculated column (used in where caluse) for the required subset */
		proc sql noprint;
		    insert into &m_tmp_lib..required_columns
		  		(var_source_table_sk, level_sk, keep_column_data_type, keep_column_name, column_info_type, outcome_flg)
		  	select col.source_table_sk,all_vars.level_sk, cdtm.column_data_type_cd, all_vars.alias_column_name, 
				'SUB_SEL' as column_info_type, "N" as outcome_flg
			from 
				&m_tmp_lib..source_sub_list_&m_abt_sk all_vars 
				inner join &m_src_lib..source_column_master col
				on (all_vars.corrsp_master_tbl_column_sk=col.source_column_sk)
				/* To identify the column_data_type_cd */
				inner join &m_src_lib..column_data_type_master cdtm
					on (col.column_data_type_sk = cdtm.column_data_type_sk);
		quit;
		  
		%dabt_err_chk(type=SQL);

		/* Get the information of all the subsets that have information map as path, so that it can be passed to the API to get the query. */
		data &m_tmp_lib..source_im_sub_list_&m_abt_sk;
			set &m_tmp_lib..source_sub_list_&m_abt_sk;
			where library_type_cd = 'SRC_IM';
			drop library_type_cd corrsp_master_tbl_column_sk;
		run;
		%dabt_err_chk(type=DATA);

		/* Count of IM in subset. */
		proc sql noprint;
			select count(*) into :sub_im_exst
			from &m_tmp_lib..source_im_sub_list_&m_abt_sk;
		quit;
		%dabt_err_chk(type=SQL);

		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..subset_info_distnct_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..source_sub_list_&m_abt_sk;
				%dabt_err_chk(type=SQL);
				drop table &m_sub_whr_tmp_tbl_nm.;
				%dabt_err_chk(type=SQL);
				drop table &m_sub_val_tmp_tbl_nm.;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..subset_table_exp_join_&m_abt_sk.;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..subset_whr_join_&m_abt_sk.;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..subset_table_exp_&m_abt_sk.;
				%dabt_err_chk(type=SQL);
				drop table &m_tmp_lib..subset_exp_join_tbl_&m_abt_sk.;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;
	%else %do;
		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_sub_whr_tmp_tbl_nm.;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;

	  /*Sorting the dataset created for keeping required columns and creating output in the staging area.*/

    proc sort data = &m_tmp_lib..required_columns out = &m_stg_lib..keep_required_columns nodupkey;
   		by var_source_table_sk level_sk outcome_flg column_info_type keep_column_name alias_column_name keep_column_data_type is_calc_flg var_src_type_cd;
    quit;

	%dabt_err_chk(type=DATA);

	/* Clear Temporary Tables */
	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..required_columns;
			%dabt_err_chk(type=SQL);
		quit;
	%end;

	/*Start: Details of information maps, used as source in the given ABT */

	%let m_im_exst = 0;
	proc sql noprint;
		select count(*) into :m_im_exst
		from &m_stg_lib..source_table_info as stm
		where kupcase(strip(var_source_type_cd)) = 'SRC_IM';
	quit;
	%dabt_err_chk(type=SQL);
	%let m_im_exst =&m_im_exst;

	/* If atleast one infomap exists then get the details of the information map */
	%if &m_im_exst gt 0 or &sub_im_exst gt 0 %then %do;
		/* Get the IM (source table) information required to be passed to API */
		proc sql noprint;
			create table &m_tmp_lib..source_im_list_&m_abt_sk as
			select stm.var_source_table_sk, stm.has_retained_key_flg, lm.im_metadata_path, 
					ld.level_sk, ld.level_cd, stm.var_source_table_cd as var_source_table_cd len=12 format=$12., 'SRC' as source_usage_cd len=3,
					stm.dummy_subset_table_nm, krm.keep_column_name, coalesce(krm.alias_column_name, krm.keep_column_name) as alias_column_name len=32, 
					krm.column_info_type, krm.outcome_flg as source_out_flg,
					'DABT_OUTPUT_LIB_NM' as output_lib_nm len=40, 'DABT_OUTPUT_TABLE_NM' as output_ds_nm len=40,
					'&DABT_FILTER_FOR_TIME_EXTRACT' as filter_for_time_extract len=40, base_source_table_nm,
					select_src_lvl_col_nm, alias_src_lvl_col_nm, ld.level_key_column_nm, 
					/* Remvoing libref, assumption that only one table is present */
					scan(stm.dummy_subset_table_nm,2,'.') as explicit_join_path_tables len=700, '' as select_column_calc_list len=32767,
/*					case when kupcase(strip(column_info_type)) IN ('BEH_USG','BEH_MSR','BEH_DIM','AVG_USG') then 'BEH'*/
/*						when kupcase(strip(column_info_type)) IN ('RNT_USG','RNT_SEL','RNT_DIM') then 'RNT'||strip(put(var_time_sk,12.))*/
/*						when kupcase(strip(column_info_type)) IN ('SPM_USG','SPM_SEL') then 'SPM' || strip(put(var_time_sk,12.))*/
/*						else '' end as var_src_type_cd len=32*/
					krm.var_src_type_cd
			from &m_stg_lib..source_table_info as stm
				inner join &m_stg_lib..keep_required_columns as krm
				on (stm.var_source_table_sk = krm.var_source_table_sk and stm.level_sk = krm.level_sk
					and kupcase(strip(column_info_type)) ne 'SUB_SEL')
				inner join &m_tmp_lib..lvl_mstr_sep_&m_abt_sk as ld
				on (stm.level_sk = ld.level_sk)
				inner join &m_src_lib..library_master as lm
				on (stm.source_library_sk = lm.library_sk)
			where kupcase(strip(var_source_type_cd)) = 'SRC_IM';
			%dabt_err_chk(type=SQL);
		quit;

		%if &sub_im_exst gt 0 %then %do;
			/* If subset had IM as its path, then append the IM info of subset to the source table for variable calculation. */
			proc append base=&m_tmp_lib..source_im_list_&m_abt_sk data=&m_tmp_lib..source_im_sub_list_&m_abt_sk force;
			run;
			%dabt_err_chk(type=DATA);

			/* Clear Temporary Tables */
			%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
				proc sql noprint;
					drop table &m_tmp_lib..source_im_sub_list_&m_abt_sk;
					%dabt_err_chk(type=SQL);
				quit;
			%end;
		%end;

		/* var_source_table_cd added in the by, as it is possible that for a subset and var_source_table the sk be same */
		proc sort data=&m_tmp_lib..source_im_list_&m_abt_sk;
			by var_source_table_sk level_sk var_source_table_cd var_src_type_cd source_out_flg ;
		run;
		%dabt_err_chk(type=DATA);

		/* Staging Area table containing IM source info. The select and alias for a given source, level and input/outcome
			is appended as comma separated value. */
		data &m_stg_lib..source_im_info;
			set &m_tmp_lib..source_im_list_&m_abt_sk;
			/* The code will execute only for the observations with column_info_type not AVG_USG.
				This is done to exclude the average column from select list */
			where kupcase(strip(column_info_type)) NOT IN ("AVG_USG", "SPM_USG");
			by var_source_table_sk level_sk var_source_table_cd var_src_type_cd source_out_flg;
			drop keep_column_name alias_column_name base_source_table_nm column_info_type im_metadata_path tmp_cd;
			drop select_src_lvl_col_nm alias_src_lvl_col_nm level_key_column_nm select_column_calc_list;
			length select_column_list where_column_list $32767.;
			length alias_column_list $32767.;
			/* Will get populated by java API */
			length im_query $32767.; 
			im_query='';
			length im_path $1000.;
			length im_query_file_nm $40. tmp_cd $3.;
			retain select_column_list alias_column_list;
			if first.source_out_flg then do;
				select_column_list = '';
				alias_column_list = '';
				%if &sub_cnt gt 0 %then %do;
					select_column_list = catx(',',select_column_list,level_key_column_nm);
					alias_column_list = catx(',',alias_column_list,level_key_column_nm);
				%end;
				%else %do;
					select_column_list = catx(',',select_column_list,select_src_lvl_col_nm);
					alias_column_list = catx(',',alias_column_list,alias_src_lvl_col_nm);
				%end;
			end;

			select_column_list = catx(',',select_column_list,keep_column_name);
			alias_column_list = catx(',',alias_column_list,alias_column_name);

			if last.source_out_flg then do;
				%if &sub_cnt gt 0 %then %do;
					where_column_list =  select_column_calc_list;
				%end;
				if source_out_flg = "N" then tmp_cd = "in";
				else tmp_cd = "out";
				if var_src_type_cd = "SUB" then do;
					im_query_file_nm = "&m_abt_type_first._&m_abt_sk._" ||strip(var_source_table_cd) || '_' ||strip(level_cd) || '_' || strip(tmp_cd) ||'_qry.sas';
				end;
				else do;
					im_query_file_nm = "&m_abt_type_first._&m_abt_sk._" || strip(var_src_type_cd) || '_' || strip(var_source_table_cd) || '_' ||strip(level_cd) || '_' || strip(tmp_cd) ||'_qry.sas';
				end;
				im_query_file_nm = klowcase(strip(im_query_file_nm));
				im_path = strip(im_metadata_path) || '/' || strip(scan(base_source_table_nm,2,'.'));
				output;
			end;
		run;
		%dabt_err_chk(type=DATA);

		/* Clear Temporary Tables */
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_tmp_lib..source_im_list_&m_abt_sk;
				%dabt_err_chk(type=SQL);
			quit;
		%end;
	%end;
	%else %do;
		data &m_stg_lib..source_im_info;
			attrib var_source_table_sk length=8.;
			attrib level_sk length=8.;
			attrib level_cd length=$1.;
			attrib var_source_table_cd length=$12.;
			attrib source_out_flg length=$1.;
			attrib select_column_list length=$32767.;
			attrib alias_column_list length=$32767.;
			attrib where_column_list length=$32767.;
			attrib output_lib_nm length=$40.;
			attrib output_ds_nm length=$40.;
			attrib im_query length=$32767.;
			attrib im_path length=$1000.;
			attrib has_retained_key_flg length=$1.;
			attrib dummy_subset_table_nm length=$100.;
			attrib im_query_file_nm length=$40.;
			attrib filter_for_time_extract length=$40.;
			attrib source_usage_cd length=$3.;
			attrib explicit_join_path_tables length=$700.;
			attrib var_src_type_cd length=$3.;
			stop;
		run;
		%dabt_err_chk(type=DATA);
	%end;
	/*End: Details of information maps, used as source in the given ABT */

	/* drop column dummy_subset_table_nm from &m_stg_lib..source_table_info, as it won't be used further. */
	proc sql noprint;
		alter table &m_stg_lib..source_table_info
		drop dummy_subset_table_nm, select_src_lvl_col_nm, alias_src_lvl_col_nm;
		%dabt_err_chk(type=SQL);
	quit;
	
	/* Clear Temporary Tables */
	%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..var_req_info_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_part_info_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..abt_info_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..all_vars_bld_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_bld_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_tmp_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_all_dup_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_all_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_msr_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..var_info_dim_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..lvl_mstr_&m_abt_sk;
			%dabt_err_chk(type=SQL);
			drop table &m_tmp_lib..lvl_mstr_sep_&m_abt_sk;
			%dabt_err_chk(type=SQL);
		quit;
	%end;

%mend dabt_create_stg_area;
