/************************************************************************************************
       Module:  dabt_scorecard_register_model

     Function:  This macro creates the SAS Scoring code for judgemental scoring.

         Date:  23 July 2013
          SAS:  9.4

************************************************************************************************/

%macro dabt_scorecard_register_model(m_project_sk=, m_model_sk=,m_folder_path=);

	%if %symexist(job_rc)=0 %then %do;
		%global job_rc;
	%end;
	%if %symexist(sqlrc)=0 %then %do;
		%global sqlrc;
	%end;

	%let m_scr_path = ;
	%let m_tmp_lib = ;
	%let m_log_path = ;
	%let m_scoring_code_path = ;
	%let m_src_lib = ;
	
	/* obtain the apdm library and log and scratch path and library */
	%dabt_assign_libs(tmp_lib=m_tmp_lib,src_lib=m_src_lib, log_path= m_log_path,
					 m_workspace_type=CHRSTC, m_called_for_abt_processing_flg=N,
					 m_abt_or_template_sk=&m_model_sk., scr_path=m_scr_path,
					 scoring_code_path=m_scoring_code_path);

	%let m_scr_path = &m_scr_path.;
	%let m_src_lib = &m_src_lib.;
	%let m_tmp_lib = &m_tmp_lib.;
	%let m_log_path = &m_log_path.;
	%let m_scoring_code_path = &m_scoring_code_path.;

	/* If the debug flag is not off then create log files */
	%if %kupcase("&CAB_APPL_DEBUG_FLG") ne "N" %then %do;
		proc printto log = "&m_log_path.dabt_scorecard_register_model_&m_model_sk..log" &log_clear;
		quit;
	%end;

	proc sql noprint;
		select abt_sk, abt_table_nm into :m_abt_sk, :abt_table_nm
			from &m_src_lib..Modeling_abt_master 
			where project_sk = &m_project_sk.;
	quit;
	%dabt_err_chk(type=SQL);

	%let m_abt_sk = &m_abt_sk.;
	%let abt_table_nm = &abt_table_nm.;

	%let m_abt_libref = &DABT_MODELING_ABT_LIBREF.;
	%let abt_exist_flag= Y;
	/*If the physical ABT does not exists then get all the variable column names for the ABT.
	  This will be used to create a dummy dataset.*/
	%if %sysfunc(exist(&m_abt_libref..&abt_table_nm.)) eq 0 %then %do;
		%let abt_exist_flag = N;
		proc sql noprint;
			create table &m_tmp_lib..abt_x_var as 
				select abt_sk,variable_sk 
					from &m_src_lib..Modeling_abt_x_variable 
					where abt_sk = &m_abt_sk.;
		quit;
		%dabt_err_chk(type=SQL);	

		proc sql noprint;
			create table &m_tmp_lib..var_col_name as 
				select a.variable_sk,b.variable_column_nm, b.column_data_type_sk, c.column_data_type_cd
					from &m_tmp_lib..abt_x_var a 
							inner join &m_src_lib..variable_master b 
								on a.variable_sk = b.variable_sk
							inner join &m_src_lib..Column_data_type_master c 
								on b.column_data_type_sk = c.column_data_type_sk;
		quit;
		%dabt_err_chk(type=SQL);

		proc sql noprint;
			select variable_column_nm,column_data_type_cd into :var_col_lst separated by '#', :col_data_type_lst separated by '#' 
				from &m_tmp_lib..var_col_name;
		quit;
		%dabt_err_chk(type=SQL);

		%let abt_col_tot = %eval(%sysfunc(countc(%quote(&var_col_lst),"#"))+1);
		%do abt_col_cnt = 1 %to &abt_col_tot;
			%let abt_col_tkn_&abt_col_cnt. = %scan(%quote(&var_col_lst),&abt_col_cnt,%str(#));
			%let abt_col_type_tkn_&abt_col_cnt. = %scan(%quote(&col_data_type_lst),&abt_col_cnt,%str(#));
		%end;

		/*Creating the dummy ABT dataset.*/
		data &m_abt_libref..&abt_table_nm.;
			%do i=1 %to &abt_col_tot.;
				%if &&abt_col_type_tkn_&i. = CHR %then %do;
					attrib &&abt_col_tkn_&i. length=$25;
				%end;
				%else %do;
					attrib &&abt_col_tkn_&i. length=8;
				%end;
			%end;
			stop;
		run;
		%dabt_err_chk(type=DATA);
	%end;

	/*Getting the target variable for the judgemental model. This will be passed as a 
	  parameter to the registering macro.*/
	proc sql noprint;
		select variable_sk into :m_tgt_var_sk 
			from &m_src_lib..Model_x_act_outcome_var
			where model_sk = &m_model_sk. and %kupcase(target_variable_flg)='Y';
			%dabt_err_chk(type=SQL);

		select variable_column_nm into :m_tgt_var_col_nm
			from &m_src_lib..variable_master
			where variable_sk = &m_tgt_var_sk.;
			%dabt_err_chk(type=SQL);
	quit;

	proc sql noprint;
		select model_id, tranwrd(tranwrd(tranwrd(model_short_nm,',',''),"'",""),'"',''), 
				tranwrd(tranwrd(tranwrd(model_desc,',',''),"'",""),'"','') 
				into :m_model_id,:m_mdl_shrt_nm, :m_mdl_desc
			from &m_src_lib..model_master
			where model_sk = &m_model_sk.;
	quit;
	%dabt_err_chk(type=SQL);

	%let m_model_id = &m_model_id.;
	%let m_mdl_shrt_nm = &m_mdl_shrt_nm.;
	%let m_mdl_desc = &m_mdl_desc.;

	%let m_scoring_code_file_nm=scoring_code_&m_model_id..sas;
	%let m_scorecode_file=&m_scr_path./&m_scoring_code_file_nm.;

	options lrecl= MAX; /*In csbinit, the lrecl is set to 1024 due to which the AA_MODEL_REGISTER does not works properly. Hence, setting the lrecl to MAX(32767)*/
	
	%AAMODEL;

	%AA_MODEL_REGISTER(modelname=&m_mdl_shrt_nm., modeldesc=%nrbquote(&m_mdl_desc.), register=Y, 
					mrpath=%nrbquote(&m_folder_path.),
					spk=Y, spkfolder=%nrbquote(&m_scr_path.), data=&m_abt_libref..&abt_table_nm., 
					target=&m_tgt_var_col_nm., LEVEL=BINARY, 
					scorecodefile=%nrbquote(&m_scorecode_file.),
					scorecodeformat=datastep, debug=Y,
					miningalgorithm=Scorecard);

	%let old_dir = &m_scr_path./&AA_MODELKEY.;
	%let new_dir = &m_scr_path./spk_jdgmntl_model_id_&m_model_id.;

	data _null_;
		rc=rename("&old_dir", "&new_dir", 'file');
		put rc;
	run;
	%dabt_err_chk(type=DATA);

	/*The macro variable aa_miningresultid contains the external model metadata ID.*/
	proc sql noprint;
		update &m_src_lib..Model_master
			set last_registered_model_id="&aa_miningresultid.",
				MODEL_MINING_ALGORITHM = "SCORECARD",
				last_registered_model_nm="&m_mdl_shrt_nm.",
				last_registered_dttm= datetime()
			where model_sk = &m_model_sk.;
	quit;
	%dabt_err_chk(type=SQL);

	/* If the debug flag is off then delete the temporary files */
	%if %kupcase("&DABT_RETAIN_APPL_SCRATCH_FLG") eq "N" and &abt_exist_flag = N %then %do;
		proc sql noprint;
			drop table &m_tmp_lib..abt_x_var;
			drop table &m_tmp_lib..var_col_name;
		quit;
		%dabt_err_chk(type=SQL);
	%end;

	%if %kupcase("&DABT_RETAIN_APPL_SCRATCH_FLG") eq "N" %then %do;
		filename tmp_file "&m_scorecode_file";
		data _null_;
			rc=fdelete('tmp_file');/* i18nOK:Line */
			put rc=;
		run;
		%dabt_err_chk(type=DATA);
	%end;

	proc printto log=log;
	quit;

%mend dabt_scorecard_register_model;
